﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Windows.Forms;
using WideStream.DownloadManager;
using System.IO;
using System.Xml;
using System.Text;
using WideStream.Configuration.Presentation;
using WideStream.Configuration.ComponentModel;
using WideStream.Startup.Resources;

namespace WideStream.Configuration
{
    public static class Config
    {
        #region Construction
        private const string FILE_NAME = "config2.xml";
        private static object syncRoot = new Object();
        #endregion


        #region Methods
		public static void Initialize()
		{
			Query<RecoveryOptions>();
		}


		public static bool Register<T>() where T : Options, new()
		{
            return Register(typeof(T));
		}


        public static bool Register(Type type)
        {
            if (RegisteredOptions == null)
                lock (syncRoot)
                    if (RegisteredOptions == null)
                        RegisteredOptions = new List<RegistrationInfo>();

            OptionsAttribute attr = (OptionsAttribute)Attribute.GetCustomAttribute(
                type, typeof(OptionsAttribute));

            if (attr != null)
            {
                RegisteredOptions.Add(new RegistrationInfo
                {
                    Category = attr.Category,
                    ShortName = attr.ShortName,
                    LogName = attr.LongName,
                    Instance = (Options)Activator.CreateInstance(type),
                    Hidden = attr.Hidden,
                    Id = type.GUID
                });

                Query(type.GUID);
            }

            return attr != null;
        }


        public static T Query<T>() where T : Options, new()
        {
			return (T)Query(typeof(T).GUID);
        }


		public static Options Query(Guid key)
		{
			if (Options == null)
				lock (syncRoot)
					if (Options == null)
						Options = new Dictionary<Guid,Options>();
					
			Options ret = null;
						
			if (!Options.TryGetValue(key, out ret))
			{
				RegistrationInfo info = RegisteredOptions.First(i => i.Id == key);
				ret = info.Instance;

				if (ApplicationRegistry.IsKeyExists() &&
					!LoadConfig(Path.Combine(ConfigPath, FILE_NAME), ret))
					ret = ProcessLoadError(ret);

				Options.Add(key, ret);
			}

			return ret;
		}


        private static bool LoadConfig(string fileName, Options instance)
        {
            FileInfo file = new FileInfo(fileName);
			
            if (file.Exists)
            {
                
                using (StreamReader sr = new StreamReader(file.FullName, 
					Encoding.GetEncoding(ConfigEncoding)))
                using (XmlReader reader = XmlReader.Create(sr))
                {
					OptionsSerializer ser = new OptionsSerializer();
					ser.Read(instance, reader);
                    return true;
                }
            }

			return false;
        }


		private static Options ProcessLoadError(Options instance)
        {
            switch (OnLoadErrorAction)
            {
                case OnLoadErrorAction.TryLoadFromDefault :
                    DirectoryInfo dir = GetDefaultDirectory();
                    
					if (LoadConfig(Path.Combine(dir.FullName, FILE_NAME), instance))
						return instance;
                    else
						return ShowChangePathDialog(instance);   
                case OnLoadErrorAction.ShowChangePathDialog :
                    return ShowChangePathDialog(instance);
                default :
                    return instance;
            }
        }


		private static Options ShowChangePathDialog(Options instance)
        {
            FilePathDialog d = new FilePathDialog();
            d.DialogTitle = StringTable.GetString("Config.ChangeFileFolder");
            d.Description = StringTable.GetString("Config.UnableToLoad");
            d.SelectedPath = ConfigPath;

            if (d.ShowDialog() == DialogResult.OK)
            {
                ConfigPath = d.SelectedPath;

				if (LoadConfig(Path.Combine(d.SelectedPath, FILE_NAME), instance))
					return instance;
            }

            ConfigPath = GetDefaultDirectory().FullName;
            Save();
			return instance;
        }


        public static void Save()
        {
            FileInfo file = new FileInfo(Path.Combine(ConfigPath, FILE_NAME));

            try
            {
                InternalSave(file);
            }
            catch (Exception)
            {
                ProcessSaveError();
            }
        }


        private static void InternalSave(FileInfo file)
        {
			using (StreamWriter sw = new StreamWriter(file.FullName, false, Encoding.GetEncoding(ConfigEncoding)))
			using (XmlWriter writer = XmlWriter.Create(sw))
			{
				writer.WriteStartElement("Configuration");
				writer.WriteAttributeString("Version", "2");
				OptionsSerializer ser = new OptionsSerializer();

				foreach (Options c in Options.Values)
					ser.Write(c, writer);

				writer.WriteEndElement();
			}
        }


        private static void ProcessSaveError()
        {
            switch (OnSaveErrorAction)
            {
                case OnSaveErrorAction.SaveDefaultLocation :
                    InternalSave(new FileInfo(Path.Combine(GetDefaultDirectory().FullName, FILE_NAME)));
                    break;
                case OnSaveErrorAction.ShowChangePathDialog:
                    FilePathDialog d = new FilePathDialog();
                    d.DialogTitle = StringTable.GetString("Config.ChangeFileFolder");
                    d.Description = StringTable.GetString("Config.UnableToSave");
                    d.SelectedPath = ConfigPath;

                    if (d.ShowDialog() == DialogResult.OK)
                    {
                        ConfigPath = d.SelectedPath;
                        InternalSave(new FileInfo(Path.Combine(ConfigPath, FILE_NAME)));
                    }
                    break;
            }
        }


		public static DirectoryInfo InitializeDefaultDirectory()
		{
			DirectoryInfo dir = GetDefaultDirectory();
			EnsureExists(dir);
			return dir;
		}


        internal static DirectoryInfo GetDefaultDirectory()
        {
            DirectoryInfo dir = new DirectoryInfo(Path.Combine(Environment.
                GetFolderPath(Environment.SpecialFolder.MyDocuments), "WideStream"));
            return dir;
        }


		internal static void EnsureExists(DirectoryInfo dir)
		{
			if (!dir.Exists)
			{
				dir.Create();
				dir.CreateSubdirectory("Downloads");
			}
		}


        private static string EncodeFolderPath(string path)
        {
            if (path.IndexOf(':') == -1)
                return path;

            StringBuilder sb = new StringBuilder(path.ToLower());
            List<Int32> list = new List<Int32>((Int32[])Enum.
                GetValues(typeof(Environment.SpecialFolder)));
            list.Sort((x,y) => y - x);
            
            foreach (int v in list)
            {
                string name = ((Environment.SpecialFolder)v).ToString();
                string namePath = Environment.GetFolderPath((Environment.SpecialFolder)v).ToLower();
                
                if (namePath.Length > 0)
                    sb.Replace(namePath, name);
            }

            return sb.ToString();
        }


        private static string DecodeFolderPath(string path)
        {
            if (path.IndexOf(':') != -1)
                return path;

            StringBuilder sb = new StringBuilder(path);

            foreach (int v in Enum.GetValues(typeof(Environment.SpecialFolder)))
            {
                string name = ((Environment.SpecialFolder)v).ToString();
                sb.Replace(name, Environment.GetFolderPath((Environment.SpecialFolder)v).ToLower());
            }

            return sb.ToString();
        }


		public static void Update()
		{
			OnConfigurationChanged();
		}
        #endregion


		#region Events
		public static event EventHandler ConfigurationChanged;
		private static void OnConfigurationChanged()
		{
			EventHandler h = ConfigurationChanged;

			if (h != null)
				h(null, EventArgs.Empty);
		}
		#endregion


		#region Properties
		internal static string ConfigPath
        {
            get
            {
                string ret = DecodeFolderPath(ApplicationRegistry.GetValue<String>(
                    "ConfigFileLocation", GetDefaultDirectory().FullName));
				EnsureExists(new DirectoryInfo(ret));
				return ret;

            }
            set { ApplicationRegistry.SetValue("ConfigFileLocation", EncodeFolderPath(value)); }
        }


        internal static OnSaveErrorAction OnSaveErrorAction
        {
            get
            {
                return (OnSaveErrorAction)ApplicationRegistry.GetValue<Int32>(
                    "OnSaveErrorAction", (Int32)OnSaveErrorAction.ShowChangePathDialog);
            }
            set { ApplicationRegistry.SetValue("OnSaveErrorAction", (Int32)value); }
        }
        

        internal static OnLoadErrorAction OnLoadErrorAction
        {
            get
            {
                return (OnLoadErrorAction)ApplicationRegistry.GetValue<Int32>(
                    "OnLoadErrorAction", (Int32)OnLoadErrorAction.ShowChangePathDialog);
            }
            set { ApplicationRegistry.SetValue("OnLoadErrorAction", (Int32)value); }
        }


        internal static int ConfigEncoding
        {
            get
            {
                return ApplicationRegistry.GetValue<Int32>(
                    "ConfigEncoding", (Int32)Encoding.UTF8.CodePage);
            }
            set { ApplicationRegistry.SetValue("ConfigEncoding", value); }
        }


		internal static Dictionary<Guid,Options> Options { get; private set; }


		public static List<RegistrationInfo> RegisteredOptions { get; private set; }
        #endregion
    }
}
