﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Reflection;
using System.Windows.Forms;
using WideStream.Configuration.ComponentModel;
using WideStream.Workspace.Forms;
using WideStream.Core.Network;
using WideStream.Startup.Resources;

namespace WideStream.Configuration
{
	public sealed class OptionPageRenderer : IOptionPageRenderer
	{
		#region Construction
		public OptionPageRenderer()
		{
      		Pages = new Dictionary<Options,Control>();
            SpecialProperties = new List<PropertyInfo>();
		}
		#endregion


		#region Methods
		public Control QueryOptions(Options optionClass)
		{
			return Pages.ContainsKey(optionClass) ? Pages[optionClass] : null;
		}


		public Control RenderOptions(Options optionClass, params string[] filter)
		{
			if (Pages.ContainsKey(optionClass))
				return Pages[optionClass];

			TableLayoutPanel panel = new TableLayoutPanel();
			panel.ColumnCount = 4;
			panel.ColumnStyles.Add(new ColumnStyle(SizeType.Percent, 50F));
			panel.ColumnStyles.Add(new ColumnStyle(SizeType.Absolute, 10F));
			panel.ColumnStyles.Add(new ColumnStyle(SizeType.Percent, 50F));
			panel.ColumnStyles.Add(new ColumnStyle(SizeType.AutoSize));
			panel.Dock = DockStyle.Fill;
			int editorCount = 0;
			int tabIndexCount = 0;

			if (!SkipHeader)
			{
				panel.RowCount = 1;
				OptionsAttribute attrClass = (OptionsAttribute)Attribute.GetCustomAttribute(
					optionClass.GetType(), typeof(OptionsAttribute));
				GroupSeparator header = new GroupSeparator();
				header.Text = StringTable.GetString(attrClass.LongName);
				header.Margin = new Padding(0, 0, 0, 0);
				header.Dock = DockStyle.Fill;
				panel.Controls.Add(header, 0, 0);
				panel.SetColumnSpan(header, 3);
				panel.RowStyles.Add(new RowStyle(SizeType.Absolute, 15));
			}

			foreach (PropertyInfo pi in optionClass.GetType().GetProperties())
			{
				if (filter.FirstOrDefault(s => s == pi.Name) != null)
					continue;

				EditorAttribute attr = Attribute.GetCustomAttribute(pi, 
					typeof(EditorAttribute)) as EditorAttribute;

				if (attr == null)
					continue;

				bool newRow = editorCount++ % 2 == 0 || attr.ColSpan;
				int rowCount = 3 - (attr.SkipLabel ? 1 : 0) -
					(String.IsNullOrEmpty(attr.Description) ? 1 : 0);

				if (newRow)
					panel.RowCount += rowCount;

                CheckBox cb = null;

                if (!attr.SkipLabel && AllowDefault)
                {
                    cb = new CheckBox();
                    cb.AutoSize = true;
                    cb.Text = StringTable.GetString(attr.DisplayName) + ":";
                    cb.TabIndex = tabIndexCount++;
                    cb.Margin = new Padding(3, 12, 3, 0);
                    panel.Controls.Add(cb, newRow ? 0 : 2, panel.RowCount - rowCount);	
                }
				else if (!attr.SkipLabel)
				{
					Label l = new Label();
					l.AutoSize = true;
					l.Text = StringTable.GetString(attr.DisplayName) + ":";
					l.TabIndex = tabIndexCount++;
					l.Margin = new Padding(0, 12, 3, 0);
					panel.Controls.Add(l, newRow ? 0 : 2, panel.RowCount - rowCount);				
				}
				
				object val = pi.GetValue(optionClass, null);
				IEditor e = CreateEditor(attr, pi);
				e.RelatedProperty = pi;
				e.SetStandardValues(val);
				e.Value = val;
				Control c = (Control)e;

                if (attr.RequiresRestart)
                    SpecialProperties.Add(pi);

                if (cb != null)
                {
                    c.Enabled = optionClass.IsEnabled(pi.Name);
                    cb.Checked = c.Enabled;
                    
                    cb.CheckedChanged += (o, ev) =>
                    {
                        c.Enabled = cb.Checked;
                        IEditor ed = (IEditor)c;

                        if (cb.Checked)
                            optionClass.AddEnabledValue(ed.RelatedProperty.Name);
                        else
                            optionClass.RemoveEnabledValue(ed.RelatedProperty.Name);
                    };
                }

				c.TabIndex = tabIndexCount++;

				if (attr.Height != 0)
				{
					c.Height = attr.Height;
					c.Dock = DockStyle.Top;
				}
				else
					c.Dock = DockStyle.Top;

				panel.Controls.Add(c, newRow ? 0 : 2, panel.RowCount - rowCount + 1);

				if (rowCount == 3)
				{
					Label lc = new Label();
					lc.AutoSize = true;
					lc.Text = StringTable.GetString(attr.Description);
					lc.Margin = new Padding(2, 0, 2, 6);
					panel.Controls.Add(lc, newRow ? 0 : 2, panel.RowCount - 1);

					if (attr.ColSpan)
						panel.SetColumnSpan(lc, 3);
				}

				if (attr.ColSpan)
				{
					panel.SetColumnSpan(c, 3);
					editorCount++;
				}
			}

			Pages.Add(optionClass, panel);
			return panel;
		}


		public bool ApplyChanges()
		{
            bool restart = false;

			foreach (Options o in Pages.Keys)
			{
				foreach (Control c in Pages[o].Controls)
				{
					if (c is IEditor)
					{
						IEditor ed = (IEditor)c;
						
                        if (SpecialProperties.FirstOrDefault(p => p == ed.RelatedProperty) != null &&
                            ed.IsDirty)
                            restart = true;

                        ed.RelatedProperty.SetValue(o, ed.Value, null);
					}
				}
			}

            return restart;
		}


		private IEditor CreateEditor(EditorAttribute attr, PropertyInfo pi)
		{
			switch (attr.Editor)
			{
				case PropertyEditor.Default:
					if (pi.PropertyType == typeof(Boolean) ||
						pi.PropertyType.IsSubclassOf(typeof(Enum)))
						return new DropDownList();
					else if (pi.PropertyType == typeof(Color))
						return new ColorPicker();
					else if (pi.PropertyType == typeof(String))
						return new TextEditor();
					else if (pi.PropertyType == typeof(Credentials))
						return new CredentialsEditor();
					break;
				case PropertyEditor.ComboBox:
					return new DropDownList();
				case PropertyEditor.FlagsEditor:
					return new FlagsEditor();
				case PropertyEditor.TextBox:
					return new TextEditor();
				case PropertyEditor.ColorPicker:
					return new ColorPicker();
				case PropertyEditor.FolderPicker:
					return new FolderPicker();
				case PropertyEditor.MultilineTextBox:
					{
						TextEditor editor = new TextEditor();
						editor.Multiline = true;
						return editor;
					}
				case PropertyEditor.CredentialsEditor:
					return new CredentialsEditor();
				case PropertyEditor.DirectoryTextBox:
					{
						TextEditor editor = new TextEditor();
						editor.AutoCompleteMode = AutoCompleteMode.Append;
						editor.AutoCompleteSource = AutoCompleteSource.FileSystemDirectories;
						return editor;
					}
                case PropertyEditor.FilePicker:
                    return new FilePicker();
                case PropertyEditor.LanguageComboBox:
                    return new LanguagesDropDownList();
				case PropertyEditor.Custom:
					return (IEditor)Activator.CreateInstance(attr.ControlType);
			}

			return null;
		}
		#endregion


		#region Properties
		public bool SkipHeader { get; set; }

        public bool AllowDefault { get; set; }

		internal Dictionary<Options,Control> Pages { get; private set; }

        internal List<PropertyInfo> SpecialProperties { get; private set; }
		#endregion
	}
}
