﻿using System;
using System.Linq;
using WideStream.Configuration.ComponentModel;
using System.Reflection;
using System.Xml;
using WideStream.Core.Network;
using System.IO;
using System.Text;
using System.Drawing;
using System.Collections;

namespace WideStream.Configuration
{
	internal sealed class OptionsSerializer
	{
		#region Construction
		public OptionsSerializer()
		{

		}
		#endregion


		#region Methods
		public void Write(object options, XmlWriter writer)
		{
			writer.WriteStartElement("Options");
			writer.WriteAttributeString("Id", options.GetType().FullName);
			WriteDataOnly(options, writer, null);
			writer.WriteEndElement();
		}


		public void WriteDataOnly(object options, XmlWriter writer, string[] enabled, params string[] filter)
		{
            if (options is Options)
            {
                string val = String.Join(",", ((Options)options).ObtainEnabledValues());
                writer.WriteAttributeString("EnabledFields", val);
            }

			Type type = options.GetType();
			
			foreach (PropertyInfo pi in type.GetProperties())
			{
				SaveRuleAttribute saveRule = SaveRuleAttribute.GetAttribute(pi);

				if (((filter.Length == 0 || filter.FirstOrDefault(f => f == pi.Name) == null)) &&
                    (enabled == null || enabled.FirstOrDefault(e => e == pi.Name) != null) &&
					(saveRule == null || !saveRule.Ignore))
				{
					object val = pi.GetValue(options, null);
					Write(writer, saveRule, pi, val);
				}
			}			
		}


		private void Write(XmlWriter writer, SaveRuleAttribute attr, PropertyInfo pi, object val)
		{
            string[] enabled = attr != null && attr.OnlyEnabled && val is Options ?
               ((Options)val).ObtainEnabledValues() : null;

			if (attr != null && attr.Rule == SaveRule.Collection)
			{
				writer.WriteStartElement(pi.Name);
                Write(writer, (IList)val, attr.OnlyEnabled);
				writer.WriteEndElement();
			}
			else if (attr != null && attr.Rule == SaveRule.NestedOptions)
			{
				writer.WriteStartElement(pi.Name);
                WriteDataOnly(val, writer, enabled);
				writer.WriteEndElement();
			}
			else if (val is Credentials)
				Write(writer, (Credentials)val);
			else
				Write(writer, pi.Name, val);
		}


		public void Read(object options, XmlReader reader)
		{
			Type type = options.GetType();
			string id = type.FullName;

			while (reader.ReadToFollowing("Options"))
				if (reader.GetAttribute("Id") == id)
				{
					ReadDataOnly(options, reader);
					return;
				}
		}


		public void ReadDataOnly(object options, XmlReader reader, params string[] filter)
		{
			if (options is Options)
            {
                ((Options)options).ReadEnabledValues(reader.GetAttribute("EnabledFields"));
            }

			foreach (PropertyInfo pi in options.GetType().GetProperties())
			{
				SaveRuleAttribute saveRule = SaveRuleAttribute.GetAttribute(pi);

				if (((filter.Length == 0 || filter.FirstOrDefault(f => f == pi.Name) == null)) &&
					(saveRule == null || !saveRule.Ignore))
					Read(reader, saveRule, options, pi);
			}		
		}


		private void Read(XmlReader reader, SaveRuleAttribute attr, object options, PropertyInfo pi)
		{           
			if (attr != null && attr.Rule == SaveRule.Collection)
			{                
				if (reader.ReadToFollowing(pi.Name))
				{
					XmlReader subReader = reader.ReadSubtree();
					IList list = (IList)pi.GetValue(options, null);
					Type type = attr.ChildType;
                    
					if (type == null)
					{
						foreach (object o in list)
							if (subReader.ReadToFollowing(o.GetType().Name))
								ReadDataOnly(o, subReader);
					}
                    else
                    {
                        while (subReader.ReadToFollowing(type.Name))
                        {
                            if (type == typeof(String))
                            {
                                string str = reader.GetAttribute("Value");
                                list.Add(str);
                            }
                            else
                            {
                                object o = Activator.CreateInstance(type);
                                ReadDataOnly(o, subReader);
                                list.Add(o);
                            }
                        }
                    }
				}
			}
			else if (attr != null && attr.Rule == SaveRule.NestedOptions)
			{
				if (reader.ReadToFollowing(pi.Name))
				{
					XmlReader subReader = reader.ReadSubtree();
					subReader.ReadToFollowing(pi.Name);
					ReadDataOnly(pi.GetValue(options, null), subReader);
				}
			}
			else
			{
				object value = Read(pi, reader);
				
				if (value != null)
					pi.SetValue(options, value, null);
			}
		}


		private void Write(XmlWriter writer, Credentials credentials)
		{
			if (credentials != null)
			{
				writer.WriteAttributeString("Domain", credentials.Domain);
				writer.WriteAttributeString("UserName", credentials.UserName);
				writer.WriteAttributeString("Password", UsePasswordEncryption ?
						Convert.ToBase64String(Encoding.UTF8.GetBytes(
						credentials.Password)) : credentials.Password);
			}
		}


		private void Write(XmlWriter writer, IList collection, bool onlyEnabled)
		{
			foreach (object o in collection)
			{
				Type type = o.GetType();
                writer.WriteStartElement(type.Name);

                if (type == typeof(String))
                    writer.WriteAttributeString("Value", (String)o);
                else
                {
                    string[] enabled = onlyEnabled && o is Options ?
                        ((Options)o).ObtainEnabledValues() : null;
                    WriteDataOnly(o, writer, enabled);
                }
                
                writer.WriteEndElement();
			}
		}


		private void Write(XmlWriter writer, string key, object value)
		{
			if (value != null)
			{
				string serValue;

				if (value is Color)
					serValue = ((Color)value).Name;
				else if (value is DateTime)
					serValue = ((DateTime)value).ToString("yyyy-MM-dd");
				else if (value is Enum)
					serValue = ((Int32)value).ToString();
				else
					serValue = value.ToString();

				writer.WriteAttributeString(key, serValue);
			}	
		}


		private object Read(PropertyInfo pi, XmlReader reader)
		{
			string value = reader.GetAttribute(pi.Name);
			bool isCred = pi.PropertyType == typeof(Credentials);

			if (String.IsNullOrEmpty(value) && !isCred ||
                (isCred && (String.IsNullOrEmpty(reader.GetAttribute("UserName")) ||
                String.IsNullOrEmpty(reader.GetAttribute("Password")))))
				return null;
			else
			{
				if (isCred)
				{
                    return new Credentials(reader.GetAttribute("Domain"),
						reader.GetAttribute("UserName"),
						 UsePasswordEncryption ? Base64Decrypt(reader.GetAttribute("Password")) :
						 reader.GetAttribute("Password"));
				}
				else if (pi.PropertyType.IsEnum)
					return Int32.Parse(value);
				else if (pi.PropertyType == typeof(Boolean))
					return String.Compare(Boolean.TrueString, value, true) == 0;
				else if (pi.PropertyType == typeof(Uri))
					return new Uri(value);
				else if (pi.PropertyType == typeof(FileInfo))
					return new FileInfo(value);
				else if (pi.PropertyType == typeof(Color))
					return Color.FromName(value);
				else
					return Convert.ChangeType(value, pi.PropertyType);
			}
		}


        private string Base64Decrypt(string value)
        {
            try
            {
                return Encoding.UTF8.GetString(Convert.FromBase64String(value));
            }
            catch (FormatException)
            {
                return value;
            }
        }
		#endregion


		#region Properties
        private bool? _usePasswordEncyption;
		public bool UsePasswordEncryption 
        {
            get
            {
                return _usePasswordEncyption != null ? _usePasswordEncyption.Value :
                    Config.Query<RecoveryOptions>().UsePasswordEncryption;
            }
            set { _usePasswordEncyption = value; }
        }
		#endregion
	}
}
