﻿using System;
using System.Linq;
using System.Collections.Generic;
using WideStream.Configuration;
using WideStream.DownloadManager.Configuration;
using WideStream.DownloadManager.Network.ObjectModel;
using System.Diagnostics;

namespace WideStream.DownloadManager
{
	public sealed class ConnectionLeaseManager
	{
		#region Construction
		internal ConnectionLeaseManager(IDownloadStatistics statistics)
		{
			Statistics = statistics;
		}
		#endregion


		#region Methods
		public void Execute(IEnumerable<IDownloadProcess> downloads)
		{
            if (downloads.Count(p => p.Busy) > 0)
                StartDownloads(downloads);
            else
            {
                int maxCon = (Int32)Config.Query<DownloadProcessOptions>().MaximumDownloadConnections;
                int acc = downloads.Sum(d => d.ActiveConnectionsCount);
                DownloadPriority maxPriority = (DownloadPriority)downloads.
                    Max(p => (Int32)p.Properties.GeneralOptions.Priority);
                ShareLease(downloads, CalculateLease(maxPriority, downloads), acc > maxCon);
            }
		}


		private void ShareLease(IEnumerable<IDownloadProcess> downloads, ConnectionLease lease, bool overLimit)
		{
			int ccc = 0;

            foreach (IDownloadProcess p in overLimit ? downloads.Reverse() : downloads)
                lease.Ticket -= p.ActiveConnectionsCount - 1;

			foreach (IDownloadProcess p in overLimit ? downloads.Where(p => !p.Busy).Reverse() : downloads.Where(p => !p.Busy))
			{
			    int acc = p.ActiveConnectionsCount;
								
				if (acc > lease.Limit || lease.Ticket < 0 || p.MaximumConnections < acc ||
					(acc > 1 && (Int32)p.Priority < (Int32)lease.Priority))
				{
					ccc++;

					if (p.DropConnection())
						lease.Ticket++;

					//Debug.WriteLine(p.Properties.GetDisplayName() + " :: DROP => ACC=" + acc + ";LEASE=" + lease.Limit + ";TICKET=" + lease.Ticket);
				}
				else if (acc == 0 || (p.MaximumConnections > p.ActiveConnectionsCount &&
					p.Priority == lease.Priority && p.ActiveConnectionsCount < lease.Limit &&
					lease.Ticket > 0))
				{
					ccc++;

					if (p.AddConnection())
						lease.Ticket--;

					//Debug.WriteLine(p.Properties.GetDisplayName() + " :: ADD => ACC=" + acc + ";LEASE=" + lease.Limit + ";TICKET=" + lease.Ticket);
				}
			}

			//if (ccc > 0)
			//	Debug.WriteLine("================");
		}


        private void StartDownloads(IEnumerable<IDownloadProcess> downloads)
        {
            foreach (IDownloadProcess p in downloads)
                if (p.ActiveConnectionsCount == 0 && !p.Busy)
                    p.AddConnection();
        }


		private ConnectionLease CalculateLease(DownloadPriority priority, IEnumerable<IDownloadProcess> downloads)
		{
			int maxCon = (Int32)Config.Query<DownloadProcessOptions>().MaximumDownloadConnections;
			double singleCon = downloads.Count(p => p.MaximumConnections == 1 || (Int32)p.Priority < (Int32)priority);
			double multiCon = downloads.Count(p => p.MaximumConnections > 1 && p.Priority == priority);
            double limitPre = multiCon == 0 ? maxCon : (maxCon - singleCon) / multiCon;
			int limit = (Int32)limitPre + (limitPre % 1 > 0 ? 1 : 0);
			return new ConnectionLease(limit, maxCon - Statistics.CountActiveDownloads(), priority);
		}
		#endregion


		#region Properties
		public IDownloadStatistics Statistics { get; private set; }
		#endregion
	}
}
