﻿using System;
using System.Diagnostics;
using System.Linq;
using System.IO;
using System.Net;
using System.Threading;
using System.Windows.Forms;
using WideStream.Core.Runtime;
using WideStream.Configuration;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.Core.ComponentModel;
using WideStream.DownloadManager.Configuration;
using WideStream.Startup;
using WideStream.Startup.ExceptionHandling;

namespace WideStream.DownloadManager.Network
{
	public abstract class DownloadConnection : DownloadTask, IDownloadConnection
	{
		#region Construction
		private const int SECTION_SIZE = 10000;
        private volatile bool terminating;
		
		protected DownloadConnection(IDownloadProcess process) : base(process)
		{
			
		}


		~DownloadConnection()
		{
			Close();
		}
		#endregion


		#region Methods
		public virtual void Dispose()
		{
			Close();
			GC.SuppressFinalize(this);
		}


		protected void Close()
		{
			if (ResponseStream != null)
			{
                try
                {
                    ResponseStream.Close();
                }
                catch (WebException ex) 
                {
                    ExceptionManager.Log(ex, Process != null ?
                        Process.Properties.Url : null);
                }

				ResponseStream = null;
			}

			if (Response != null)
			{
				Response.Dispose();
				Response = null;
			}
		}


		public virtual void Initialize(Segment segment)
		{
			Segment = segment;
		}

       
		public virtual void Start(IDownloadResponse response)
		{
            terminating = !(Active = true);
			ResponseStream = ObtainResponseStream(response);
			AnalyzeResponse(response);
			
			if (ResponseStream == null)
				throw new NotSupportedException();

			AsParallel(() => ProcessSection());
		}


		protected abstract Stream ObtainResponseStream(IDownloadResponse response);


		protected virtual void AnalyzeResponse(IDownloadResponse response)
		{

		}


		public virtual void Stop()
		{
            terminating = true;
		}


        protected virtual void InternalStop()
        {
            if (ResponseStream != null)
                lock (SyncRoot)
                {
                    if (ResponseStream != null)
                    {
                        ResponseStream.Close();
                        ResponseStream = null;
                    }
                }

            Active = false;            
        }


		protected virtual void ProcessSection()
		{
			if (!Active)
				return;

			TimeStamp = DateTime.Now;			
			long size = Segment.Length - Segment.Read > SECTION_SIZE ?
				SECTION_SIZE : Segment.Length - Segment.Read;
            
            if (size < 0)
            {
                FinilizeConnection();
                return;
            }

			byte[] buffer = new byte[size];
			
			if (AnalyzeSection(buffer))
			{
				int read = 0;
	
				if (!SafeExecute(() => read = ReadStream(buffer, 0, buffer.Length), false))
					return;
				
				AsParallel(() => ProcessReadResult(read, buffer));
			}
		}


		protected virtual bool AnalyzeSection(byte[] buffer)
		{
			return true;
		}


		protected virtual void ProcessReadResult(int read, byte[] buffer)
		{
			IConnectionInformation inf = Process.ConnectionInformation;

			if (inf == null)
				return;

            if (read == 0)// && inf.ChunkedMode)
                FinilizeConnection();
            else
            {
                int waitTime = 0;

                if (AnalyzeReadResult(read))
                {
                    if (read > 0)
                    {
                        if (!SafeExecute(() => State.File.Write(buffer, read, Segment.Offset + Segment.Read), true))
                            return;

                        TimeSpan waitSpan = DateTime.Now - TimeStamp;
                        waitTime = CorrectPriority(waitSpan) + CorrectTrafficUsage(waitSpan, read);
                        Segment.Read += read;
                        Process.Indicator.ReportStep(State.BytesRead, read);
                    }

                    if (Segment.Read == Segment.Length)
                        FinilizeConnection();
                    else
                    {
                        Thread.Sleep(waitTime);
                        ProcessSection();
                    }
                }
            }
		}


		protected virtual bool AnalyzeReadResult(int read)
		{
			return true;
		}


		protected virtual bool SafeExecute(MethodInvoker callBack, bool critical)
		{
			try
			{
                if (terminating)
                {
                    InternalStop();
                    return false;
                }

                callBack();
                return true;
			}
			catch (Exception ex)
			{
                if (Active)
				{
					if (ResponseStream != null)
						lock (SyncRoot)
						{
							if (ResponseStream != null)
							{
								ResponseStream.Close();
								ResponseStream = null;
							}
						}

					Fail(ex, critical);
				}
			
				return false;
			}
		}
		

		protected virtual int CorrectTrafficUsage(TimeSpan waitSpan, int read)
		{
			TrafficUsage traffic = Config.Query<QueueManagerOptions>().TrafficUsage;
			
			if (traffic != TrafficUsage.Full && TimeStamp != default(DateTime))
			{
				int waitPeriod = read / (((Int32)traffic / QueueManager.Instance.CountActiveConnections()) / 1000) -
					(Int32)waitSpan.TotalMilliseconds;
				return waitPeriod > 0 ? waitPeriod : 0;
			}

			return 0;
		}


		protected virtual int CorrectPriority(TimeSpan waitSpan)
		{
			int waitTime = (Int32)waitSpan.TotalMilliseconds > 0 ? (Int32)waitSpan.TotalMilliseconds : 1;
			int priorityWaitIndex = Properties.GeneralOptions.Priority == DownloadPriority.Low ? 5 :
				Properties.GeneralOptions.Priority == DownloadPriority.Normal ? 2 : 0;
			int priorityWaitTime = QueueManager.Instance.ActiveHighPriority * priorityWaitIndex * 2 * waitTime +
				QueueManager.Instance.ActiveNormalPriority * priorityWaitIndex * waitTime *
					(Properties.GeneralOptions.Priority == DownloadPriority.Normal ? 0 : 1);
			return priorityWaitTime > 0 ? priorityWaitTime : 0;
		}


		protected virtual void FinilizeConnection()
		{
			Dispose();
            LogSuccess("SegmentDownloaded",
				Segment.Id, Segment.Offset, Segment.Offset + Segment.Length);

            Active = !(Complete = true);
			OnFinished();
		}


		protected virtual void Fail(Exception ex, bool critical)
		{
			Dispose();
			Active = false;
			OnErrorOccured(new ConnectionErrorEventArgs(ex, critical));
		}


        private int ReadStream(byte[] buffer, int offset, int length)
        {
            Stream s = ResponseStream;
            return s != null && s.CanRead ? s.Read(buffer, offset, length) : 0;
        }
		#endregion


		#region Events
		public event EventHandler Finished;
		protected virtual void OnFinished()
		{
			EventHandler h = Finished;

			if (h != null)
				h(this, EventArgs.Empty);
		}


		public event EventHandler<ConnectionErrorEventArgs> ErrorOccured;
		protected virtual void OnErrorOccured(ConnectionErrorEventArgs e)
		{
			EventHandler<ConnectionErrorEventArgs> h = ErrorOccured;

			if (h != null)
				h(this, e);
		}
		#endregion


		#region Properties
		protected virtual int SectionSize
		{
			get { return SECTION_SIZE; }
		}
                		
		public DateTime TimeStamp { get; private set; }

		protected IDownloadResponse Response { get; set; } 

		protected Stream ResponseStream { get; set; }

		public bool Complete { get; private set; }

		public bool Active { get; private set; }

		public Segment Segment { get; private set; }
		#endregion
	}
}
