﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Security;
using System.Runtime.CompilerServices;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using WideStream.Configuration;
using WideStream.Core.Network;
using WideStream.DownloadManager.Configuration;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.DownloadManager.Presentation;
using WideStream.Logging;
using WideStream.Startup;
using WideStream.Workspace.ComponentModel;
using System.Windows.Forms;
using System.Threading;
using System.Diagnostics;
using System.Security;
using System.Web;
using WideStream.Startup.Resources;
using WideStream.Startup.ExceptionHandling;

namespace WideStream.DownloadManager.Network
{
	public sealed class DownloadProcess : LogTask, IDownloadProcess
	{
		#region Construction
		public DownloadProcess() 
		{
			Connections = new List<IDownloadConnection>();
		}
		#endregion

		
		#region Methods
		public void Initialize(Guid key, DownloadProperties properties, ILog log, IProgressIndicator indicator)
		{
			Key = key;
			Properties = properties;
			Log = log;
			Indicator = indicator;
			Status = properties.State.PreviousStatus;

			if (properties.State.Segments.Count > 0 && properties.State.BytesRead > 0)
			{
				Indicator.Initialize(properties.State.ContentLength, Properties.State.StartTime, Properties);
				Indicator.ReportStep(properties.State.BytesRead, 0);
			}

			Indicator.ChangeAttribute(ProgressAttribute.Status, Properties.State.PreviousStatus);
			Indicator.ChangeAttribute(ProgressAttribute.Priority, Properties.GeneralOptions.Priority);
			Properties.GeneralOptions.PriorityChanged += (o, e) => Indicator.ChangeAttribute(
				ProgressAttribute.Priority, Properties.GeneralOptions.Priority);
			Properties.UrlChanged += (o, e) => Indicator.ChangeAttribute(ProgressAttribute.DownloadDisplay, Properties);
            Properties.GeneralOptions.MaximumConnectionsChanged += (o, e) => MaximumConnections =
                (Int32)Properties.GeneralOptions.MaximumDownloadConnections;
		}


		public void Wait()
		{
			Indicator.AutoRedraw = true;
			ChangeStatus(DownloadStatus.Default);
		}


		public void Start()
		{
			Indicator.AutoRedraw = true;
            Properties.RedirectUrl = null;
            Certificate = null;
			ServicePointManager.ServerCertificateValidationCallback +=
				new RemoteCertificateValidationCallback(CertificateValidation);		
			Busy = true;
			LogClosed = false;
			MaximumConnections = (Int32)Properties.GeneralOptions.MaximumDownloadConnections;
			ChangeStatus(DownloadStatus.Started);
			AsParallel(() => InitialStart(false));
		}

		public void Pause()
		{
			Indicator.AutoRedraw = true;
			Status = DownloadStatus.Paused;
			ChangeStatus(DownloadStatus.Paused);			
		}


		public void Abort()
		{
			Indicator.AutoRedraw = true;
			Status = DownloadStatus.Aborted;
			ChangeStatus(DownloadStatus.Aborted);			
		}

		
		private void InitialStart(bool restart)
		{
            if (Status != DownloadStatus.Started)
                return;

			Properties.State.StartTime = DateTime.Now;

            if (!restart || Adapter == null)
            {
                Adapter = DownloadAdapterManager.ObtainAdapter(Properties.Adapter);
                Adapter.Initialize(this);
                DownloadAdapterAddin addin = DownloadAdapterManager.ObtainAdapterData(Properties.Adapter);
                LogInfo("UsingAdapter", addin.Name);
            }
                
            IDownloadCreator creator = DownloadProtocolManager.IdentifyProtocol(Properties.RedirectUrl ?? Properties.Url).
                Provider.ObtainCreator();
			LogInfo("MakeConnection", Properties.Url.Host);
			IServerConnection serverConnection = creator.Create<IServerConnection>(this);
			ConnectionInformation = serverConnection.EstablishConnection();

			if (ConnectionInformation == null)
			{
				ChangeStatus(DownloadStatus.Failed);
				return;
			}
			else if (ConnectionInformation.NeedRestart && !restart)
			{
                LogImportant("RestartDownload", Properties.Url.Host);
                InitialStart(true);
				return;
			}

			Indicator.Initialize(ConnectionInformation.ContentLength, Properties.State.StartTime, Properties);
            LogVerbose("PrepareSegments");

			if (!CreateFile())
			{
				ChangeStatus(DownloadStatus.Failed);
				return;
			}

			InitializeSegments();
            LogVerbose("OpenConnections");
			Busy = false;
		}



		private void ChangeStatus(DownloadStatus value)
		{
			Status = value;
			Properties.State.PreviousStatus = value;
			Indicator.ChangeAttribute(ProgressAttribute.Status, value);
			Indicator.ChangeAttribute(ProgressAttribute.DownloadDisplay, Properties);
            QueueManagerOptions conf = Config.Query<QueueManagerOptions>();
			
			switch (value)
			{
				case DownloadStatus.Aborted:
                    LogError("Aborted");
					Clean(true, conf.RemoveFile == RemoveFileCondition.FailedAborted || 
                        conf.RemoveFile == RemoveFileCondition.AbortedOnly);
                    Indicator.ReportStep(0, 0);
					break;
				case DownloadStatus.Failed:
                    LogError("Failed");
					bool nonRecovery = ConnectionInformation != null && 
						!ConnectionInformation.SupportRanges && (conf.RemoveFile == RemoveFileCondition.FailedOnly ||
                        conf.RemoveFile == RemoveFileCondition.FailedAborted);
					Clean(nonRecovery, nonRecovery);
					break;
				case DownloadStatus.Finished:
					DateTime dt = new DateTime((DateTime.Now - Properties.State.StartTime).Ticks);
                    LogSuccess("Finished");
                    LogInfo("DownloadTook", dt);
					Clean(true, false);
					break;
				case DownloadStatus.Started:
                    Indicator.ReportStep(0, 0);
                    LogImportant("Started");
					break;
				case DownloadStatus.Paused:
                    LogImportant("Paused");
					Clean(false, false);					
					break;
			}

			OnStatusChanged();
		}


		private void Clean(bool nonRecovery, bool dropFile) 
		{
			ServicePointManager.ServerCertificateValidationCallback -=
				new RemoteCertificateValidationCallback(CertificateValidation);		
			
			LogClosed = true; 
			StopConnections();
			Properties.State.Close(nonRecovery, dropFile);
			Connections.Clear();
			ConnectionInformation = null;
		}


		private void StopConnections()
		{
			foreach (IDownloadConnection c in Connections.Where(cn => cn.Active))
				c.Stop();
		}

		
		private bool StopConnection(IDownloadConnection c)
		{
			if (c.Active)
			{
				c.Stop();
                LogImportant("ConnectionTerminated", c.Segment.Id,
					c.Segment.Offset, c.Segment.Offset + c.Segment.Length);				
			}

			return c.Active;
		}


		[MethodImpl(MethodImplOptions.Synchronized)]
		private void ProcessFinishedConnection()
		{
			if (Status == DownloadStatus.Finished)
				return;

			if (Connections.Count(c => !c.Complete) == 0)
				ChangeStatus(DownloadStatus.Finished);
		}


		[MethodImpl(MethodImplOptions.Synchronized)]
		private void ProcessConnectionError(object sender, ConnectionErrorEventArgs e)
		{
			try
			{
				Busy = true;
				ExceptionManager.Log(e.Exception, Properties.Url);
				int segmentId = ((IDownloadConnection)sender).Segment.Id;

				if (e.Critical)
				{
                    LogError("SegmentCriticalError", segmentId, e.Exception.Message);
					ChangeStatus(DownloadStatus.Failed);
				}
				else
				{
					Properties.State.ErrorCount++;

					if (Properties.State.ErrorCount >= (Int32)Properties.GeneralOptions.ErrorLimit)
					{
                        LogError("SegmentError", segmentId, e.Exception.Message);
                        LogImportant("ErrorCount", Properties.State.ErrorCount);
                        LogError("ErrorLimit",
							Properties.GeneralOptions.ErrorLimit);
						ChangeStatus(DownloadStatus.Failed);
					}
					else
					{
						if (!IsErrorCritical(e.Exception))
						{
                            LogInfo("SegmentRetry", segmentId, (Int32)Properties.GeneralOptions.RetryTime / 1000);
							Sleep((Int32)Properties.GeneralOptions.RetryTime);
						}
					}
				}
			}
			finally
			{
				Busy = false;
			}
		}


		private bool IsErrorCritical(Exception ex)
		{
			Properties.State.ErrorCount++;
            LogError("Error", ex.Message, Properties.State.ErrorCount);

			if (Properties.State.ErrorCount >= (Int32)Properties.GeneralOptions.ErrorLimit)
			{
                LogError("ErrorLimitExceed",
					Properties.GeneralOptions.ErrorLimit);
				ChangeStatus(DownloadStatus.Failed);
				return true;
			}
			else
				return false;
		}


		private void OpenConnection(IDownloadConnection c)
		{
			try
			{
				if (c == null || c.Active || Status != DownloadStatus.Started)
					return;

                IDownloadCreator creator = DownloadProtocolManager.IdentifyProtocol(Properties.RedirectUrl ?? Properties.Url).
                        Provider.ObtainCreator();
				IDownloadProtocol protocol = creator.Create<IDownloadProtocol>(this);
				IDownloadConfiguration configuration = creator.Create<IDownloadConfiguration>(this);
                IDownloadRequest request = protocol.CreateRequest(Properties.RedirectUrl ?? Properties.Url);
				configuration.Authorize(request);
				configuration.Configure(request);
                configuration.Negotiate(request);
                Adapter.Negotiate(request);
                configuration.PrepareForDownload(request, c.Segment);
                IDownloadResponse response = protocol.ObtainResponse(request);				
				c.Start(response);
                LogInfo("ConnectionOpened",
					c.Segment.Id, c.Segment.Offset + c.Segment.Read, c.Segment.Offset + c.Segment.Length);
			}
			catch (WebException ex)
			{
				ExceptionManager.Log(ex, Properties.Url);
				MaximumConnections--;
				
				if (!IsErrorCritical(ex))
				{
					if (ActiveConnectionsCount == 0)
					{
                        LogWarning("SegmentRetry", c.Segment.Id, 
							(Int32)Properties.GeneralOptions.RetryTime / 1000);
						Sleep((Int32)Properties.GeneralOptions.RetryTime);
					}
					else
                        LogWarning("ConnectionTerminated",
							c.Segment.Id, c.Segment.Offset, c.Segment.Offset + c.Segment.Length);
				}
			}
		}


		private bool CreateFile()
		{
			if (Properties.State.ResumeMode)
			{
				Properties.State.FileLocation.Refresh();
				Properties.State.File = DownloadFile.Open(Properties.State.FileLocation);

				if (Properties.State.File == null)
				{
                    LogError("UnableResume",
						Properties.State.FileLocation.ToString());
					Properties.State.File = DownloadFile.Create(Properties.State.FileLocation,
						ConnectionInformation.ChunkedMode ? null : (long?)ConnectionInformation.ContentLength);
					Properties.State.Segments.Clear();
				}
				else
                    LogInfo("FileOpened", Properties.State.FileLocation.FullName);					
			}
			else
			{
				FileInfo fileName;

				if (!String.IsNullOrEmpty(Properties.NewName))
				{
					fileName = new FileInfo(Path.Combine(Properties.GeneralOptions.DownloadPath, Properties.NewName));
                    LogVerbose("FileNameByUser", fileName.Name);
				}
				else if (!String.IsNullOrEmpty(ConnectionInformation.FileName))
				{
					fileName = new FileInfo(Path.Combine(Properties.GeneralOptions.DownloadPath, 
                        HttpUtility.UrlDecode(ConnectionInformation.FileName)));
                    LogVerbose("FileNameByServer", fileName.Name);
				}
				else if (Properties.Url.LocalPath.Length > 1)
				{
					string shortName = Properties.Url.LocalPath.Trim('/');
					int index = shortName.LastIndexOf("/");
					
					if (index != -1)
						shortName = shortName.Substring(index + 1);

					fileName = new FileInfo(Path.Combine(Properties.GeneralOptions.DownloadPath, shortName));

					if (fileName.Extension.Length == 0)
						fileName = new FileInfo(String.Concat(fileName.FullName, DetermineExtension()));

                    LogVerbose("FileNameDefault", fileName.Name);
				}
				else
				{
					string ext = DetermineExtension();
					fileName = AutogenerateFileName(Path.Combine(Properties.GeneralOptions.DownloadPath, "unknown"), ext);
                    LogWarning("FileNameGenerated", fileName.Name);
				}

				long? len = ConnectionInformation.ChunkedMode ? null :
					ConnectionInformation.ChunkedMode ? null : (long?)ConnectionInformation.ContentLength;
				Properties.State.File = DownloadFile.Create(fileName, len);

				if (Properties.State.File == null)
				{
                    LogError("FileExists", fileName.FullName);
                    QueueManagerOptions opt = Config.Query<QueueManagerOptions>();
                    
                    switch (opt.ExistingFileAction)
                    {
                        case ExistingFileAction.StopDownload :
                            return FileFail();
                        case ExistingFileAction.Autogenerate :
                            fileName = AutogenerateFileName(fileName.FullName.Replace(fileName.Extension, 
                                String.Empty), fileName.Extension);
                            break;
                        case ExistingFileAction.Overwrite :
                            if (!RemoveFile(fileName))
                                return FileFail();
                            break;
                        case ExistingFileAction.ShowSelectDialog :
                            DownloadFileDialog dfd = null;
                            
                            try
                            {
                                dfd = new DownloadFileDialog();
                                dfd.DownloadName = Properties.GetDisplayName();
                                dfd.InvalidFile = fileName;
                                WaitHandle handle = dfd.SafeShow();
                                handle.WaitOne();

                                if (dfd.DialogResult == DialogResult.Cancel)
                                    return FileFail();
                                else
                                {
                                    if (dfd.Overwrite)
                                    {
                                        if (!RemoveFile(fileName))
                                            return FileFail();
                                    }
                                    else
                                        fileName = dfd.SafeGetSelected();
                                }
                            }
                            finally
                            {
                                dfd.SafeClose();
                            }
                            break;
                    }

                    fileName.Refresh();
                    Properties.State.File = DownloadFile.Create(fileName, len);

                    if (Properties.State.File == null)
                        return FileFail();
				}

                Properties.State.FileLocation = fileName;
                LogVerbose("FileCreated", fileName.FullName);
			}

			return true;
		}


        private bool RemoveFile(FileInfo fileName)
        {
            try
            {
                fileName.Delete();
                return true;
            }
            catch (Exception ex)
            {
                if (ex is IOException || ex is SecurityException ||
                    ex is UnauthorizedAccessException)
                    return false;
                else
                    throw ex;
            }
        }


        private FileInfo AutogenerateFileName(string baseFileName, string ext)
        {
            FileInfo fileName;
            int count = 0;

            do
            {
                fileName = new FileInfo(String.Concat(baseFileName, "(", ++count, ")", ext));
            }
            while (fileName.Exists);

            return fileName;
        }


        private bool FileFail()
        {
            LogImportant("FileNameChange");
            return false;
        }


		private string DetermineExtension()
		{
			return !String.IsNullOrEmpty(ConnectionInformation.ContentType) &&
				ConnectionInformation.ContentType.StartsWith("text/html", StringComparison.OrdinalIgnoreCase) ?
					".htm" : String.Empty;
		}


		private void InitializeSegments()
		{
			if (Properties.State.Segments.Count == 0)
			{
				Properties.State.Segments.AddRange(SplitSegments());
				LogVerbose("SegmentsCreatedReport", Properties.State.Segments.Count);
			}

			foreach (Segment s in Properties.State.Segments)
			{
				if (s.Read != s.Length)
				{
					IDownloadCreator creator = DownloadProtocolManager.IdentifyProtocol(
                        Properties.RedirectUrl ?? Properties.Url).Provider.ObtainCreator();
					IDownloadConnection downConnection = creator.Create<IDownloadConnection>(this);
					downConnection.Initialize(s);
					downConnection.Finished += (o, e) => ProcessFinishedConnection();
					downConnection.ErrorOccured += ProcessConnectionError;
					Connections.Add(downConnection);
				}
			}

            LogVerbose("FragmentationFinished");
		}


		private IEnumerable<Segment> SplitSegments()
		{
			DataSize cl = ConnectionInformation.ContentLength;
			int minSlicedSize = (Int32)Config.Query<QueueManagerOptions>().AccelerateFileSize;
			int maxSlices = 10;

			if (cl.MegaBytes < 40)
				maxSlices = MaximumConnections;
			else if (cl.MegaBytes > 100)
				maxSlices = (Int32)(cl.MegaBytes / 10);

			long sliceSize = cl > minSlicedSize && !ConnectionInformation.ChunkedMode &&
				ConnectionInformation.SupportRanges && MaximumConnections > 1 ? cl / maxSlices : (Int64)cl;
			long rest = 0;

			for (int i = 0; i < (cl != sliceSize ? maxSlices : 1); i++)
			{
				yield return new Segment(i + 1, rest, i != (maxSlices - 1) ? sliceSize : cl - rest);
				rest += sliceSize;				
			}

			if (cl == sliceSize)
				MaximumConnections = 1;
		}


		private void ChangeActiveConnections(bool open)
		{
			try
			{
				var conn = open ? Connections.FirstOrDefault(c => !c.Active && !c.Complete) :
					Connections.Reverse<IDownloadConnection>().FirstOrDefault(c => c.Active);

				if (conn != null && !open)
					StopConnection(conn);
                else if (conn != null)
                    OpenConnection(conn);
                
				ReportOpenedConnections();
				Indicator.ChangeAttribute<Int32>(ProgressAttribute.Connections, ActiveConnectionsCount);
			}
			finally
			{
				Busy = false;
			}
		}


		public bool AddConnection()
		{
            if (Busy)
                return false;

			bool cond = Connections.Count > 0 && ActiveConnectionsCount < MaximumConnections;

			if (Busy = cond)
				AsParallel(() => ChangeActiveConnections(true));

			return cond;
		}


		public bool DropConnection()
		{
            if (Busy)
                return false;

			bool cond = Connections.Count > 0 && ActiveConnectionsCount > 1;

			if (Busy = cond)
				AsParallel(() => ChangeActiveConnections(false));
		
			return cond;
		}


		private void ReportOpenedConnections()
		{
			if (LastOpenedConnectionReport != ActiveConnectionsCount)
			{
                LogInfo("ActiveConnectionsReport", ActiveConnectionsCount);
				LastOpenedConnectionReport = ActiveConnectionsCount;
			}
		}


		private bool CertificateValidation(object sender, X509Certificate cert, X509Chain chain, SslPolicyErrors error)
		{
            bool ret = true;

			if (Certificate == null)
			{
				Certificate = cert;

				if (cert != null)
				{
                    LogInfo("CertificateInformation");
					StringBuilder sb = new StringBuilder();
                    sb.AppendFormat(StringTable.GetString("Downloads.CertificateType"), cert.GetFormat() + Environment.NewLine);
                    sb.AppendFormat(StringTable.GetString("Downloads.CertificateSubject"), cert.Subject + Environment.NewLine);
                    sb.AppendFormat(StringTable.GetString("Downloads.CertificateIssuer"), cert.Issuer + Environment.NewLine);
                    sb.AppendFormat(StringTable.GetString("Downloads.CertificateSerial"), cert.GetSerialNumberString() + Environment.NewLine);
                    sb.AppendFormat(StringTable.GetString("Downloads.CertificateEffective"), cert.GetEffectiveDateString() + Environment.NewLine);
                    sb.AppendFormat(StringTable.GetString("Downloads.CertificateExpires"), cert.GetExpirationDateString() + Environment.NewLine);
                    LogEntry(new LogEntry(LogEntryType.Information, sb.ToString(), "CertificateLogCategory"));
				}

                if (error != SslPolicyErrors.None)
                {
                    LogError("CertificateError", error.ToString());
                    QueueManagerOptions opt = Config.Query<QueueManagerOptions>();
                    Func<CertificateErrors,Boolean> func = (e) =>
                        (((Int32)opt.FailOnCertificateErrors & (Int32)e) != (Int32)e) || (((Int32)error & (Int32)e) != (Int32)e);

                    ret = func(CertificateErrors.RemoteCertificateChainErrors) &&
                        func(CertificateErrors.RemoteCertificateNameMismatch) &&
                        func(CertificateErrors.RemoteCertificateNotAvailable);                        				    
                }
                else
                    LogSuccess("CertificateValidationError");
			}

			return ret;
		}
		#endregion


		#region Events
		public event EventHandler StatusChanged;
		private void OnStatusChanged()
		{
			EventHandler h = StatusChanged;

			if (h != null)
				h(this, EventArgs.Empty);
		}
		#endregion


		#region Properties
		private int _maximumConnections;
		public int MaximumConnections
		{
			get 
			{
				int ccc = 0;

				if (Connections != null && Connections.Count != 0 && _maximumConnections != 0 && 
					(ccc = Connections.Count - CompleteConnectionsCount) < _maximumConnections)
					_maximumConnections = ccc;

                if (Adapter != null)
                {
                    int? amc = Adapter.MaximumConnections;

                    if (amc != null && amc.Value < _maximumConnections)
                        _maximumConnections = amc.Value;
                }

				return _maximumConnections; 
			}
			private set
			{
				if (value <= 0)
					_maximumConnections = 1;
				else
					_maximumConnections = value;
			}
		}

		
		public int ActiveConnectionsCount
		{
			get 
			{ 
				int ret = 0;

				for (int i = 0; i < Connections.Count; i++)
					if (Connections[i].Active)
						ret++;

				return ret;
			}
		}


		internal int CompleteConnectionsCount
		{
			get
			{
				int ret = 0;

				for (int i = 0; i < Connections.Count; i++)
					if (Connections[i].Complete)
						ret++;

				return ret;
			}
		}


		internal int LastOpenedConnectionReport { get; private set; }

		public DownloadPriority Priority 
		{
			get { return Properties.GeneralOptions.Priority; }
		}
		
		internal X509Certificate Certificate { get; private set; }
		
		public bool Busy { get; private set; }

		public IConnectionInformation ConnectionInformation { get; private set; }

        public IDownloadAdapter Adapter { get; private set; }

		public Guid Key { get; private set; }

		public IProgressIndicator Indicator { get; private set; }

		public DownloadProperties Properties { get; set; }

		public DownloadStatus Status { get; private set; }

		public List<IDownloadConnection> Connections { get; private set; }

        public int Position { get; set; }
		#endregion
	}
}
