﻿using System;
using System.Net;
using System.Net.Security;
using System.Text;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.Startup;
using System.IO;
using System.Reflection;
using WideStream.Logging;
using WideStream.Startup.Resources;
using WideStream.Startup.ExceptionHandling;

namespace WideStream.DownloadManager.Network.Protocols.Ftp
{
	public sealed class FtpServerConnection : DownloadTask, IServerConnection
	{
		#region Construction
		private const string FEAT = "FEAT";
		
		public FtpServerConnection(IDownloadProcess process) : base(process)
		{
		
		}
		#endregion
		

		#region Methods
		public IConnectionInformation EstablishConnection()
		{
			try
			{
				return InternalEstablishConnection();
			}
			catch (Exception ex)
			{
				ExceptionManager.Log(ex);
				LogError(ex.Message);
				return null;
			}
		}


        private IConnectionInformation InternalEstablishConnection()
		{
			FtpFeatures feat = RequestFeat();

            FtpWebRequest ftpRequest = CreateRequest();
			ftpRequest.Method = WebRequestMethods.Ftp.DownloadFile;
            LogInfo("FileInformationObtain");
			PrintRequestDetails(ftpRequest);

			using (FtpWebResponse response = (FtpWebResponse)ftpRequest.GetResponse())
			{
                LogSuccess("ResponseAcquired", ftpRequest.RequestUri.Host);
				feat.UpdateContentData(response);

				StringBuilder sb = new StringBuilder();
				sb.AppendFormat("Content-Length: {0}\r\n", feat.ContentLength > 0 ? feat.ContentLength.ToString() : 
                    StringTable.GetString("Downloads.Unknown"));
				sb.AppendFormat("Date: {0}\r\n", feat.FileDate != default(DateTime) ? feat.FileDate.ToString() :
                    StringTable.GetString("Downloads.Unknown"));
                LogEntry(new LogEntry(LogEntryType.Information, sb.ToString(), "ResponseDetails"));

				if (feat.ContentLength <= 0 && (feat.ContentLength = ParseFileSize(response.StatusDescription)) <= 0 &&
					feat.SupportFileSizeRequest)
				{
                    LogWarning("FileSizeMissing");
					feat.ContentLength = ExecuteSizeRequest();

					if (feat.ContentLength > 0)
                        LogSuccess("FileSizeObtained", feat.ContentLength);
				}
				else if (feat.ContentLength <= 0)
				{
                    LogWarning("FileSizeUnableObtain");
					feat.ChunkedMode = true;
					feat.ContentLength = Int64.MaxValue;
				}
			}
			
			return feat;
		}


		private void PrintRequestDetails(FtpWebRequest request)
		{
			StringBuilder sb = new StringBuilder();
			sb.AppendFormat("FTP {0}\r\n", request.Method);
			sb.AppendFormat("Host: {0}\r\n", request.RequestUri.Host);
            LogEntry(new LogEntry(LogEntryType.Information, sb.ToString(), "RequestDetails"));
		}


		private FtpFeatures RequestFeat()
		{
			FtpWebRequest req = CreateRequest();
			req.KeepAlive = false;
			SetMethod("FEAT", req);
			PrintRequestDetails(req);

			try
			{
				using (FtpWebResponse response = (FtpWebResponse)req.GetResponse())
				{
                    LogSuccess("ResponseAcquired", req.RequestUri.Host);
					LogInfo(response.BannerMessage);
					LogInfo(response.WelcomeMessage);
					FtpFeatures feat = FtpFeatures.ParseFeatures(response.StatusDescription);
                    LogEntry(new LogEntry(LogEntryType.Information, feat.ResponseString, "FtpFeatures"));
					return feat;
				}
			}
			catch (Exception ex)
			{
				LogError(ex.Message);
				return FtpFeatures.CreateDefault();
			}
		}


		private void SetMethod(string method, FtpWebRequest request)
		{
			Type t = Type.GetType(String.Format("System.Net.FtpMethodInfo, {0}", typeof(FtpWebRequest).Assembly.FullName));
			ConstructorInfo ci = t.GetConstructors(BindingFlags.Instance | BindingFlags.NonPublic)[0];
			object obj = ci.Invoke(new object[] { method, 13, 0, null });
			FieldInfo fi = typeof(FtpWebRequest).GetField("m_MethodInfo", BindingFlags.Instance | BindingFlags.NonPublic);
			fi.SetValue(request, obj);
		}


		private FtpWebRequest CreateRequest()
		{
            FtpWebRequest req = (FtpWebRequest)WebRequest.Create(Properties.RedirectUrl ?? Properties.Url);
			FtpDownloadRequest ftpRequest = new FtpDownloadRequest(req);
			FtpDownloadConfiguration config = new FtpDownloadConfiguration(Process);
			config.FullConfigure(ftpRequest);
			return req;
		}


		private long ExecuteSizeRequest()
		{
			long size = -1;

			try
			{
				FtpWebRequest req = CreateRequest();
				req.Method = WebRequestMethods.Ftp.GetFileSize;
				req.KeepAlive = false;

				using (FtpWebResponse resp = (FtpWebResponse)req.GetResponse())
					size = (Int64)resp.ContentLength;
			}
			catch (Exception ex)
			{
                ExceptionManager.Log(ex, Properties.RedirectUrl ?? Properties.Url);
                LogError("UnexpectedError", ex.Message);
			}

			return size;
		}


		private long ParseFileSize(string statusDescription)
		{
			string[] arr = statusDescription.Split('(');

			if (arr.Length < 2)
				return -1;
			else
			{
				try
				{
					string sizeStr = arr[1];
					StringBuilder sb = new StringBuilder();

					foreach (char c in sizeStr)
					{
						if (Char.IsNumber(c))
							sb.Append(c);
						else
							return Int64.Parse(sb.ToString());
					}
				}
				catch (Exception)
				{
					return -1;
				}
			}

			return -1;
		}
		#endregion
	}
}
