﻿using System;
using System.Linq;
using System.Net;
using System.Net.Security;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.Logging;
using WideStream.Startup;
using WideStream.Core.Network;
using System.IO;
using WideStream.Startup.ExceptionHandling;

namespace WideStream.DownloadManager.Network.Protocols.Http
{
	public sealed class HttpServerConnection : DownloadTask, IServerConnection
	{
		#region Construction
		private const string CONTENT_TYPE = "application/x-www-form-urlencoded";

        public HttpServerConnection(IDownloadProcess process) : base(process)
		{
				
		}
		#endregion
		

		#region Methods
		public IConnectionInformation EstablishConnection()
		{
			try
			{
				return InternalEstablishConnection(Process.Adapter.DiscoveryMethod);
			}
			catch (Exception ex)
			{
				ExceptionManager.Log(ex);
				LogError(ex.Message);
				return null;
			}
		}


		private IConnectionInformation InternalEstablishConnection(string method)
		{
            HttpDownloadConfiguration conf = new HttpDownloadConfiguration(Process);
            HttpDownloadRequest req = (HttpDownloadRequest)new HttpDownloadProtocol(Process).CreateRequest(
                Process.Adapter.DiscoveryUrl);
            conf.ConfigureAll(req);
            Process.Adapter.Negotiate(req);
            HttpWebRequest httpRequest = req.Request;
			httpRequest.Method = method;
            GenerateForm(req);

			StringBuilder sb = new StringBuilder();
			sb.AppendFormat("HTTP {0} {1} {2}\r\n", ((HttpOptions)Properties.ProtocolOptions.FirstOrDefault(o => o is HttpOptions)).
				ProtocolVersion == HttpProtocolVersion.Http11 ? "1.1" : "1.0", httpRequest.Method, req.RequestUrl.LocalPath);

            if (req.Form != null)
                sb.AppendFormat("Form: {0}\r\n", HidePassword(req.Form.ToString()));

            sb.AppendFormat("Connection: {0}\r\n", httpRequest.KeepAlive ? "Keep-Alive" : "Close");

            if (httpRequest.Credentials != null && !String.IsNullOrEmpty(((Credentials)httpRequest.Credentials).UserName))
                sb.AppendFormat("Authorization: {0}\r\n", HidePassword(((Credentials)httpRequest.Credentials).ToString()));

            httpRequest.KeepAlive = false; 
           
			foreach (string k in httpRequest.Headers.AllKeys)
			{
				string value = HidePassword(httpRequest.Headers[k] ?? String.Empty);
                sb.AppendFormat("{0}: {1}\r\n", k, value);
			}

			sb.AppendFormat("Host: {0}\r\n", httpRequest.RequestUri.Host);
			LogEntry(new LogEntry(LogEntryType.Information, sb.ToString(), "RequestDetails"));
            
			using (HttpWebResponse response = GetResponse(httpRequest))
			{
                LogSuccess("ResponseAcquired", httpRequest.RequestUri.Host);
				HttpHeaders ret = HttpHeaders.Read(response.Headers);
                              
                LogEntry(new LogEntry(LogEntryType.Information, ret.ResponseString, "ResponseHeaders"));
                Uri uri = null;

                if (!String.IsNullOrEmpty(ret.Location) && Uri.TryCreate(ret.Location, UriKind.Absolute, out uri) &&
                    String.Compare(uri.Scheme, Properties.Url.Scheme, true) != 0)
                {
                    LogImportant("ProtocolRedirection", uri.Scheme.ToUpper());
                    Properties.RedirectUrl = uri;
                }
                else
                {
                    var res = Process.Adapter.Analyze(ret, new HttpDownloadResponse(response));

                    if (!res.CompletedSynchroniously)
                        res.WaitHandle.WaitOne();

                    ret.NeedRestart = Process.Adapter.RequiresRestart;
                    Properties.RedirectUrl = Process.Adapter.DownloadUrl ?? Properties.RedirectUrl;
                }

				return ret;
			}
		}


        private void GenerateForm(HttpDownloadRequest request)
        {
            HttpWebRequest webRequest = request.Request;

            if (String.Compare(Process.Adapter.DiscoveryMethod, WebRequestMethods.Http.Post, true) == 0 &&
                request.Form != null)
            {
                webRequest.ContentType = CONTENT_TYPE;
                string form = request.Form.ToString();
                byte[] buffer = Encoding.UTF8.GetBytes(form);
                webRequest.ContentLength = buffer.Length;
                webRequest.ServicePoint.Expect100Continue = false;

                using (Stream rs = webRequest.GetRequestStream())
                    rs.Write(buffer, 0, buffer.Length);
            }
        }


        private string HidePassword(string value)
        {
            if (value != null && Properties.Credentials != null &&
                !String.IsNullOrEmpty(Properties.Credentials.Password))
                return value.Replace(Properties.Credentials.Password, Base64Password);
            else
                return value;
        }


        private HttpWebResponse GetResponse(HttpWebRequest request)
        {
            try
            {
                return (HttpWebResponse)request.GetResponse();
            }
            catch (WebException ex)
            {
                if (ex.Response != null)
                    return (HttpWebResponse)ex.Response;
                else
                    throw ex;
            }
        }
        #endregion


        #region Properties
        private string _base64Password;
        public string Base64Password
        {
            get
            {
                if (_base64Password == null)
                    lock (SyncRoot)
                        if (_base64Password == null)
                        {
                            _base64Password = Convert.ToBase64String(Encoding.UTF8.GetBytes(
                                Properties.Credentials.Password));
                        }

                return _base64Password;
            }
        }
		#endregion
	}
}
