﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using WideStream.Configuration;
using WideStream.Core.Network;
using WideStream.DownloadManager.Configuration;
using WideStream.DownloadManager.Network;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.Configuration.ComponentModel;
using System.Drawing;
using WideStream.HostManager.Configuration;
using WideStream.Startup.Resources;

namespace WideStream.DownloadManager.Presentation
{
    public partial class ConfigureDownloadDialog : Form
    {
        #region Construction
        private bool disableAutoUpdate;
        private UrlAnalyzer analyzer;
		
        public ConfigureDownloadDialog() : this(true)
        {
			   
        }


        public ConfigureDownloadDialog(bool processClipboard)
        {
            InitializeComponent();
            _downloads = new List<DownloadProperties>();
            analyzer = new UrlAnalyzer(true);

            if (processClipboard)
            {
                string value = Clipboard.GetText();
                AnalysisResult res = analyzer.AnalyzeInput(new CurrentData { InputValue = value });

                if (res.Downloads != null)
                    foreach (DownloadProperties p in res.Downloads)
                        AddDownload(p);
            }

			SingleLinkMode = _downloads.Count < 2;
			analyzer = new UrlAnalyzer(false);
            PropertyControls = new Dictionary<DownloadProperties,GeneralPropertiesControl>();
			Renderer = new OptionPageRenderer();
			Renderer.SkipHeader = true;
			BuildLinkList();
		}
        #endregion


        #region Methods
        public new DialogResult ShowDialog(IWin32Window owner)
        {
            if (owner == null || !((Control)owner).Visible)
                StartPosition = FormStartPosition.CenterScreen;

            return base.ShowDialog(owner);
        }

     
        private void ConfigureDownloadDialog_Load(object sender, EventArgs e)
        {
            if (Downloads.Count == 0)
            {
				CurrentDownload = new DownloadProperties();
				analyzer.InitializeDownload(CurrentDownload, null, null);
				Downloads.Add(CurrentDownload);
            }

			if (CurrentDownload == null)
				CurrentDownload = Downloads[0];

			AddPropertyPages(CurrentDownload);
            
			if (linksList.Items.Count > 0)
				linksList.SelectedIndex = 0;

			BuildLinkList();
            Text = StringTable.GetString(Text);
            acceptButton.Text = StringTable.GetString(acceptButton.Text);
            cancelButton.Text = StringTable.GetString(cancelButton.Text);
            applyAllCheckBox.Text = StringTable.GetString(applyAllCheckBox.Text);
            GeneralTabPage.Text = StringTable.GetString(GeneralTabPage.Text);
            linksLabel.Text = StringTable.GetString(linksLabel.Text);
		}


		private void AddPropertyPages(DownloadProperties properties)
		{
			GeneralPropertiesControl control;

			if (!PropertyControls.TryGetValue(properties, out control))
			{
				control = new GeneralPropertiesControl();
				control.LimitedMode = LimitedMode;
				control.BuildLinkList += (o, e) => BuildLinkList();
				control.UpdateLinkList += (o, e) => UpdateLinkList(e.Properties);
				control.AddEmptyDownload += (o, e) => AddEmptyDownload();
				control.UpdateLink += (o, e) => UpdateLink();
				control.HostRecognized += (o, e) => ProcessHost(e.Host);
				control.Link = properties.Url != null ? properties.Url.ToString() : String.Empty;
				control.DownloadPath = properties.GeneralOptions.DownloadPath;
				control.NewName = properties.NewName;
                control.Adapter = properties.Adapter;
				control.Credentials = properties.Credentials;
				control.LinksCaption = SingleLinkMode ? StringTable.GetString("AddDownload.Link") : 
                    StringTable.GetString("AddDownload.Details");
				PropertyControls.Add(properties, control);
			}

			if (propPanel.Controls.Count != 0)
				propPanel.Controls.Clear();
				
			propPanel.Controls.Add(control);			
			AddPage(properties.GeneralOptions, StringTable.GetString("AddDownload.Settings"));

			foreach (Options opt in properties.ProtocolOptions)
			{
				var attr = OptionsAttribute.GetAttribute(opt.GetType());
				AddPage(opt, StringTable.GetString(attr.ShortName));
			}
		}


		private void ProcessHost(Host host)
		{
			CompareControls(CurrentDownload.GeneralOptions, host.GeneralOptions);

			foreach (Options opt in CurrentDownload.ProtocolOptions)
				CompareControls(opt, host.ProtocolOptions.First(o => o.GetType() == opt.GetType()));
		}


		private void CompareControls(Options options, Options hostOptions)
		{
			Control cont = Renderer.QueryOptions(options);

			if (cont != null)
				foreach (Control c in cont.Controls)
					if (c is IEditor && ((IEditor)c).RelatedProperty.Name != "DownloadPath")
					{
						IEditor e = (IEditor)c;
						bool equal = e.Value != null && e.Value.Equals(e.RelatedProperty.GetValue(options, null));

						if (!equal)
							e.Value = e.RelatedProperty.GetValue(hostOptions, null);
					}
		}


		private void UpdateLink()
		{
			disableAutoUpdate = true;
			
			if (linksList.SelectedIndex != -1 && PropertyControls.ContainsKey(CurrentDownload))
				linksList.Items[linksList.SelectedIndex] = CompileDownloadDisplay(
					PropertyControls[CurrentDownload].Link);
			
			disableAutoUpdate = false;
		}


		private void AddPage(Options options, string name)
		{
			TabPage tab = null;

			foreach (TabPage p in tabControl.TabPages)
				if (p.Text == name)
				{
					tab = p;
					tab.Controls.Clear();
				}

			if (tab == null)
			{
				tab = new TabPage();
				tab.Padding = new Padding(10, 3, 10, 10);
				tab.BackColor = Color.Transparent;
				tab.Text = name;
				tab.UseVisualStyleBackColor = true;
				tabControl.TabPages.Add(tab);
			}

			Control control = Renderer.RenderOptions(options, "DownloadPath");
			tab.Controls.Add(control);			
		}


        private void BuildLinkList()
        {
			int selectedIndex = linksList.SelectedIndex;
			disableAutoUpdate = true;
            
			linksList.Items.Clear();

			foreach (DownloadProperties d in Downloads)
			{
				linksList.Items.Add(d.Url != null ?
					CompileDownloadDisplay(d.Url.ToString()) : StringTable.GetString("AddDownload.Empty"));
			}

			linksList.SelectedIndex = selectedIndex;
			disableAutoUpdate = false;
        }


		private string CompileDownloadDisplay(string link)
		{
			Uri uri;

			if (Uri.TryCreate(link, UriKind.Absolute, out uri))
			{
				string display = uri.ToString();
				int index = -1;

				if (uri.LocalPath.Length > 1)
				{
					string lc = String.Empty;
					display = (index = uri.LocalPath.LastIndexOf("/")) != -1 ?
						(lc = uri.LocalPath.Substring(index + 1)).Length > 1 ? lc : 
							uri.LocalPath : uri.LocalPath;
				}

				return display;
			}
			else
				return link;
		}


		private void AddEmptyDownload()
		{
			DownloadProperties d = new DownloadProperties();
			PropertyControls[CurrentDownload].UrlAnalyzer.InitializeDownload(d, null, null);
			Downloads.Add(d);
			AddPropertyPages(d);
			BuildLinkList();
			linksList.SelectedIndex = linksList.Items.Count - 1;
			PropertyControls[CurrentDownload].FocusLinkTextBox();
		}


		private void UpdateLinkList(List<DownloadProperties> downs)
		{
			Downloads.Remove(CurrentDownload);
			Uri uri = downs[0].Url;
			Credentials oldCred = downs[0].Credentials;
			downs[0].Url = uri;
			Downloads.AddRange(downs);
			BuildLinkList();
			PropertyControls[CurrentDownload].DisableAutoUpdate = false;
			linksList.SelectedIndex = linksList.Items.Count - 1;

		}


        private void acceptButton_Click(object sender, EventArgs e)
        {
            if (applyAllCheckBox.Checked)
                ApplyCurrentSettingsForAll();
        }


		private void ConfigureDownloadDialog_FormClosing(object sender, FormClosingEventArgs e)
		{
			if (DialogResult == DialogResult.OK && CurrentDownload != null && Validate())
				ApplyChanges();
			else if (DialogResult == DialogResult.OK)
				e.Cancel = true;
		}


		private void ApplyChanges()
		{
			foreach (DownloadProperties p in PropertyControls.Keys)
			{
				GeneralPropertiesControl c = PropertyControls[p];
				p.Url = new Uri(c.Link);
				p.NewName = c.NewName;
				p.Credentials = c.Credentials;
                p.Adapter = c.Adapter;
				p.GeneralOptions.DownloadPath = c.DownloadPath;
			}

            foreach (DownloadProperties p in Downloads)
            {
                if (String.IsNullOrEmpty(p.Adapter))
                    p.Adapter = DownloadAdapterManager.ObtainAdaptersInformation(p.Url).First().Key;
            }

			Renderer.ApplyChanges();
		}


        private void ReadDownload(DownloadProperties properties)
        {
            AddPropertyPages(properties);
        }


        public void AddDownload(Uri uri)
        {
            DownloadProperties prop = new DownloadProperties();
            analyzer.InitializeDownload(prop, null, uri);
            _downloads.Add(prop); 
        }


        public void AddDownload(DownloadProperties properties)
        {
            _downloads.Add(properties);
        }


        public IEnumerable<DownloadProperties> EnumerateDownloads()
        {
            foreach (DownloadProperties p in _downloads)
                yield return p;
        }
        

        private void linksList_SelectedIndexChanged(object sender, EventArgs e)
        {
			if (disableAutoUpdate)
				return;

			if (linksList.SelectedIndex == -1 || (CurrentDownload != null && !Validate()))
			{
				for (int i = 0; i < linksList.Items.Count; i++)
					if ((String)linksList.Items[i] == PropertyControls[CurrentDownload].Link)
					{
						disableAutoUpdate = true;
						linksList.SelectedIndex = i;
						disableAutoUpdate = false;
					}

				return;
			}

            CurrentDownload = (DownloadProperties)_downloads[linksList.SelectedIndex];
            ReadDownload(CurrentDownload);
        }



		private new bool Validate()
		{
			bool ret = PropertyControls[CurrentDownload].IsValid();

			if (!ret)
				tabControl.SelectedIndex = 0;

			return ret;
		}



        private void cancelButton_Click(object sender, EventArgs e)
        {
            Close();
        }


        private void ApplyCurrentSettingsForAll()
        {
			if (!PropertyControls[CurrentDownload].IsValid())
				return;

			foreach (DownloadProperties d in Downloads)
			{
				d.Credentials = new Credentials(PropertyControls[CurrentDownload].Credentials.FullUserName,
                    PropertyControls[CurrentDownload].Credentials.Password);
				d.GeneralOptions.DownloadPath = PropertyControls[CurrentDownload].DownloadPath;
				GroupSetProperties(Renderer.QueryOptions(CurrentDownload.GeneralOptions), d.GeneralOptions);

				foreach (Options o in CurrentDownload.ProtocolOptions)
				{
					Options targetOpt = d.ProtocolOptions.First(opt => opt.GetType() == o.GetType());
					GroupSetProperties(Renderer.QueryOptions(o), targetOpt);
				}
			}			
        }


		private void GroupSetProperties(Control source, Options target)
		{
			foreach (Control c in source.Controls)
				if (c is IEditor)
				{
					IEditor e = (IEditor)c;
					e.RelatedProperty.SetValue(target, e.Value, null);
				}

		}
        #endregion


        #region Properties
		public bool LimitedMode 
		{
			get { return !linksList.Enabled; }
			set { linksList.Enabled = !value; }
		}


        public int DownloadCount
        {
            get { return _downloads.Count; }
        }


        private bool _singleLinkMode;
        public bool SingleLinkMode
        {
            get { return _singleLinkMode; }
            set
            {
                if (value && !_singleLinkMode)
                {
                    foreach (Control c in GeneralTabPage.Controls)
                        c.Top -= linksList.Height + linksLabel.Height + 10;

                    cancelButton.Top = acceptButton.Top -= linksList.Height + linksLabel.Height + 10;
                    tabControl.Height -= linksList.Height + linksLabel.Height + 10;
                    Height -= linksList.Height + linksLabel.Height + 10;
			    }
                else if (!value && _singleLinkMode)
                {
                    foreach (Control c in GeneralTabPage.Controls)
                        c.Top += linksList.Height + linksLabel.Height + 10;

                    cancelButton.Top = acceptButton.Top += linksList.Height + linksLabel.Height + 10;
                    tabControl.Height += linksList.Height + linksLabel.Height + 10;
                    Height += linksList.Height + linksLabel.Height + 10;
                }

                applyAllCheckBox.Visible = !value;
                linksList.Visible = linksLabel.Visible = !value;
                _singleLinkMode = value;
            }
        }


		protected DownloadProperties CurrentDownload { get; private set; }


        private List<DownloadProperties> _downloads;
        public List<DownloadProperties> Downloads
        {
            get { return _downloads; }
        }
       
		internal OptionPageRenderer Renderer { get; private set; }

		internal Dictionary<DownloadProperties,GeneralPropertiesControl> PropertyControls { get; private set; }
		#endregion
    }
}
