﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using WideStream.Configuration;
using WideStream.DownloadManager.Configuration;
using WideStream.DownloadManager.Network;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.Logging;
using WideStream.Logging.Configuration;
using WideStream.Workspace;
using WideStream.Workspace.ComponentModel;
using WideStream.Workspace.Configuration;
using WideStream.Workspace.Forms;
using WideStream.Configuration.ComponentModel;
using System.Collections;
using System.IO;
using System.ComponentModel;
using WideStream.Configuration.Presentation;
using WideStream.Core.Runtime;
using WideStream.Startup.Resources;
using WideStream.Workspace.State;
using System.Drawing;

namespace WideStream.DownloadManager.Presentation
{
    public partial class DownloadPage : UserControl, IPage
    {
        #region Construction
        private Main parent;

        public DownloadPage()
        {
            InitializeComponent();
            LocalizationManager.Localize(this);
            ThemeManager.ApplyTheme(this);
            Dock = DockStyle.Fill;
        }
        #endregion


        #region Methods
        public void Initialize(Main parent, DataGridView logControl)
        {
            this.parent = parent;
            _logControl = logControl;
            QueueManagerOptions conf = Config.Query<QueueManagerOptions>();
            string path = conf.QueueDefinitionPath;
            QueueDefinition def = QueueDefinition.OpenQueue(path);

            if (def == null && ApplicationRegistry.IsKeyExists())
            {
                FilePathDialog d = new FilePathDialog();
                d.DialogTitle = StringTable.GetString("DownloadPage.PathDialogTitle");
                d.Description = StringTable.GetString("DownloadPage.PathDialogDescription");
                d.SelectedPath = conf.QueueDefinitionPath;
                d.ShowFileDialog = true;
                def = d.ShowDialog(this) == DialogResult.OK ? QueueDefinition.OpenQueue(d.SelectedPath) :
                    QueueDefinition.CreateQueue(d.SelectedPath);
                conf.QueueDefinitionPath = d.SelectedPath;

                if (def == null)
                    def = QueueDefinition.CreateQueue(d.SelectedPath);
            }
            else if (def == null)
                def = QueueDefinition.CreateQueue(path);

            QueueManager.Instance.Definition = def;            
            AddDownloads(QueueManager.Instance.Definition.Downloads, false);
			NotificationManager = new NotificationManager(parent);
			QueueManager.Instance.DownloadCompleted += (o, e) =>
				NotificationManager.ShowNotification(Notifications.DownloadCompleted, e.Download.GetDisplayName());
			QueueManager.Instance.DownloadFailed += (o, e) =>
				NotificationManager.ShowNotification(Notifications.DownloadFailed, e.Download.GetDisplayName());            
        }
        
        
        public void Open()
        {
            SetActiveDownload();
            
            if (Config.Query<DownloadManagerDisplayOptions>().QueueScroll == DownloadQueueScroll.Always &&
                threadGrid.Rows.Count > 0)
                threadGrid.FirstDisplayedScrollingRowIndex = threadGrid.Rows.Count - 1;
        }


        public bool CanClose()
        {
            List<String> links = new List<String>();
            List<DownloadProperties> downloads = new List<DownloadProperties>();

            foreach (DataGridViewRow dr in threadGrid.Rows)
            {
                if (!QueueManager.Instance.SupportRanges((Guid)dr.Tag) &&
                    QueueManager.Instance.QueryStatus((Guid)dr.Tag) == DownloadStatus.Started)
                    links.Add(dr.Cells["Url"].Value as String);

                downloads.Add(QueueManager.Instance.QueryProperties((Guid)dr.Tag));
            }

            if (links.Count > 0)
            {
                DownloadsDialog d = new DownloadsDialog();
                d.Downloads = links;

                if (d.ShowDialog(this) == DialogResult.No)
                    return false;
            }

            QueueManager.Instance.StopQueue();
            return true;
        }


        private DataGridViewRow FindRow(Guid key)
        {
            foreach (DataGridViewRow tr in threadGrid.Rows)
                if ((Guid)tr.Tag == key)
                    return tr;

            return null;
        }


        internal void SetActiveDownload()
        {
            if (ActiveLog != null)
            {
                _activeLog.Active = false;
                _activeLog = null;
                _logControl.Rows.Clear();
            }

            if (threadGrid.SelectedRows.Count > 0)
            {
                DataGridViewRow tr = threadGrid.SelectedRows[0];

                if (tr.Tag != null && tr.Tag is Guid)
                {
                    _activeLog = QueueManager.Instance.QueryLog((Guid)tr.Tag);
                    _activeLog.Active = true;

					if (Config.Query<LogOptions>().ScrollRule == LogScroll.Always ||
						Config.Query<LogOptions>().ScrollRule == LogScroll.WhenSelected)
						_activeLog.ScrollToEnd();
                }
            }
        }


        public IEnumerable<Guid> EnumerateDownloads()
        {
            foreach (DataGridViewRow r in threadGrid.Rows)
                yield return (Guid)r.Tag;
        }


        private void threadGrid_CellMouseClick(object sender, DataGridViewCellMouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right && e.RowIndex != -1)
                threadGrid.Rows[e.RowIndex].Selected = true;
        }
        
        private void threadGrid_SelectionChanged(object sender, EventArgs e)
        {
            SetActiveDownload();
        }


        private void addDownloadsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AddDownloads();
        }


        public void AddDownloads()
        {
            InternalAddDownloads(null, true);
        }


        private void InternalAddDownloads(IEnumerable<DownloadProperties> downloads, bool processClipboard)
        {
            ConfigureDownloadDialog d = new ConfigureDownloadDialog(processClipboard);

            if (downloads != null)
                d.Downloads.AddRange(downloads);

            if (d.ShowDialog(this) == DialogResult.OK)
                AddDownloads(d.EnumerateDownloads(),
                    (Int32)Config.Query<DownloadManagerDisplayOptions>().QueueScroll > 0);

            SetActiveDownload();
        }


        public void ConfigureDownloads(IEnumerable<DownloadProperties> downloads)
        {
            InternalAddDownloads(downloads, false);
        }


        public void AddDownloads(IEnumerable<DownloadProperties> downloads, bool scroll)
        {
            DataGridViewRow lastTr = null;
            int count;

            if ((count = downloads.Count()) > 50)
                threadGrid.Visible = false;

            foreach (DownloadProperties p in downloads)
            {
                DataGridViewRow tr = CreateRow(p);
                IProgressIndicator indicator = new ProgressIndicator(imageList, imageList2, tr);
                DownloadProperties prop = (DownloadProperties)tr.Tag;
                threadGrid.Rows.Add(tr);

				if (prop.State == null)
					prop.State = new DownloadState();
			
				prop.State.PreviousStatus = prop.State.PreviousStatus == DownloadStatus.Default || 
                    prop.State.PreviousStatus == DownloadStatus.Started ?
                    DownloadStatus.Default : prop.State.PreviousStatus;

                if (prop.GeneralOptions.ManualStart && prop.State.PreviousStatus == DownloadStatus.Default &&
                    prop.State.Segments.Count == 0)
                    prop.State.PreviousStatus = DownloadStatus.Paused;
			
				tr.Tag = QueueManager.Instance.QueueDowload<DownloadProcess>(prop, indicator, new GridLog(_logControl));
                lastTr = tr;
            }

            if (count > 50)
                threadGrid.Visible = true;

			if (downloads != null && NotificationManager != null)
				NotificationManager.ShowNotification(downloads.Count() > 1 ?
					Notifications.ManyDownloadAdded : Notifications.DownloadAdded, 
					downloads.First().GetDisplayName());

            if (scroll && threadGrid.Rows.Count > 0 && threadGrid.Rows.Count > 0)
                threadGrid.FirstDisplayedScrollingRowIndex = threadGrid.Rows.Count - 1;
        }


        private DataGridViewRow CreateRow(DownloadProperties p)
        {
            DataGridViewRow item = new DataGridViewRow();
            item.ContextMenuStrip = threadMenu;
            DataGridViewIconCell urlCell = new DataGridViewIconCell();
            
			if (p.Url != null)
				urlCell.Value = p.Url.ToString();

			urlCell.TextCallBack = () => p.GetDisplayName();
            urlCell.Padding = 3;
            DataGridViewTextBoxCell statusCell = new DataGridViewTextBoxCell();
            DataGridViewIconCell speedCell = new DataGridViewIconCell();
            item.Cells.AddRange(urlCell, new DataGridViewProgressBarCell(),
                new DataGridViewTextBoxCell(), new DataGridViewTextBoxCell(), speedCell,
                new DataGridViewTextBoxCell(), statusCell);
            item.Tag = p;
            return item;
        }


        private void removeDownloadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if ((Config.Query<WorkspaceOptions>().PromptDialogs & PromptDialogs.RemoveDownload) != PromptDialogs.RemoveDownload ||
                MessageBox.Show(this, StringTable.GetString("DownloadPage.PromptDelete"), Application.ProductName,
                MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
            {
                foreach (DataGridViewRow tr in threadGrid.SelectedRows)
                {
                    QueueManager.Instance.RemoveDownload((Guid)tr.Tag);
                    threadGrid.Rows.Remove(tr);
                }

				SetActiveDownload();
            }
        }

        private void removeAllSuccessfulDownloadsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if ((Config.Query<WorkspaceOptions>().PromptDialogs & PromptDialogs.RemoveAllSuccessfulDownloads) != PromptDialogs.RemoveAllSuccessfulDownloads ||
               MessageBox.Show(this, StringTable.GetString("DownloadPage.PromptDeleteAll"), Application.ProductName,
               MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
            {

                DataGridViewRow tr;

                foreach (Guid k in QueueManager.Instance.RemoveAllSuccessful())
                    if ((tr = FindRow(k)) != null)
                        threadGrid.Rows.Remove(tr);
            }
        }

        private void pauseDownloadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            foreach (Guid k in SelectedDownloads)
                QueueManager.Instance.PauseDownload(k);
        }

        private void resumeDownloadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            foreach (Guid k in SelectedDownloads)
                QueueManager.Instance.StartDownload(k);
        }

        private void stopDownloadToolStripMenuItem_Click(object sender, EventArgs e)
        {
			if ((Config.Query<WorkspaceOptions>().PromptDialogs & PromptDialogs.AbortDownload) != PromptDialogs.AbortDownload ||
              MessageBox.Show(this, StringTable.GetString("DownloadPage.PromptStop"), Application.ProductName,
              MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
            {
                foreach (Guid k in SelectedDownloads)
                    QueueManager.Instance.StopDownload(k);
            }
        }

        private void ClearSelection()
        {
            foreach (DataGridViewRow tr in threadGrid.SelectedRows)
                tr.Selected = false;
        }


        private void pauseAllDownloadsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            PauseAllDownloads();
        }


        public void PauseAllDownloads()
        {
            GroupApplyAction(k => QueueManager.Instance.PauseDownload(k), DownloadStatus.Started);
        }


        public void ResumeAllDownloads()
        {
            GroupApplyAction(k => QueueManager.Instance.StartDownload(k), DownloadStatus.Paused, 
				DownloadStatus.Aborted, DownloadStatus.Failed);
        }


        private void resumeAllDownloadsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ResumeAllDownloads();
        }


        private void GroupApplyAction(DownloadCallBack callBack, params DownloadStatus[] states)
        {
            foreach (DataGridViewRow tr in threadGrid.Rows)
            {
                Guid key = (Guid)tr.Tag;
				DownloadStatus state = QueueManager.Instance.QueryStatus(key);

                foreach (DownloadStatus s in states)
                    if (state == s)
                        callBack(key);
            }
        }


        private void moveDownloadUpToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (threadGrid.SelectedRows.Count == 1)
            {
                DataGridViewRow tr = threadGrid.SelectedRows[0];
                
                if (QueueManager.Instance.MoveUp((Guid)tr.Tag))
                {
                    int index = tr.Index;
                    threadGrid.Rows.Remove(tr);
                    threadGrid.Rows.Insert(--index, tr);
                    ClearSelection();
                    tr.Selected = true;
                }
            }
        }

        private void moveDownloadDownToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (threadGrid.SelectedRows.Count == 1)
            {
                DataGridViewRow tr = threadGrid.SelectedRows[0];

                if (QueueManager.Instance.MoveDown((Guid)tr.Tag))
                {
                    int index = tr.Index;
                    threadGrid.Rows.Remove(tr);
                    threadGrid.Rows.Insert(++index, tr);
                    ClearSelection();
                    tr.Selected = true;
                }
            }
        }

        private void downloadPropertiesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (threadGrid.SelectedRows.Count == 1)
            {
                ConfigureDownloadDialog d = new ConfigureDownloadDialog(false);
                Guid id = (Guid)threadGrid.SelectedRows[0].Tag;
                d.SingleLinkMode = true;
				DownloadStatus state = QueueManager.Instance.QueryStatus(id);
				d.LimitedMode = state == DownloadStatus.Default ||
					state == DownloadStatus.Started || state == DownloadStatus.Paused;
				DownloadProperties orig = QueueManager.Instance.QueryProperties(id);
				DownloadProperties copy = orig.Clone();
                d.AddDownload(orig);

				if (d.ShowDialog(this) == DialogResult.Cancel)
				{
					QueueManager.Instance.ReplaceProperties(id, copy);
					copy.Refresh();
				}
            }
        }


        private void threadGrid_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            var conf = Config.Query<DownloadManagerDisplayOptions>();

            if (conf.DefaultAction == DefaultAction.ShowProperties)
                downloadPropertiesToolStripMenuItem_Click(sender, EventArgs.Empty);
            else if (conf.DefaultAction == DefaultAction.OpenFolder)
                openContainingFolderToolStripMenuItem_Click(sender, EventArgs.Empty);
            else if (conf.DefaultAction == DefaultAction.FileOrProperties &&
                QueueManager.Instance.QueryStatus(SelectedDownload) == DownloadStatus.Finished)
                openFileToolStripMenuItem_Click(sender, EventArgs.Empty);
            else if (conf.DefaultAction == DefaultAction.FileOrProperties)
                downloadPropertiesToolStripMenuItem_Click(sender, EventArgs.Empty);
            else if (conf.DefaultAction == DefaultAction.FileOrFolder && 
                QueueManager.Instance.QueryStatus(SelectedDownload) == DownloadStatus.Finished)
                openFileToolStripMenuItem_Click(sender, EventArgs.Empty);
            else if (conf.DefaultAction == DefaultAction.FileOrFolder)
                openContainingFolderToolStripMenuItem_Click(sender, EventArgs.Empty);              
        }


        private void openContainingFolderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (threadGrid.SelectedRows.Count == 1)
            {
                DownloadProperties p = QueueManager.Instance.
                    QueryProperties((Guid)threadGrid.SelectedRows[0].Tag);
				Process.Start(p.GeneralOptions.DownloadPath);
            }
        }

        private void openFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
			if (threadGrid.SelectedRows.Count == 1)
			{
				DownloadProperties p = QueueManager.Instance.
					QueryProperties((Guid)threadGrid.SelectedRows[0].Tag);

                if (p.State.FileLocation != null)
                {
                    try
                    {
                        Process.Start(p.State.FileLocation.FullName);
                    }
                    catch (Win32Exception ex)
                    {
                        MessageBox.Show(this, String.Format(StringTable.GetString("DownloadPage.FileOpenError"), ex.Message), 
                            ProductName, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    }
                }
			}
        }

        
        private void copyDownloadURLToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (threadGrid.SelectedRows.Count > 0)
               CopyDownloadUrls(threadGrid.SelectedRows);
        }


        private void copyallDownloadsURLsToolStripMenuItem_Click(object sender, EventArgs e)
        {
			CopyDownloadUrls(threadGrid.Rows);
        }


		private void CopyDownloadUrls(IEnumerable rows)
		{
			StringBuilder sb = new StringBuilder();

			foreach (DataGridViewRow tr in rows)
			{
				sb.Append(tr.Cells["Url"].Value as String);
				sb.Append("\r\n");
			}

			Clipboard.SetText(sb.ToString());
		}


        private void suspendQueueToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SuspendQueue();  
        }


        public void SuspendQueue()
        {
            Config.Query<QueueManagerOptions>().SuspendQueue =
			   !Config.Query<QueueManagerOptions>().SuspendQueue;
        }
        #endregion


        #region Properties
        public ToolStrip ToolBar
        {
            get { return toolStrip; }
        }


        public Guid SelectedDownload
        {
            get
            {
                return threadGrid.SelectedRows.Count != 1 ? Guid.Empty :
                    (Guid)threadGrid.SelectedRows[0].Tag;
            }
        }


        public List<Guid> SelectedDownloads
        {
            get
            {
                List<Guid> guids = new List<Guid>();

                if (threadGrid.SelectedRows.Count > 0)
                {
                    foreach (DataGridViewRow tr in threadGrid.SelectedRows)
                        guids.Add((Guid)tr.Tag);
                }

                return guids;
            }
        }

        public Page PageType
        {
            get { return Page.DownloadManager; }
        }

        public MenuStrip MainMenu
        {
            get { return downloadMenu; }
        }


        public new ContextMenuStrip ContextMenu
        {
            get { return threadMenu; }
        }


        public ContextMenuStrip ColumnsMenu
        {
            get { return columnsMenu; }
        }


        private ILog _activeLog;
        public ILog ActiveLog
        {
            get { return _activeLog; }
        }


        public DataGridView DisplayControl 
        {
            get { return threadGrid; }
        }


        private DataGridView _logControl;
        public DataGridView LogControl
        {
            get { return _logControl; }
            set { _logControl = value; }
        }


		[Obsolete]
        public string CurrentStatus
        {
            get
            {
                if (QueueManager.Instance.Queue.Count == 0)
                    return StringTable.GetString("DownloadPage.NoDownloads");
                else
                {
                    int finished = 0;
                    int failed = 0;
                    int paused = 0;
                    int stopped = 0;
                    int active = 0;
                    int count = 0;

                    foreach (var p in QueueManager.Instance.Queue.Values)
                    {
                        count++;

                        switch (p.Status)
                        {
							case DownloadStatus.Started:
                                active++;
                                break;
							case DownloadStatus.Aborted:
                                stopped++;
                                break;
							case DownloadStatus.Failed:
                                failed++;
                                break;
							case DownloadStatus.Finished:
                                finished++;
                                break;
							case DownloadStatus.Paused:
                                paused++;
                                break;
                        }
                    }

                    return String.Format(StringTable.GetString("DownloadPage.TotalDownloadCount"),
                        count, active, finished, paused, stopped, failed);
                }
            }
        }

		internal NotificationManager NotificationManager { get; private set; }
        #endregion               

       
        private void ConnectionLimit_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem item = (ToolStripMenuItem)sender;
            int count = Convert.ToInt32(item.ToolTipText);
            Config.Query<QueueManagerOptions>().MaximumDownloads = (IntRange)count;            
        }


        private void createNewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (QueueManager.Instance.ActiveDownloads.Count() != 0)
            {
                MessageBox.Show(this, StringTable.GetString("DownloadPage.CreateQueueError"), ProductName, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            QueueManager.Instance.ApplyQueueChanges(); 
            QueueManager.Instance.Definition.Save();
            openFileDialog.Title = StringTable.GetString("DownloadPage.CreateDialogTitle");
            openFileDialog.Filter = StringTable.GetString("DownloadPage.OpenDialogFilter");
            openFileDialog.CheckFileExists = false;

            if (openFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                QueueManager.Instance.Definition = QueueDefinition.CreateQueue(openFileDialog.FileName);
                threadGrid.Rows.Clear();
                QueueManager.Instance.Definition.Save();
				parent.DisplayProgressBar(2000);
                Config.Query<QueueManagerOptions>().QueueDefinitionPath = openFileDialog.FileName;
            }
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (QueueManager.Instance.ActiveDownloads.Count() != 0)
            {
                MessageBox.Show(this, StringTable.GetString("DownloadPage.OpenQueueError"), ProductName, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            QueueManager.Instance.ApplyQueueChanges(); 
            QueueManager.Instance.Definition.Save();
            openFileDialog.Title = StringTable.GetString("DownloadPage.OpenDialogTitle");
            openFileDialog.Filter = StringTable.GetString("DownloadPage.OpenDialogFilter");
            openFileDialog.CheckFileExists = true;

            if (openFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                try
                {
                    QueueManager.Instance.Definition = QueueDefinition.OpenQueue(openFileDialog.FileName);
                    Config.Query<QueueManagerOptions>().QueueDefinitionPath = openFileDialog.FileName;
                }
                catch (Exception ex)
                {
                    string message;

                    if (ex is XmlException)
                        message = StringTable.GetString("DownloadPage.ReadQueueError") + ex.Message;
                    else
                        message = StringTable.GetString("DownloadPage.ReadXmlError") + ex.Message;

                    MessageBox.Show(this, message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                threadGrid.Rows.Clear();
                AddDownloads(QueueManager.Instance.Definition.Downloads,
                    (Int32)Config.Query<DownloadManagerDisplayOptions>().QueueScroll > 0);
            }
        }

        private void saveQueueToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveQueue();
        }


        private void saveQueueAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            openFileDialog.Title = StringTable.GetString("DownloadPage.SaveDialogTitle");
            openFileDialog.Filter = StringTable.GetString("DownloadPage.OpenDialogFilter");
            openFileDialog.CheckFileExists = false;

            if (openFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                QueueManager.Instance.ApplyQueueChanges();
                QueueManager.Instance.Definition.SaveAs(openFileDialog.FileName);
                Config.Query<QueueManagerOptions>().QueueDefinitionPath = openFileDialog.FileName;
				parent.DisplayProgressBar(2000);
            }
        }


        public void SaveQueue()
        {
            QueueDefinition def = QueueManager.Instance.Definition;

            if (def != null)
            {
                QueueManager.Instance.ApplyQueueChanges();
                def.Save();
                parent.DisplayProgressBar(2000);
            }
        }
		      
		
		private void downloadPriorityToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
		{
			if (SelectedDownloads.Count == 1)
			{
				DownloadProperties properties = QueueManager.Instance.QueryProperties(SelectedDownloads[0]);
				lowToolStripMenuItem.Checked = properties.GeneralOptions.Priority == DownloadPriority.Low;
				normalToolStripMenuItem.Checked = properties.GeneralOptions.Priority == DownloadPriority.Normal;
				highToolStripMenuItem.Checked = properties.GeneralOptions.Priority == DownloadPriority.High;
			}
			else
				lowToolStripMenuItem.Checked = normalToolStripMenuItem.Checked = 
					highToolStripMenuItem.Checked = false;
		}


		private void priority_Click(object sender, EventArgs e)
		{
			ToolStripMenuItem item = (ToolStripMenuItem)sender;
			DownloadPriority priority = (DownloadPriority)item.MergeIndex;

			foreach (Guid k in SelectedDownloads)
				QueueManager.Instance.QueryProperties(k).GeneralOptions.Priority = priority;
		}

		private void maximumDownloadsToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
		{
			if (maximumDownloadsToolStripMenuItem.DropDownItems.Count < 3)
            {
                foreach (int i in Enum.GetValues(typeof(IntRange)))
                {
                    var mi = new ToolStripMenuItem(String.Format(StringTable.GetString("DownloadPage.DownloadCount"), i));
                    mi.ToolTipText = i.ToString();
                    mi.Click += ConnectionLimit_Click;
                    maximumDownloadsToolStripMenuItem.DropDownItems.Add(mi);
                }
            }

            int max = (Int32)Config.Query<QueueManagerOptions>().MaximumDownloads;
            
            foreach (ToolStripItem i in maximumDownloadsToolStripMenuItem.DropDownItems)
				if (i is ToolStripMenuItem)
					((ToolStripMenuItem)i).Checked = false;

			if (max > 0)
				((ToolStripMenuItem)maximumDownloadsToolStripMenuItem.DropDownItems[max + 1]).Checked = true;
		}

		private void threadGrid_EnterPressed(object sender, EventArgs e)
		{
			downloadPropertiesToolStripMenuItem_Click(sender, e);
		}

        private void columnsMenu_Opening(object sender, CancelEventArgs e)
        {            
            var config = Config.Query<DownloadManagerDisplayOptions>();
         
            if (columnsMenu.Items.Count < 3)
            {
                foreach (var v in config.QueueColumns.GetDisplayNames())
                {
                    var mi = new ToolStripMenuItem(v.Name);
                    mi.Tag = v.Value;
                    mi.Click += (o, ev) => {
                        if (((ToolStripMenuItem)o).Checked)
                            config.QueueColumns ^= (QueueColumns)((ToolStripMenuItem)o).Tag;
                        else
                            config.QueueColumns |= (QueueColumns)((ToolStripMenuItem)o).Tag;
                    };
                    columnsMenu.Items.Insert(columnsMenu.Items.Count - 2, mi);
                }
            }

            foreach (ToolStripItem mi in columnsMenu.Items)
            {
                if (mi.Tag != null)
                {
                    QueueColumns val = (QueueColumns)mi.Tag;
                    ((ToolStripMenuItem)mi).Checked = (config.QueueColumns & val) == val;
                }
            }
        }

        private void columnsMoreItem_Click(object sender, EventArgs e)
        {
            ((Main)this.ParentForm).ShowOptionsDialog(StringTable.
                GetString("DownloadManagerDisplayOptions.ShortName"));
        }

        private void threadMenu_Opening(object sender, CancelEventArgs e)
        {
            contextOpenFile.Font = new Font(contextOpenFile.Font, FontStyle.Regular);
            contextOpenFolder.Font = new Font(contextOpenFolder.Font, FontStyle.Regular);
            contextDownloadProperties.Font = new Font(contextDownloadProperties.Font, FontStyle.Regular);
            ToolStripMenuItem item = null;

            var conf = Config.Query<DownloadManagerDisplayOptions>();

            if (conf.DefaultAction == DefaultAction.ShowProperties)
                item = contextDownloadProperties;
            else if (conf.DefaultAction == DefaultAction.OpenFolder)
                item = contextOpenFolder;
            else if (conf.DefaultAction == DefaultAction.FileOrProperties)
                item = QueueManager.Instance.QueryStatus(SelectedDownload) == DownloadStatus.Finished ?
                    contextOpenFile : contextDownloadProperties;
            else if (conf.DefaultAction == DefaultAction.FileOrFolder)
                item = QueueManager.Instance.QueryStatus(SelectedDownload) == DownloadStatus.Finished ?
                    contextOpenFile : contextOpenFolder;

            item.Font = new Font(item.Font, FontStyle.Bold);
        }

    }
}
