﻿using System;
using System.Drawing;
using System.Windows.Forms;
using WideStream.Core.Runtime;
using WideStream.Configuration.ComponentModel;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.Workspace.ComponentModel;
using WideStream.Workspace.Forms;
using WideStream.Core.Network;
using WideStream.Configuration;
using WideStream.DownloadManager.Configuration;
using WideStream.Startup.Resources;

namespace WideStream.DownloadManager.Presentation
{
	public sealed class ProgressIndicator : IProgressIndicator
	{
		#region Construction
		private DataGridViewRow item;
		private DownloadUserControl downloadControl;
		private ImageList imageList;
		private ImageList imageListSmall;
		private DateTime lastUpdate;
		private DateTime lastSpeedUpdate;
		private int? lastSpeed;
		private long lastTotalBytesRead;
		private long aggregativeBytesRead;
		private float fakeLength;
		private DownloadProperties properties;
        private int currentConnections;
        private bool staticsSet;
        private string lastImage;

		public ProgressIndicator(ImageList imageList, ImageList imageListSmall, DataGridViewRow item)
		{
			this.imageList = imageList;
			this.imageListSmall = imageListSmall;
            this.lastImage = null;
            this.item = item;
		}
		#endregion


		#region Methods
		public void Initialize(long totalLength, DateTime startTime, object data)
		{
			_contentLength = totalLength;
			_startTime = startTime;
			fakeLength = 2000000;
			properties = data as DownloadProperties;
		}


        public void ReportStep(long totalProgress, int currentProgress)
        {
            aggregativeBytesRead += currentProgress;
           
            if (!staticsSet)
                ResetStyle();

            lastTotalBytesRead = totalProgress;
            string length = CalculateLength(_contentLength);
            bool unknown = length == null;
            long progress = CalculateProgress(totalProgress, unknown);
            ChangeCellValue<Int64>("Progress", progress);

            if (!staticsSet && _contentLength != 0)
            {
                ChangeCellValue<String>("SizeBox", length ?? StringTable.GetString("Progress.Unknown"));

                if (unknown && item.Cells["SizeBox"].Style.ForeColor != Color.Red)
                    item.Cells["SizeBox"].Style.ForeColor = Color.Red;
                else if (!unknown && item.Cells["SizeBox"].Style.ForeColor != default(Color))
                    item.Cells["SizeBox"].Style.ForeColor = default(Color);

                staticsSet = true;
            }

            ChangeCellValue<String>("Downloaded", CalculateLength(totalProgress));
            double secLeft = (DateTime.Now - lastSpeedUpdate).TotalMilliseconds;

            if (secLeft >= 3000D || lastSpeed == null)
            {
                int speedNew = (Int32)(aggregativeBytesRead / (secLeft / 1000));
                ChangeCellValue<String>("Speed", new DataSize(speedNew < 0 ? 0 : speedNew).ToString(SizeFormat.Speed));
                int diff = (lastSpeed == null ? 0 : lastSpeed.Value) - speedNew;
                string image = diff > 1000 ? "decrease" : diff < -1000 ? "increase" : "nochange";
                
                if (image != lastImage)
                {
                    var speedCell = (DataGridViewIconCell)item.Cells["Speed"];
                    speedCell.SmallImage = imageListSmall.Images[image];
                    lastImage = image;
                }

                lastSpeedUpdate = DateTime.Now;
                lastSpeed = speedNew;
                aggregativeBytesRead = 0;
            }

           TimeSpan? time = null;

            if (unknown && !staticsSet)
            {
                ChangeCellValue<String>("TimeLeft", StringTable.GetString("Progress.Unknown"));

                if (item.Cells["TimeLeft"].Style.ForeColor != Color.Red)
                    item.Cells["TimeLeft"].Style.ForeColor = Color.Red;
            }
            else if (!unknown)
            {
                if (item.Cells["TimeLeft"].Style.ForeColor != default(Color))
                    item.Cells["TimeLeft"].Style.ForeColor = default(Color);

                float timeLeft = lastSpeed != null && lastSpeed.Value != 0 ?
                    (_contentLength - totalProgress) / lastSpeed.Value : -1;
                time = new TimeSpan(0, 0, (Int32)timeLeft);

                if (timeLeft > 0)
                    ChangeCellValue<String>("TimeLeft", time.Value.ToString());
            }


            if (IsControlAlive())
            {
                downloadControl.Progress = (Int32)progress;
                downloadControl.DownloadSize = new DataSize(_contentLength);
                downloadControl.Downloaded = new DataSize(totalProgress);
                downloadControl.Connections = currentConnections;

                if (time != null)
                    downloadControl.TimeRemaining = time.Value;

                if (lastSpeed != null)
                    downloadControl.Speed = new DataSize(lastSpeed.Value);

                if (properties != null)
                    downloadControl.DownloadName = properties.GetDisplayName();
            }

            lastUpdate = DateTime.Now;
        }


		private bool IsControlAlive()
		{
			return downloadControl != null && !downloadControl.IsDisposed &&
				downloadControl.IsHandleCreated;
		}


		private void ChangeCellValue<T>(string key, T value)
		{
            if ((item.Cells[key].Value == null || !item.Cells[key].Value.DirectCast<T>().Equals(value)) &&
                item.DataGridView != null && !item.DataGridView.IsDisposed && item.DataGridView.IsHandleCreated)
            {
                try
                {
                    item.Cells[key].Value = value;
                }
                catch (ArgumentException)
                {

                }

            }
		}


		public void ChangeAttribute<T>(Enum attribute, T value)
		{
			if (!(attribute is ProgressAttribute))
				throw new NotSupportedException();

			switch ((ProgressAttribute)attribute)
			{
				case ProgressAttribute.Status:
					ChangeStatus(value.DirectCast<DownloadStatus>());
					break;
				case ProgressAttribute.Priority:
					ChangePriority(value.DirectCast<DownloadPriority>());
					break;
				case ProgressAttribute.DownloadDisplay:
					ChangeDownloadDisplay(value.DirectCast<DownloadProperties>());
					break;
				case ProgressAttribute.Connections:
					ChangeConnections(value.DirectCast<Int32>());
					break;
			}
		}


		private void ChangeConnections(int connections)
		{
			SafeInvoke(() =>
				{
                    currentConnections = connections;
                    ChangeCellValue<Int32>("Connections", currentConnections);
                    item.Cells["Connections"].Style.ForeColor = currentConnections == 0 ? Color.Gray : default(Color);
				}
			);
		}


		private void ChangeStatus(DownloadStatus state)
		{
            staticsSet = false;

			if (state != DownloadStatus.Started)
			{
				ActiveDownloadsManager.RemoveActiveDownload(downloadControl);
				downloadControl = null;
			}
			else if (state == DownloadStatus.Started && downloadControl == null)
			{
				downloadControl = new DownloadUserControl();
				ActiveDownloadsManager.AddActiveDownload(downloadControl);
			}

			FillDefault();
			SafeInvoke(() =>
				{
					Image image = imageList.Images.ContainsKey(state.ToString()) ?
						imageList.Images[state.ToString()] : null;
					((DataGridViewIconCell)item.Cells["Url"]).Image = image;
					item.Cells["Status"].Value = state.ToDisplayString();

					if (AutoRedraw)
						item.DataGridView.Refresh();

					if (state == DownloadStatus.Finished || state == DownloadStatus.Aborted ||
						state == DownloadStatus.Paused)
					{
						if (state != DownloadStatus.Paused)
						{
							item.Cells["Progress"].Value = state == DownloadStatus.Aborted ? 0 : 100;
							item.Cells["TimeLeft"].Value = new TimeSpan(0, 0, 0).ToString();
							item.Cells["TimeLeft"].Style.ForeColor = default(Color);
							item.Cells["SizeBox"].Value = CalculateLength(lastTotalBytesRead);
							item.Cells["SizeBox"].Style.ForeColor = default(Color);
							lastSpeed = null;
						}
					}
					else if (state == DownloadStatus.Started)
						lastSpeedUpdate = DateTime.Now;
				}
			);
		}


		private void ResetStyle()
		{
			if (item.Index != -1)
			{
				item.Cells["Speed"].Style.ForeColor = default(Color);

				if (item.Cells["SizeBox"].Style.ForeColor != Color.Red)
					item.Cells["SizeBox"].Style.ForeColor = default(Color);

				item.Cells["Downloaded"].Style.ForeColor = default(Color);
				item.Cells["Connections"].Style.ForeColor = default(Color);

				if (item.Cells["TimeLeft"].Style.ForeColor != Color.Red)
					item.Cells["TimeLeft"].Style.ForeColor = default(Color);
			}
		}


		private void FillDefault()
		{
			if (item.Index != -1)
			{
				SetDefaultValue("Speed", new DataSize(0).ToString(SizeFormat.Speed));
				SetDefaultValue("SizeBox", StringTable.GetString("Progress.NoData"));
                SetDefaultValue("Downloaded", StringTable.GetString("Progress.NoData"));
				SetDefaultValue("Connections", 0);
				SetDefaultValue("TimeLeft", "00:00:00");
				((DataGridViewIconCell)item.Cells["Speed"]).SmallImage = imageListSmall.Images["nochange"];
			}
		}


		private void SetDefaultValue(string cellKey, object value)
		{
			if (item.Cells[cellKey].Value == null ||
				String.IsNullOrEmpty(item.Cells[cellKey].Value as String))
			{
				item.Cells[cellKey].Value = value;
				item.Cells[cellKey].Style.ForeColor = Color.Gray;
			}
		}


		public void ChangePriority(DownloadPriority priorEnum)
		{
			SafeInvoke(() =>
				{
					DataGridViewIconCell cell = (DataGridViewIconCell)item.Cells["Url"];

					switch (priorEnum)
					{
						case DownloadPriority.High:
							cell.SmallImage = imageListSmall.Images["high"];
							break;
						case DownloadPriority.Normal:
							cell.SmallImage = imageListSmall.Images["normal"];
							break;
						case DownloadPriority.Low:
							cell.SmallImage = imageListSmall.Images["low"];
							break;
					}

					item.DataGridView.Refresh();
				}
			);
		}


		public void ChangeDownloadDisplay(DownloadProperties properties)
		{
			SafeInvoke(() =>
				{
					DataGridViewIconCell cell = (DataGridViewIconCell)item.Cells["Url"];
					cell.Value = properties.GetDisplayName();
					item.DataGridView.Refresh();
				}
			);
		}


		private string CalculateLength(float value)
		{
			if (value == Int64.MaxValue)
				return null;

            return new DataSize((Int64)value).ToString();
		}


		private long CalculateProgress(long totalBytesRead, bool unknown)
		{
			float calcLength = unknown ? fakeLength : _contentLength;
			long val = (Int64)(totalBytesRead / (calcLength / 100));

			if (unknown && val >= 100)
				fakeLength *= 2;

			return val > 100 ? 100 : (val < 0 ? 0 : val);
		}


		private void SafeInvoke(MethodInvoker func)
		{
			if (item.DataGridView != null && !item.DataGridView.IsDisposed && 
				item.DataGridView.IsHandleCreated)
			{
				try
				{
                    item.DataGridView.Invoke(func);
				}
				catch (ArgumentException)
				{

				}
			}
		}
		#endregion


		#region Properties
		private long _contentLength;
		public long ContentLength
		{
			get { return _contentLength; }
		}


		private DateTime _startTime;
		public DateTime StartTime
		{
			get { return _startTime; }
		}


		public bool AutoRedraw { get; set; }
		#endregion
	}
}
