﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Windows.Forms;
using WideStream.Configuration;
using WideStream.DownloadManager.Configuration;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.Logging;
using WideStream.Workspace.ComponentModel;
using downevent = System.EventHandler<WideStream.DownloadManager.DownloadEventArgs>;
using WideStream.Startup.Resources;

namespace WideStream.DownloadManager
{
    public sealed class QueueManager : IDownloadStatistics
    {
        #region Construction
		private System.Threading.Timer timer;
       
		private QueueManager()
        {
			_queue = new Dictionary<Guid,IDownloadProcess>();
            LeaseManager = new ConnectionLeaseManager(this);

			timer = new System.Threading.Timer(
				(o) =>
				{
					if (!processQueue)
					{
						processQueue = true;

						lock (Queue)
						{
                            int acnt = CountActiveDownloads();

							if (!Config.Query<QueueManagerOptions>().SuspendQueue && !Terminating && 
								acnt < (Int32)Config.Query<QueueManagerOptions>().MaximumDownloads)
								PeekDownload();

							if (acnt > 0)
								LeaseManager.Execute(ActiveDownloads);
						}

						processQueue = false;
					}
				}, null, 0, 1000);
        }
        #endregion


        #region Methods
		volatile bool processQueue;
        public Guid QueueDowload<T>(DownloadProperties properties, IProgressIndicator indicator, ILog log)
			where T : IDownloadProcess, new()
        {
            IDownloadProcess process = new T();
            var key = Guid.NewGuid();
			process.Initialize(key, properties, log, indicator);
			process.StatusChanged += (o, e) => ProcessStatusChanged((IDownloadProcess)o);
            process.Position = Queue.Count;
            Queue.Add(key, process);
            log.AddEntry(LogEntryType.Information, StringTable.GetString("Downloads.DownloadQueued"));
			return key;
        }

        public void RemoveDownload(Guid key)
        {
			var p = Queue[key];
			p.StatusChanged -= (o, e) => ProcessStatusChanged((IDownloadProcess)o);

			if (p.Status == DownloadStatus.Started || p.Status == DownloadStatus.Default)
				p.Abort();

			Queue.Remove(key);
        }


        public IEnumerable<Guid> RemoveAllSuccessful()
        {
            var downs = new List<IDownloadProcess>(Queue.Values.Where(pr => pr.Status == DownloadStatus.Finished));

			foreach (var p in downs)
			{
				Queue.Remove(p.Key);
				yield return p.Key;
			}
        }


        public void PauseDownload(Guid key)
        {
			Queue[key].Pause();
        }

        
        public void StopDownload(Guid key)
        {
			Queue[key].Abort();
        }
        

        public void StartDownload(Guid key)
        {
			Queue[key].Wait();
        }


        public IEnumerable<IDownloadProcess> Query(Func<IDownloadProcess,Boolean> callBack)
        {
            return Queue.Values.Where(callBack);
        }


        public DownloadStatus QueryStatus(Guid key)
        {
            IDownloadProcess p = null;
            Queue.TryGetValue(key, out p);
            return p != null ? p.Status : DownloadStatus.Default;
        }


        public ILog QueryLog(Guid key)
        {
			return Queue[key].Log;
		}


        public bool MoveUp(Guid key)
        {
            return Move(key, -1);
        }


        public bool MoveDown(Guid key)
        {
            return Move(key, 1);
        }


        public bool SupportRanges(Guid key)
        {
			IDownloadProcess proc;
			return (proc = Queue[key]).ConnectionInformation != null ?
				proc.ConnectionInformation.SupportRanges : false;
        }


        [MethodImpl(MethodImplOptions.Synchronized)]
        public void StopQueue()
        {
			Terminating = true;

			foreach (IDownloadProcess p in Queue.Values.Where(pr => pr.Status == DownloadStatus.Started))
			{
				if (p.ConnectionInformation != null && p.ConnectionInformation.SupportRanges)
				{
					DownloadStatus status = p.Status;
					p.Pause();
					p.Properties.State.PreviousStatus = status;
				}
				else
					p.Abort();
			}
		 }


        public DownloadProperties QueryProperties(Guid key)
        {
			return Queue[key].Properties;
        }


		public void ReplaceProperties(Guid key, DownloadProperties properties)
		{
			Queue[key].Properties = properties;
		}


        private bool Move(Guid key, int shift)
        {
			var p = Queue[key];
			bool moveable = shift > 0 ? p.Position != _queue.Count - 1 : p.Position != 0;

            if (moveable)
                p.Position += shift;

            return moveable;
        }


        private void PeekDownload()
        {
			var proc = Queue.Values.OrderBy(p => p.Position).FirstOrDefault(p => p.Status == DownloadStatus.Default ||
					p.Status == DownloadStatus.Failed && Config.Query<QueueManagerOptions>().AutoRestartFailed);

			if (proc != null)
				proc.Start();            
        }


        public void ApplyQueueChanges()
        {
			if (Definition != null)
			{
				Definition.Downloads.Clear();

				foreach (var p in Queue.Values)
					Definition.Downloads.Add(p.Properties);
			}
        }


        public void Save()
        {
            ApplyQueueChanges();

			if (Definition != null)
				Definition.Save();
        }


		private void ProcessStatusChanged(IDownloadProcess process)
		{
			if (process.Status == DownloadStatus.Finished)
				OnDownloadCompleted(new DownloadEventArgs(process.Properties));
			else if (process.Status == DownloadStatus.Failed)
				OnDownloadFailed(new DownloadEventArgs(process.Properties));
		}

		public int CountActiveConnections()
		{
            return InternalCount(null, true);
		}

		public int CountActiveConnections(DownloadPriority priority)
		{
            return InternalCount(priority, true);
		}


        private int InternalCount(DownloadPriority? priority, bool countConnections)
        {
            int count = 0;

            foreach (var p in Queue.Values)
            {
                if (p.Status == DownloadStatus.Started && (priority == null || p.Priority == priority.Value))
                    count += countConnections ? p.ActiveConnectionsCount : 1;
            }

            return count;
        }

		public int CountActiveDownloads()
		{
            return InternalCount(null, false);
		}

		public int CountActiveDownloads(DownloadPriority priority)
		{
            return InternalCount(priority, false);
		}
        #endregion


		#region Events
		public event downevent DownloadCompleted;
		private void OnDownloadCompleted(DownloadEventArgs e)
		{
			downevent h = DownloadCompleted;

			if (h != null)
				h(this, e);
		}


		public event downevent DownloadFailed;
		private void OnDownloadFailed(DownloadEventArgs e)
		{
			downevent h = DownloadFailed;

			if (h != null)
				h(this, e);
		}
		#endregion


		#region Properties
		internal ConnectionLeaseManager LeaseManager { get; private set; }

		public bool Terminating { get; private set; }

		public IEnumerable<IDownloadProcess> ActiveDownloads
		{
			get { return Queue.Values.Where(p => p.Status == DownloadStatus.Started); }
		}


        private QueueDefinition _definition;
        public QueueDefinition Definition
        {
            get { return _definition; }
            set 
            {
                if (value != _definition)
                    Queue.Clear();

                _definition = value; 
            }
        }


        public int ActiveLowPriority
        {
            get { return InternalCount(DownloadPriority.Low, false); }
        }
        

        public int ActiveNormalPriority
        {
            get { return InternalCount(DownloadPriority.Normal, false); }
        }


        public int ActiveHighPriority
        {
            get { return InternalCount(DownloadPriority.High, false); }
        }


		private Dictionary<Guid,IDownloadProcess> _queue;
        internal Dictionary<Guid,IDownloadProcess> Queue
        {
            get { return _queue; }
        }


        private static QueueManager _instance;
        public static QueueManager Instance
        {
            get
            {
                if (_instance == null)
                    lock (typeof(QueueManager))
                        if (_instance == null)
                            _instance = new QueueManager();

                return _instance;
            }
        }
        #endregion
    }
}
