﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using WideStream.Configuration;
using WideStream.DownloadManager;
using WideStream.DownloadManager.Network;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.Core.Network;
using WideStream.Core.Runtime;
using WideStream.DownloadManager.Configuration;
using WideStream.HostManager.Configuration;
using WideStream.Configuration.ComponentModel;
using System.Reflection;

namespace WideStream.DownloadManager
{
    public sealed class UrlAnalyzer
    {
        #region Construction
        private bool populateForOne;

        public UrlAnalyzer(bool populateForOne) 
        {
            this.populateForOne = populateForOne;
        }
        #endregion


        #region Methods
        public AnalysisResult AnalyzeInput(CurrentData data)
        {
            bool clipboardMode = data.InputValue.Length - LastInputLength > 3;

            if (!clipboardMode && data.InputValue.EndsWith("\r\n"))
                return new AnalysisResult { Actions = RequiredActions.AddEmptyDownload, 
                    InputValue = data.InputValue.Replace("\r\n", String.Empty) };

            LastInputLength = data.InputValue.Length;
            string[] arr = data.InputValue.Split(new char[] { '\r', '\n' }, StringSplitOptions.RemoveEmptyEntries);

            if (arr.Length > 1 || populateForOne)
            {
                string newInputText = null;
                List<DownloadProperties> list = new List<DownloadProperties>();
                
                for (int i = 0; i < arr.Length; i++)
                {
                    string link = arr[i];
                    bool add = true;
                    Uri uri;

                    if (!(add = Uri.TryCreate(link, UriKind.Absolute, out uri)) && !clipboardMode)
                    {
                        add = true;
                        newInputText = link;
                    }

                    if (add)
                    {
                        DownloadProperties p = new DownloadProperties();
                        InitializeDownload(p, null, uri);
                        list.Add(p);
                    }
                }

                if (list.Count > 0)
                {
                    RequiredActions actions = RequiredActions.UpdateLinkList;

                    if (IsRenameGenerated(data))
                        actions |= RequiredActions.ClearRenameValue;

                    return new AnalysisResult { Actions = actions, Downloads = list, 
                        InputValue = newInputText };
                }
                else
                    return new AnalysisResult { Actions = RequiredActions.UpdateInput, 
                        InputValue = data.InputValue.Replace("\r\n", String.Empty) };
            }
            else
                return RefreshLinkData(data);
        }


        public void InitializeDownload(DownloadProperties properties, CurrentData data, Uri uri)
        {
            properties.Url = uri;
            properties.GeneralOptions = (DownloadProcessOptions)Config.Query<DownloadProcessOptions>().Clone();
            			
			if (uri != null)
			{
				Host host;

				if ((host = Config.Query<HostManagerOptions>().Hosts.FirstOrDefault(h => h.Slike(uri.Host))) != null)
				{
					if (host.Credentials != null && !String.IsNullOrEmpty(host.Credentials.UserName) && 
						(data == null || !data.HasCredentials))
						properties.Credentials = host.Credentials;

		            ApplyOptions(properties.GeneralOptions, host.GeneralOptions);
                    
					foreach (Options o in host.ProtocolOptions)
                        ApplyOptions(properties.ProtocolOptions.First(op => op.GetType() == o.GetType()), o);
				}

                if (!String.IsNullOrEmpty(uri.UserInfo))
                {
                    string[] arr = uri.UserInfo.Split(':');
                    properties.Credentials = new Credentials(arr.Length > 0 ?
                        arr[0] : String.Empty, arr.Length > 1 ? arr[1] : String.Empty);
                }
			}
        }


        private void ApplyOptions(Options target, Options source)
        {
            Type type = source.GetType();

            foreach (string s in source.IterateEnableValues())
            {
                PropertyInfo pi = type.GetProperty(s);
                object val = pi.GetValue(source, null);
                pi.SetValue(target, val, null);
            }
        }


        private AnalysisResult RefreshLinkData(CurrentData data)
        {
            RequiredActions actions = RequiredActions.None;
            NetworkCredential credentials = null;
            string fileName = null;
            Uri uri;
			Host host = null;
            
            if (Uri.TryCreate(data.InputValue, UriKind.Absolute, out uri))
            {
                if (!String.IsNullOrEmpty(uri.UserInfo))
                {
                    actions |= RequiredActions.SetCredentials;
					string[] arr = uri.UserInfo.Split(':');
                    credentials = new NetworkCredential(arr.Length > 0 ? arr[0] : String.Empty,
						arr.Length > 1 ? arr[1] : String.Empty);
                }
                else if (!data.HasCredentials)
                {
                    if ((host = Config.Query<HostManagerOptions>().Hosts.FirstOrDefault(h => h.Slike(uri.Host))) != null)
					{
						actions |= RequiredActions.SetHostDefaultData;
						credentials = host.Credentials;

						if (credentials != null && !String.IsNullOrEmpty(credentials.UserName))
							actions |= RequiredActions.SetCredentials;
					}
                }               

                try
                {
                    fileName = new FileInfo(uri.LocalPath).Name;

                    if (IsRenameGenerated(data))
                        actions |= RequiredActions.SetRenameValue;
                }
                catch 
                { 
                    if (IsRenameGenerated(data))
                        actions |= RequiredActions.ClearRenameValue;
                }

                actions |= RequiredActions.UpdateLink;
                LastUri = uri;
            }
            else
            {
                if (LastUri != null && !String.IsNullOrEmpty(LastUri.UserInfo) && data.HasCredentials)
                    actions |= RequiredActions.ClearCredentials;

                if (IsRenameGenerated(data))
                    actions |= RequiredActions.ClearRenameValue;

                LastUri = null;
            }

            return new AnalysisResult { Actions = actions, RenameValue = fileName, SampleHost = host, Credentials = credentials };
        }


        private bool IsRenameGenerated(CurrentData data)
        {
            try
            {
                string fn = null;
                return String.IsNullOrEmpty(data.RenameValue) || (LastUri != null && 
                    (String.Compare(fn = new FileInfo(LastUri.LocalPath).Name, data.RenameValue, true) == 0 || 
                    (!String.IsNullOrEmpty(fn) && data.RenameValue.StartsWith(fn, StringComparison.OrdinalIgnoreCase))));
            }
            catch
            {
                return false;
            }
        }


        public void ChangeState(string inputValue, Uri uri)
        {
            LastInputLength = inputValue.Length;
            LastUri = uri;
        }
        #endregion


        #region Properties
        public int LastInputLength { get; private set; }

        public Uri LastUri { get; private set; }
        #endregion
    }
}
