﻿using System;
using System.IO;
using System.Net;
using System.Runtime.CompilerServices;
using WideStream.Configuration;
using WideStream.FtpManager.Configuration;
using WideStream.Logging;
using WideStream.Startup;
using WideStream.Startup.Resources;
using WideStream.Startup.ExceptionHandling;
using System.Text;

namespace WideStream.FtpManager
{
    public sealed class FtpExplorer
    {
        #region Construction
        private FtpWebRequest currentRequest;
        private int retryCount;
       
        private FtpExplorer()
        {
            
        }
        #endregion


        #region Methods
        public Uri ListParentDirectory()
        {
            retryCount = 0;
            
            if (HasParent)
            {
                DirectoryProperties prop = new DirectoryProperties();
                prop.Credentials = _properties.Credentials;
                string parentUri = _properties.Uri.ToString();
                prop.Uri = new Uri(parentUri.Substring(0, parentUri.Trim('/').LastIndexOf("/")));
                _properties = prop;
                InternalListDirectory(_properties, _log, _explorer);
                return _properties.Uri;
            }
            else
                return null;
        }

		
		public void Refresh()
        {
            retryCount = 0;
            
            if (_properties != null)
                InternalListDirectory(_properties, _log, _explorer);
        }


        public void ListDirectory(DirectoryProperties properties, ILog log, IExplorer explorer)
        {
            retryCount = 0;
            InternalListDirectory(properties, log, explorer);
        }


        private void InternalListDirectory(DirectoryProperties properties, ILog log, IExplorer explorer)
        {
            _isWorking = true;
            _log = log;
            _explorer = explorer;
            _properties = properties;
            FtpWebRequest req = (FtpWebRequest)FtpWebRequest.Create(properties.Uri);
			req.UsePassive = Config.Query<FtpManagerOptions>().UsePassive;
			req.KeepAlive = Config.Query<FtpManagerOptions>().KeepAlive;

            if (properties.Credentials != null)
                req.Credentials = properties.Credentials;

			req.Timeout = (Int32)Config.Query<FtpManagerOptions>().Timeout;
            _log.AddEntry(LogEntryType.Information, StringTable.GetString("Explorer.Timeout"), req.Timeout);            
            req.Method = WebRequestMethods.Ftp.ListDirectoryDetails;
            currentRequest = req;
            _log.AddEntry(LogEntryType.Information, StringTable.GetString("Explorer.Connecting"), _properties.Uri);
            _log.AddEntry(LogEntryType.Information, StringTable.GetString("Explorer.Method"), req.Method);
            IAsyncResult result = req.BeginGetResponse(ProcessRequest, req);
        }


        private void ProcessRequest(IAsyncResult res)
        {
            if (res.IsCompleted)
            {
                try
                {
                    FtpWebRequest req = (FtpWebRequest)res.AsyncState;
                    FtpWebResponse response = (FtpWebResponse)req.EndGetResponse(res);
                    _log.AddEntry(LogEntryType.Information, response.BannerMessage);
                    _log.AddEntry(LogEntryType.Information, response.WelcomeMessage);
                    _log.AddEntry(LogEntryType.Warning, response.StatusDescription);
                    ListingParser parser = new ListingParser(_properties.Uri);
                    Stream responseStream = response.GetResponseStream();

                    using (StreamReader sr = new StreamReader(responseStream, Encoding.UTF8))
                    {
                        string responseString = sr.ReadToEnd();
                        _log.AddEntry(LogEntryType.Verbose, responseString);
                        _explorer.Clear();
						string[] arr = responseString.Split(new char[] { '\r', '\n' },
							StringSplitOptions.RemoveEmptyEntries);
												
						foreach (string s in arr)
						{
							_explorer.ListNode(parser.Parse(s));

							if (parser.LastError != null)
								_log.AddEntry(LogEntryType.Error, parser.LastError.ToString());
						}
                    }

                    _explorer.FinishListing();
                    _log.AddEntry(LogEntryType.Important, response.StatusDescription);

                    if (!String.IsNullOrEmpty(response.ExitMessage))
                        _log.AddEntry(LogEntryType.Important, response.ExitMessage);

                    currentRequest = null;
                    _isWorking = false;
                }
                catch (Exception ex)
                {
					ExceptionManager.Log(ex);
                    _log.AddEntry(LogEntryType.Error, ex.Message);
                    _explorer.Clear();
                    _explorer.ListError(ex.Message);
					int retryMode = (Int32)Config.Query<FtpManagerOptions>().RetryMode;

                    if (retryCount++ < retryMode)
                    {
                        _log.AddEntry(LogEntryType.Warning, StringTable.GetString("Explorer.Retry"), 
                            retryCount, retryMode);
                        InternalListDirectory(_properties, _log, _explorer);
                    }

                    currentRequest = null;
                    _isWorking = false;
                }                
            }
        }


        [MethodImpl(MethodImplOptions.Synchronized)]
        public void Abort()
        {
            if (currentRequest != null)
                currentRequest.Abort();
        }
        #endregion


        #region Properties
        private bool _isWorking;
        public bool IsWorking
        {
            get { return _isWorking; }
        }


        private ILog _log;
        public ILog Log
        {
            get { return _log; }
        }


        public bool HasParent
        {
            get
            {
                return _properties != null && _properties.Uri != null &&
                    _properties.Uri.LocalPath.Length > 1;
            }
        }


        private IExplorer _explorer;
        public IExplorer Explorer
        {
            get { return _explorer; }
        }
        
       
        private DirectoryProperties _properties;
        public DirectoryProperties Properties
        {
            get { return _properties; }
            set { _properties = value; }
        }


        private static FtpExplorer _instance;
        public static FtpExplorer Instance
        {
            get
            {
                if (_instance == null)
                    lock (typeof(FtpExplorer))
                        if (_instance == null)
                            _instance = new FtpExplorer();

                return _instance;
            }
        }
        #endregion
    }
}
