﻿using System;
using System.IO;

namespace WideStream.FtpManager
{
    internal class ListingParser
    {
        #region Construction
        private Uri currentUri;

        public ListingParser(Uri currentUri)
        {
            this.currentUri = currentUri;
        }
        #endregion


        #region Methods
        public FileNode Parse(string value)
        {
			LastError = null;

			try
			{
				if (value.Length < 15)
					return null;
				else
					return value.StartsWith("-") || value.StartsWith("d") || value.StartsWith("l") ?
						ParseUnixListing(value) : ParseWindowsListing(value);
			}
			catch (Exception ex)
			{
				LastError = ex;
				return null;
			}
        }


        private FileNode ParseUnixListing(string line)
        {
            string source = line;
            string fullPath = currentUri.ToString();
            bool isDir = false; 
            bool isFile = false;
            int node = 0;
            int year = 0;
            string month = String.Empty;
            string day = String.Empty;
            string ext = String.Empty;
            string[] arr = new string[9];

            try
            {
                for (int i = 0; i < 8; i++)
                {
                    arr[i] = source.Substring(0, source.IndexOf(" "));
                    source = source.Remove(0, source.IndexOf(" ")).Trim();                    
                }

                arr[8] = source;
            }
            catch
            {
                source = line;

                for (int i = 0; i < 8; i++)
                {
                    if (i == 3)
                        arr[i] = String.Empty;
                    else
                    {
                        arr[i] = source.Substring(0, source.IndexOf(" "));
                        source = source.Remove(0, source.IndexOf(" "));
                    }

                    source = source.Trim();
                }

                arr[8] = source;
            }

            string permissions = arr[0];

            if (permissions.StartsWith("d")) 
                isDir = true;
            else if (permissions.StartsWith("l"))
                isDir = isFile = true;
            else
            {
                isDir = isFile = false;

                if (source.IndexOf(".") >= 0)
                    ext = source.Substring(source.IndexOf("."));
            }

            node = Int32.Parse(arr[1]);
            string owner = arr[2];
            string group = arr[3];

            long size;
            Int64.TryParse(arr[4], out size);
            month = arr[5];
            day = arr[6];
            int hour = 0;
            int minute = 0;

            if (arr[7].IndexOf(":") == -1) 
                year = int.Parse(arr[7]);
            else
            {
                year = DateTime.Now.Year;
                hour = int.Parse(arr[7].Substring(0, arr[7].IndexOf(":")));
                minute = int.Parse(arr[7].Substring(arr[7].IndexOf(":") + 1));
            }

            string linksTo = String.Empty;
            string fileName = source;

            if (isFile)
            {
				int sindex;

				if ((sindex = source.IndexOf("->")) != -1)
				{
					fileName = source.Substring(0, sindex - 1);
					linksTo = source.Substring(source.IndexOf("->") + 3);
					fullPath = linksTo;
				}
				else
				{
					linksTo = fileName;
					fullPath = Path.Combine(fullPath, fileName);
				}                
            }
            else
                fullPath = Path.Combine(fullPath, fileName);

			if (fileName == "." || fileName == "..")
				return null;
            
            DateTime date = DateTime.Parse(month + " " + day + " " + year, 
                new System.Globalization.CultureInfo("en-US"));
            
            if (date.Month > DateTime.Now.Month) 
                --year;
            
            date = new DateTime(date.Year, date.Month, date.Day, hour, minute, 0);
			Uri nodeUri;

			if (!Uri.TryCreate(fullPath, UriKind.Absolute, out nodeUri))
				return null;

            return new FileNode { FileName = fileName, FileUri = nodeUri, IsDirectory = isDir,
                FileExtension = ext, Size = size, ChangeDate = date };
        }


        private FileNode ParseWindowsListing(string line)
        {
            string source = line;
            string month = source.Substring(0, 2);
            string day = source.Substring(3, 2);
            string year = source.Substring(6, 2);
            string hour = source.Substring(10, 2);
            string min = source.Substring(13, 2);
            string tod = source.Substring(15, 2);
            source = source.Substring(17).Trim();
            long size = 0;
            bool isDir = false;
            
            if (source.StartsWith("<DIR>"))
            {
                isDir = true;
                size = 0;
                source = source.Substring(5).Trim();
            }
            else
            {
                isDir = false;
                Int64.TryParse(source.Split(' ')[0], out size);
                source = source.Substring(source.Split(' ')[0].Length).Trim();
            }
            
            string fileName = source;

            if (year.StartsWith("0")) 
                year = "20" + year;
            else 
                year = "19" + year;

            int hr = 0;
            Int32.TryParse(hour, out hr);
            hr = (hr + (String.Compare(tod, "PM", true) == 0 ? 12 : 0)) == 24 ? 0 : hr;
            string dtStr = month + " " + day + " " + year;
            DateTime dt;
            DateTime.TryParse(dtStr, out dt);
            int dtMin;
            Int32.TryParse(min, out dtMin);
            dt = new DateTime(dt.Year, dt.Month, dt.Day, hr, dtMin, 0);            
            
            string path;
            
            if (TryPathCombine(currentUri.ToString(), fileName, out path))
            {
                Uri fileUri;
                Uri.TryCreate(path, UriKind.RelativeOrAbsolute, out fileUri);
                FileInfo info;

                if (TryParseFileInfo(fileName, out info))
                    return new FileNode
                    {
                        FileName = fileName,
                        FileUri = fileUri,
                        IsDirectory = isDir,
                        FileExtension = info != null ? info.Extension : String.Empty,
                        ChangeDate = dt,
                        Size = size
                    };
            }

            return null;
        }


        private bool TryPathCombine(string dir, string file, out string path)
        {
            path = null;

            try
            {
                path = Path.Combine(dir, file);
                return true;
            }
            catch
            {
                return false;
            }
        }


        private bool TryParseFileInfo(string fileName, out FileInfo info)
        {
            info = null;

            try
            {
                info = new FileInfo(fileName);
                return true;
            }
            catch
            {
                return false;
            }
        }
        #endregion


		#region Properties
		public Exception LastError { get; set; }
		#endregion
	}
}
