﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using Microsoft.Win32;
using WideStream.Configuration;
using System.Windows.Forms;

namespace WideStream.Integration.Opera
{
	public sealed class OperaIntegration : IBrowserIntegration
	{
		#region Construction
		private const string REGISTRY_KEY = @"HKEY_CURRENT_USER\Software\Opera Software";
		private const string KEY = "Last Directory3";
		private const string MENU_ENTRY = "Item, \"Download in WideStream\"=Execute program,\"{0}\",\"%l\"";

		public OperaIntegration()
		{

		}
		#endregion


		#region Methods
		public bool IsInstalled()
		{
			return IsAvailable() && FindMenuEntry(DetermineConfigLocation());
		}


		public bool IsAvailable()
		{
			return OperaPath != null;
		}


		public bool Install()
		{
			Uninstall();
			return InstallDefaults();
		}


		private FileInfo GetDefaultFile()
		{
			return new FileInfo(Path.Combine(OperaPath, "defaults\\standard_menu.ini"));
		}


		private bool InstallDefaults()
		{
			FileInfo fi = DetermineConfigLocation();

			if (fi == null || !fi.Exists)
				return false;

			InsertMenuEntry(fi.FullName);
			return true;
		}


		private FileInfo DetermineConfigLocation()
		{
			FileInfo conf = new FileInfo(Path.Combine(Environment.GetFolderPath(Environment.
				SpecialFolder.ApplicationData), @"Opera\Opera\profile\opera6.ini"));

			if (!conf.Exists)
				conf = new FileInfo(Path.Combine(OperaPath, @"profile\opera6.ini"));

			FileInfo fi = !conf.Exists ? GetDefaultFile() : ScanConfig(conf.FullName);

			return fi;			
		}


		private FileInfo ScanConfig(string config)
		{
			using (StreamReader sr = File.OpenText(config))
			{
				string line;

				while ((line = sr.ReadLine()) != null)
				{
					if (line.ToLower().StartsWith("menu configuration="))
						return new FileInfo(line.ToLower().Replace("menu configuration=", String.Empty));
				}

				return null;
			}
		}


		private void InsertMenuEntry(string fileName)
		{
			List<String> lines = new List<string>();
			int? firstIndex = null;
			int? lastIndex = null;

			using (StreamReader sr = File.OpenText(fileName))
			{
				string line;

				while ((line = sr.ReadLine()) != null)
				{
					lines.Add(line);

					if (firstIndex == null && line.ToLower().StartsWith("[link popup menu]"))
						firstIndex = lines.Count - 1;
					else if (firstIndex != null && lastIndex == null && line.StartsWith("["))
						lastIndex = lines.Count - 1;
				}
			}

			if (lastIndex != null)
				lines.Insert(lastIndex.Value - 1, String.Format(MENU_ENTRY, Application.ExecutablePath));

			Write(lines, fileName);
		}


		private void Write(List<String> lines, string fileName)
		{
			using (StreamWriter sw = new StreamWriter(File.Create(fileName)))
			{
				foreach (string l in lines)
					sw.WriteLine(l);
			}
		}


		private bool FindMenuEntry(FileInfo fi)
		{
			if (fi == null)
				return false;

			using (StreamReader sr = File.OpenText(fi.FullName))
			{
				string line;
				while ((line = sr.ReadLine()) != null)
				{
					if (line.StartsWith("Item, \"Download in WideStream\""))
						return true;
				}
			}

			return false;
		}


		public bool Uninstall()
		{
			FileInfo fi = DetermineConfigLocation();

			if (fi == null || !fi.Exists)
				return false;

			int? index = null;
			List<String> lines = new List<string>();

			using (StreamReader sr = File.OpenText(fi.FullName))
			{
				string line;
				while ((line = sr.ReadLine()) != null)
				{
					lines.Add(line);

					if (line.StartsWith("Item, \"Download in WideStream\""))
						index = lines.Count - 1;
				}
			}

			if (index != null)
				lines.RemoveAt(index.Value);
			else
				return false;

			Write(lines, fi.FullName);
			return true;
		}


		public bool Repair()
		{
			return Install();
		}
		#endregion


		#region Properties
		internal string OperaPath
		{
			get { return Registry.GetValue(REGISTRY_KEY, KEY, null) as String; }
		}
		#endregion
	}
}
