﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Windows.Forms;
using WideStream.Configuration;
using WideStream.Logging.Configuration;
using WideStream.Workspace.Forms;
using System.IO;
using WideStream.Startup.Resources;

namespace WideStream.Logging
{
    public sealed class GridLog : ILog
    {
        #region Construction
        private DataGridView grid;
        private List<DataGridViewRow> rows;
        private int messageCount;

        public GridLog(DataGridView grid)
        {
            this.grid = grid;
            this.rows = new List<DataGridViewRow>();
        }
        #endregion


        #region Methods
        public void AddEntry(LogEntryType type, string message, params object[] args)
        {
			LogEntry entry = new LogEntry(type, String.Format(message, args));
			AddEntry(entry);
        }


      	public void AddEntry(LogEntry entry)
		{
            string message = String.IsNullOrEmpty(entry.Header) ? entry.Message :
				entry.Header + "\r\n" + entry.Message;
			LogOptions conf = Config.Query<LogOptions>();
			LoggingMode mode = conf.LoggingMode;

			if ((mode == LoggingMode.Normal && entry.Type == LogEntryType.Verbose) ||
				(mode == LoggingMode.Minimal && (entry.Type == LogEntryType.Information || entry.Type == LogEntryType.Verbose)))
				return;

			bool first = true;
			string[] arr = message.Split(new char[] { '\r', '\n' }, StringSplitOptions.RemoveEmptyEntries);

			for (int i = 0; i < (arr.Length > 50 ? 50 : arr.Length); i++)
			{
				bool lastElement = (arr.Length > 50 ? 50 : arr.Length) - 1 == i;
				string line = arr[i];
				DataGridViewRow tr = new DataGridViewRow();
				Color color =
						entry.Type == LogEntryType.Error ? conf.ErrorColor :
						entry.Type == LogEntryType.Warning ? conf.WarningColor :
						entry.Type == LogEntryType.Verbose ? conf.VerboseColor :
						entry.Type == LogEntryType.Important ? conf.ImportantColor :
						entry.Type == LogEntryType.Success ? conf.SuccessColor :
						conf.InformationColor;
				tr.Cells.Add(CreateCell(color, first ? (++messageCount).ToString() : String.Empty,
					arr.Length > 1, 0, lastElement, i == 0, !String.IsNullOrEmpty(entry.Header)));
				tr.Cells.Add(CreateCell(color, first ? DateTime.Now.ToString(StringTable.GetString("General.DateFormat")) : String.Empty,
					arr.Length > 1, 1, lastElement, i == 0, !String.IsNullOrEmpty(entry.Header)));
				tr.Cells.Add(CreateCell(color, first ? DateTime.Now.ToLongTimeString() : String.Empty,
					arr.Length > 1, 2, lastElement, i == 0, !String.IsNullOrEmpty(entry.Header)));
				tr.Cells.Add(CreateCell(color, line,
					arr.Length > 1, 3, lastElement, i == 0, !String.IsNullOrEmpty(entry.Header)));
                rows.Add(tr);

                if (rows.Count > 300)
                    rows.RemoveAt(0);

				SafeInvoke(() =>
					{
						if (Active && !grid.Rows.Contains(tr))
						{
							grid.Rows.Add(tr);

							if (conf.ScrollRule == LogScroll.Always ||
								conf.ScrollRule == LogScroll.WhenEntryAdded)
								grid.FirstDisplayedScrollingRowIndex = grid.Rows.Count - 1;
						}
					}
				);

				first = false;
			}
		}


		public void ScrollToEnd()
		{
			SafeInvoke(() =>
				{
					if (grid.Rows.Count > 0)
						grid.FirstDisplayedScrollingRowIndex = grid.Rows.Count - 1;				
				}
			);
		}


        public void Clear()
        {
			SafeInvoke(() =>
				{
					rows.Clear();

					if (!grid.IsDisposed)
						grid.Rows.Clear();

					messageCount = 0;
				}
			);
        }


        public void Copy()
        {
			SafeInvoke(() =>
				{
					if (!grid.IsDisposed && grid.SelectedRows.Count > 0)
					{
						StringBuilder sb = new StringBuilder(grid.SelectedRows.Count * 500);

						foreach (DataGridViewRow tr in grid.SelectedRows.OfType<DataGridViewRow>().Reverse())
							sb.AppendLine(ReadRow(tr));

						Clipboard.SetText(sb.ToString());
					}
				}
			);
        }


        public void CopyAll()
        {
			SafeInvoke(() =>
			   {
				   if (!grid.IsDisposed)
				   {
					   StringBuilder sb = new StringBuilder(grid.Rows.Count * 500);

					   foreach (DataGridViewRow tr in grid.Rows)
						   sb.AppendLine(ReadRow(tr));

					   Clipboard.SetText(sb.ToString());
				   }
			   }
			);
        }


        private string ReadRow(DataGridViewRow row)
        {
            return String.Format("{0}\t{1}\t{2}\t{3}",
                row.Cells[0].Value as String,
                row.Cells[1].Value as String,
                row.Cells[2].Value as String,
                row.Cells[3].Value as String);
        }


		private DataGridViewBorderCell CreateCell(Color color, string message, bool paint, int cellIndex, 
			bool last, bool first, bool hasHeader)
        {
			LogOptions conf = Config.Query<LogOptions>();
			DataGridViewBorderCell td = new DataGridViewBorderCell();
            td.Value = paint && !first && cellIndex == 3 && hasHeader ? "    " + message : message;

			if (paint)
			{
				td.Style.BackColor = conf.MultilineBackgroundColor;
				td.Top = DataGridViewAdvancedCellBorderStyle.None;
				td.Left = cellIndex == 0 ? DataGridViewAdvancedCellBorderStyle.Outset :
					DataGridViewAdvancedCellBorderStyle.None;
				td.Right = DataGridViewAdvancedCellBorderStyle.Outset;
				td.Bottom = last ? DataGridViewAdvancedCellBorderStyle.Outset :
					DataGridViewAdvancedCellBorderStyle.None;

				if (first && cellIndex == 3 && hasHeader)
					td.Style.Font = new Font(grid.DefaultCellStyle.Font, FontStyle.Italic);
			}
			else
			{
				td.Style.BackColor = conf.BackgroundColor;
				td.Top = td.Bottom = td.Right = td.Left = DataGridViewAdvancedCellBorderStyle.Outset;
			}
			
            td.Style.ForeColor = color;
            td.Style.Alignment = DataGridViewContentAlignment.TopLeft;
            td.Style.WrapMode = DataGridViewTriState.False;
			return td;
        }


        private void FillGrid()
        {
			SafeInvoke(() => 
				{
					grid.Rows.Clear();

					if (rows.Count > 0)
						foreach (DataGridViewRow tr in rows)
							grid.Rows.Add(tr);
				 }
			);
        }


		public void Save(LogSaveType type, FileInfo file)
		{
            SaveHelper helper = new SaveHelper(file, rows);
            SafeInvoke(() => helper.Save(type));
		}


		private void SafeInvoke(MethodInvoker func)
		{
			if (grid.IsHandleCreated && !grid.IsDisposed)
			{
				try
				{
					grid.Invoke(func);
				}
				catch (ArgumentException)
				{

				}
				catch (InvalidOperationException)
				{

				}
			}
		}
        #endregion


        #region Properties
        private bool _active;
        public bool Active
        {
            get { return _active; }
            set 
            { 
                _active = value;

                if (value)
                    FillGrid();
            }
        }
        #endregion
    }
}
