﻿using System;
using System.IO;
using System.Security;
using System.Windows.Forms;
using System.Xml;
using WideStream.Configuration;
using WideStream.DownloadManager.Configuration;
using WideStream.FtpManager.Configuration;
using WideStream.Integration;
using WideStream.Logging.Configuration;
using WideStream.Startup.CommandLine;
using WideStream.Workspace;
using WideStream.Workspace.ComponentModel;
using WideStream.Workspace.Configuration;
using Microsoft.VisualBasic.ApplicationServices;
using WideStream.Startup.ExceptionHandling;

namespace WideStream.Startup
{
	public sealed class ApplicationEnvironment : ApplicationContext
	{
		#region Construction
        
		public ApplicationEnvironment(Main form, CommandLineProperties startupProperties)
		{
			MainForm = form;
            StartupProperties = startupProperties;
			Initialize(!startupProperties.Minimize);
		}
		#endregion


		#region Methods
		private void Initialize(bool showForm)
        {
            ApplicationRegistry.SetStartupPath();
            AppDomain.CurrentDomain.UnhandledException += (o, e) => ProcessError(e.ExceptionObject as Exception);

			Config.Register<WorkspaceOptions>();
			Config.Register<RecoveryOptions>();
			Config.Register<LogOptions>();
			Config.Register<IntegrationOptions>();            
			Config.Register<DownloadManagerDisplayOptions>();
			Config.Register<NotificationOptions>();
			Config.Register<ActiveDownloadsOptions>();
			Config.Register<QueueManagerOptions>();
			Config.Register<DownloadProcessOptions>();
                        
            ProcessManifest(typeof(ApplicationEnvironment).Assembly.GetManifestResourceStream(
                "WideStream.Startup.Resources.Manifest.xml"));
            FileInfo fi = new FileInfo(Path.Combine(Application.StartupPath, "manifest.xml"));

            if (fi.Exists)
            {
                using (Stream stream = File.OpenRead(fi.FullName))
                    ProcessManifest(stream);
            }

            Config.Register<FtpManagerOptions>();
            Config.Register<ExplorerHistoryOptions>();
            Config.Register<HostManagerOptions>();
            Config.Initialize();
            MainForm.Initialize();
            MainForm.LoadPage(Page.DownloadManager);

            if (!String.IsNullOrEmpty(StartupProperties.ErrorLog))
            {
                FileInfo configPath = null;

                try
                {
                    configPath = StartupProperties.ErrorLog.IndexOf("\\") == -1 ?
                        new FileInfo(Path.Combine(Config.ConfigPath, StartupProperties.ErrorLog.Trim('"'))) :
                        new FileInfo(StartupProperties.ErrorLog.Trim('"'));
                }
                catch (Exception ex)
                {
                    if (!(ex is SecurityException) &&
                        !(ex is ArgumentException) &&
                        !(ex is UnauthorizedAccessException) &&
                        !(ex is PathTooLongException) &&
                        !(ex is NotSupportedException))
                        throw ex;
                }

                if (configPath == null)
                    configPath = new FileInfo(Path.Combine(Config.ConfigPath, "ErrorLog.txt"));

			    ExceptionManager.ErrorLogPath = configPath;
            }

            bool hide = !showForm && Config.Query<WorkspaceOptions>().MinimizedState != MinimizedState.HideIcon;

            if (hide)
                MainForm.Opacity = .0D;
						
			MainForm.Show();

            if (hide)
                MainForm.Hide();
        }


        private void ProcessManifest(Stream stream)
        {
            using (XmlReader reader = XmlReader.Create(stream))
            {
                ManifestReader manReader = new ManifestReader();
                manReader.Read(reader);
            }
        }


        private bool ProcessError(Exception ex)
		{
			if (ex != null)
				ExceptionManager.Process(ex, null);

            return false;
		}
		#endregion


		#region Properties
		public new Main MainForm { get; private set; }

        public CommandLineProperties StartupProperties { get; private set; }
		#endregion
	}
}
