﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Reflection;

namespace WideStream.Startup.CommandLine
{
	/// <summary>
	/// Represents a class, that provides basic functionality to 
	/// parse command line arguments.
	/// </summary>
	/// <remars>A single instance of this class can be used to parse 
	/// unlimited number of command lines.</remars>
	public sealed class CommandLineParser
	{
		#region Construction
		/// <summary>
		/// Initializes a new instance of a class.
		/// </summary>
		public CommandLineParser()
		{
			Errors = new ErrorList();
		}
		#endregion


		#region Methods
		public T Parse<T>(string[] args) where T : class, new()
		{
			T ret = new T();
			PropertyInfo defaultProperty;
			Dictionary<String,PropertyInfo> properties = ProcessType(typeof(T), out defaultProperty);
			PropertyInfo currentProperty = null;

			foreach (string s in args)
			{
				string val = s.Trim('\"', ' ');

				if (val.Length == 0)
					continue;

				if (val[0] == '/' || val[0] == '-')
				{
					if (currentProperty != null)
					{
						Errors.Add(new SwitchError(val, SwitchErrorType.MissingValue));
						currentProperty = null;
						continue;
					}

					PropertyInfo pi;

					if (properties.TryGetValue(val.TrimStart('-', '/'), out pi))
					{
						if (pi.PropertyType == typeof(Boolean))
							pi.SetValue(ret, true, null);
						else
							currentProperty = pi;

						FoundSwitches = true;
					}
					else
					{
						Errors.Add(new SwitchError(val, SwitchErrorType.UnrecognizedSwitch));
					}
				}
				else
				{
					currentProperty = currentProperty ?? defaultProperty;

					if (currentProperty != null)
					{
						object convVal = null;

						try
						{
							convVal = Convert.ChangeType(val, currentProperty.PropertyType);
						}
						catch
						{
							Errors.Add(new SwitchError(val, SwitchErrorType.InvalidArgumentType));
							currentProperty = null;
							continue;
						}

						currentProperty.SetValue(ret, convVal, null);
						currentProperty = null;
					}
					else
					{
						Errors.Add(new SwitchError(val, SwitchErrorType.MissingSwitch));
					}
				}
			}

			return ret;
		}


		private Dictionary<String,PropertyInfo> ProcessType(Type type, out PropertyInfo defaultProperty)
		{
			Dictionary<String,PropertyInfo> dict = new Dictionary<String,PropertyInfo>();
			defaultProperty = null;

			foreach (PropertyInfo pi in type.GetProperties())
			{
				CommandLineAttribute attr = Attribute.GetCustomAttribute(pi, typeof(CommandLineAttribute))
					as CommandLineAttribute;

				if (attr != null)
					dict.Add(attr.Key.ToLower(), pi);

				if (attr.Default)
					defaultProperty = pi;
			}

			return dict;
		}
		#endregion


		#region Properties
		/// <summary>
		/// Gets a collection of errors occured during the parsing of a command line.
		/// </summary>
		public ErrorList Errors { get; private set; }

		public bool FoundSwitches { get; private set; }
		#endregion
	}
}
