﻿using System;
using System.Collections.Generic;
using System.Xml;
using System.Windows.Forms;
using System.IO;
using System.Threading;
using WideStream.Configuration;
using WideStream.Workspace.Configuration;

namespace WideStream.Startup.Resources
{
    public static class StringTable
    {
        #region Construction
        private static Dictionary<String,String> strings;
        private static Dictionary<String,String> defaultStrings;
        private static object syncRoot = new Object();
        #endregion


        #region Methods
        public static string GetString(string key)
        {
            InitializeDictionary();
            string retval;            
            retval = !strings.TryGetValue(key.TrimStart('$'), out retval) ? key : retval;

            if (String.IsNullOrEmpty(retval))
            {
                ReadDefaultStrings();
                return !defaultStrings.TryGetValue(key.TrimStart('$'), out retval) ? key : retval;
            }

            return retval;
        }


        private static void InitializeDictionary()
        {
            if (strings == null)
                lock (syncRoot)
                    if (strings == null)
                    {
                        strings = new Dictionary<String, String>();

                        if (!ReadFromResource(Language))
                        {
                            FileInfo fi = GetFileForCulture(Language);

                            if (fi.Exists)
                                using (XmlReader xr = XmlReader.Create(fi.Open(FileMode.Open)))
                                    Read(xr, strings);
                            else
                                ReadFromResource("en-US");
                        }
                    }
        }


        private static bool ReadFromResource(string resName)
        {
            var stream = typeof(StringTable).Assembly.GetManifestResourceStream(
                String.Format("WideStream.Startup.Resources.Translations.{0}.wsresx", resName));

            if (stream != null)
                using (XmlReader xr = XmlReader.Create(stream))
                    Read(xr, strings);

            return stream != null;
        }


        private static void ReadDefaultStrings()
        {
            if (defaultStrings == null)
                lock (syncRoot)
                    if (defaultStrings == null)
                    {
                        defaultStrings = new Dictionary<String,String>();

                        using (XmlReader xr = XmlReader.Create(typeof(StringTable).Assembly.
                            GetManifestResourceStream("WideStream.Startup.Resources.Translations.en-US.wsresx")))
                            Read(xr, defaultStrings);
                    }
        }


        private static void Read(XmlReader xr, Dictionary<String,String> dict)
        {
            while (xr.Read())
            {
                if (xr.IsStartElement("String"))
                    dict.Add(xr.GetAttribute("Key"), xr.GetAttribute("Value"));
            }
        }


        private static FileInfo GetFileForCulture(string name)
        {
            FileInfo ret = new FileInfo(Path.Combine(Application.StartupPath, name + ".wsresx"));
            return !ret.Exists ? new FileInfo(Path.Combine(Application.StartupPath, 
                Path.Combine("Resources", name + ".wsresx"))) : ret;
        }
        #endregion


        #region Properties
        private static string _language;
        public static string Language
        {
            get
            {
                if (_language == null)
                    lock (typeof(StringTable))
                        if (_language == null)
                        {
                            _language = Config.Query<WorkspaceOptions>().Language;
                            _language = _language == "neutral" ? Thread.CurrentThread.
                                CurrentUICulture.IetfLanguageTag : _language;
                        }

                return _language;
            }
        }
        #endregion
    }
}
