﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Windows.Forms;
using WideStream.Configuration;
using WideStream.Configuration.ComponentModel;
using WideStream.DownloadManager;
using WideStream.DownloadManager.Configuration;
using WideStream.DownloadManager.Presentation;
using WideStream.FtpManager.Presentation;
using WideStream.Workspace.ComponentModel;
using WideStream.Workspace.Configuration;
using WideStream.HostManager.Presentation;
using WideStream.DownloadManager.Network.Protocols.Http;
using WideStream.DownloadManager.Network.Protocols.Ftp;
using WideStream.Configuration.Presentation;
using WideStream.Logging;
using System.IO;
using WideStream.Startup.Resources;
using WideStream.Workspace.State;

namespace WideStream.Workspace
{
    public partial class Main : Form, IControl
    {
        #region Construction
        private ItemStateManager stateManager;
        
        public Main()
        {
            InitializeComponent();
            _pages = new Dictionary<Page,IPage>();
        }
        #endregion


        #region Methods
		public void Initialize()
		{
            LocalizationManager.Localize(this);
            ThemeManager.ApplyTheme(this);
			stateManager = new ItemStateManager(this);		
		}


        public IPage ObtainPageIfLoaded(Page pageType)
        {
            IPage ret;

            if (Pages.TryGetValue(pageType, out ret))
                return ret;
            else
                return null;
        }


        public IPage ChangeCurrentPage(Page pageType)
        {
			splitContainer.Panel1.Controls.Clear();
            logView.Rows.Clear();

            if (CurrentPage != null)
            {
                ToolStripManager.RevertMerge(menuStrip);
                ToolStripManager.RevertMerge(toolStrip);

				if (CurrentPage.ActiveLog != null)
					CurrentPage.ActiveLog.Active = false;
            }

            CurrentPage = null;

			IPage page = InternalLoadPage(pageType);                
			splitContainer.Panel1.Controls.Add((Control)page);
               
            if (page != null)
            {
                CurrentPage = page;
                string pageName = Attribute.GetCustomAttribute(typeof(Page).GetField(pageType.ToString()),
                    typeof(EnumNameAttribute)).ToString();
                ToolStripManager.Merge(CurrentPage.MainMenu, menuStrip);

				if (CurrentPage.ToolBar != null)
					ToolStripManager.Merge(CurrentPage.ToolBar, toolStrip);
                
                Text = String.Concat(StringTable.GetString("MainForm.Title"), " - ", StringTable.GetString(pageName));
                statusPanel1.Text = StringTable.GetString(pageName);
                CurrentPage.Open();
            }

			return CurrentPage;
        }


		public void DisplayProgressBar(int interval)
		{
			if (Config.Query<WorkspaceOptions>().ShowSaveProgressBar)
			{
				ProgressBarVisible = true;
				System.Windows.Forms.Timer timer = new System.Windows.Forms.Timer();
				timer.Interval = interval;
				timer.Enabled = true;
				timer.Tick += delegate
				{
					ProgressBarVisible = false;
					timer.Stop();
					timer.Dispose();
				};
				timer.Start();
			}
		}



		public IPage LoadPage(Page pageType)
		{
			return InternalLoadPage(pageType);
		}


		private IPage InternalLoadPage(Page pageType)
		{
			IPage page;

			if (!Pages.TryGetValue(pageType, out page))
			{
				if (pageType == Page.DownloadManager)
					page = new DownloadPage();
				else if (pageType == Page.FtpExplorer)
					page = new FtpManagerPage();
				else if (pageType == Page.HostManager)
					page = new HostManagerPage();

				if (page != null)
					Pages.Add(pageType, page);

				EnvironmentViewState.LoadState(page);
				page.Initialize(this, logView);
			}

			return page;
		}

              
        private void downloadLogToolStripMenuItem_Click(object sender, EventArgs e)
        {
			Config.Query<WorkspaceOptions>().LogVisible = !Config.Query<WorkspaceOptions>().LogVisible;
        }


        private void statusBarToolStripMenuItem_Click(object sender, EventArgs e)
        {
			Config.Query<WorkspaceOptions>().StatusBarVisible = !Config.Query<WorkspaceOptions>().StatusBarVisible;
        }


        private void optionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ShowOptionsDialog(null);   
        }


        internal void ShowOptionsDialog(string page)
        {
            ConfigurationDialog d = new ConfigurationDialog();
            d.DisplayCategory = page;
            d.ShowDialog(this);

            if (CurrentPage is FtpManagerPage)
                ((FtpManagerPage)CurrentPage).RefreshHistory();
            else if (CurrentPage is DownloadPage)
            {
                ((DownloadPage)CurrentPage).DisplayControl.Refresh();
            }
        }


        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutDialog d = new AboutDialog();
            d.ShowDialog(this);
        }



        private void Main_FormClosing(object sender, FormClosingEventArgs e)
        {
            EnvironmentViewState.SaveState(this);

            foreach (IPage p in _pages.Values)
                if (!p.CanClose())
                {
                    e.Cancel = true;
                    return;
                }

			ApplicationRegistry.SetValue("WideStream", new Version(Application.ProductVersion).ToString(3));
            ActiveDownloadsManager.HideActiveDownloadsWindow();
			QueueManager.Instance.Save();
			Config.Save();
			notifyIcon.Dispose();
			Environment.Exit(0);
        }


        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }


        private void downloadManagerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ChangeCurrentPage(Page.DownloadManager);
        }


        private void fTPManagerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ChangeCurrentPage(Page.FtpExplorer);
        }


        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (CurrentPage != null && CurrentPage.ActiveLog != null)
                CurrentPage.ActiveLog.Copy();
        }


        private void copyAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (CurrentPage != null && CurrentPage.ActiveLog != null)
                CurrentPage.ActiveLog.CopyAll();
        }


        private void clearToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (CurrentPage != null && CurrentPage.ActiveLog != null)
                CurrentPage.ActiveLog.Clear();
        }


        private void toolBarToolStripMenuItem_Click(object sender, EventArgs e)
        {
			Config.Query<WorkspaceOptions>().ToolbarVisible = !Config.Query<WorkspaceOptions>().ToolbarVisible;
        }


        private void Main_Load(object sender, EventArgs e)
        {
			Location = new Point((Screen.PrimaryScreen.WorkingArea.Width - Width) / 2,
				(Screen.PrimaryScreen.WorkingArea.Height - Height) / 2);
            saveLogDialog.Filter = StringTable.GetString("Log.SaveLogFilter");
            saveLogDialog.Title = StringTable.GetString("Log.SaveLog");
            Text = StringTable.GetString(Text);
            EnvironmentViewState.LoadState(this);
        }
        #endregion


        #region Properties
        public bool UrlStartup { get; set; }

        public ToolStripContainer ToolContainer
        {
            get { return toolStripContainer1; }
        }

        public StatusStrip StatusBar
        {
            get { return statusBar; }
        }


        public ToolStrip ToolBar
        {
            get { return toolStrip; }
        }


        public new ContextMenuStrip ContextMenu
        {
            get { return logMenu; }
        }


        public ContextMenuStrip ColumnsMenu
        {
            get { return trayMenu; }
        }


        public DataGridView DisplayControl
        {
            get { return logView; }
        }



        public bool StatusBarVisible
        {
            get { return statusBar.Visible; }
            set { statusBar.Visible = value; }
        }

        public string StatusText
        {
            get { return statusPanel2.Text; }
            set { statusPanel2.Text = value; }
        }


        public bool LogVisible
        {
            get { return !splitContainer.Panel2Collapsed && CurrentPage != null && CurrentPage.ActiveLog != null; }
            set 
			{ 
				if (CurrentPage != null && CurrentPage.PageType != Page.HostManager)
					splitContainer.Panel2Collapsed = !value; 
				else
					splitContainer.Panel2Collapsed = true; 
			}
        }


        public string TrayIconToolTip
        {
            get { return notifyIcon.BalloonTipText; }
            set { notifyIcon.BalloonTipText = value; }
        }


        public bool ProgressBarVisible
        {
            get { return statusProgressBar.Visible; }
            set { statusProgressBar.Visible = value; }
        }


        private Dictionary<Page,IPage> _pages;
        public Dictionary<Page,IPage> Pages
        {
            get { return _pages; }
        }


        public IPage CurrentPage { get; private set; }


        public ContextMenuStrip LogMenu
        {
            get { return logMenu; }
        }


        public ContextMenuStrip TrayMenu
        {
            get { return trayMenu; }
        }



        public DataGridView LogView
        {
            get { return logView; }
        }


        public MenuStrip MainMenu
        {
            get { return menuStrip; }
        }


        public bool TrayIconVisible
        {
            get { return notifyIcon.Visible; }
            set { notifyIcon.Visible = value; }
        }


        public int SelectedLogEntries
        {
            get { return logView.SelectedRows.Count; }
        }


        public int LogEntries
        {
            get { return logView.Rows.Count; }
        }


        public int SplitterDistance
        {
            get { return splitContainer.SplitterDistance; }
            set
            {
                try
                {
                    splitContainer.SplitterDistance = value;
                }
                catch { }
            }
        }
        #endregion        

        private void trayMenu_Opening(object sender, CancelEventArgs e)
        {
			QueueManagerOptions conf = Config.Query<QueueManagerOptions>();
            trafficUsageToolStripMenuItem.DropDownItems.Clear();

            foreach (int v in Enum.GetValues(typeof(TrafficUsage)))
            {
                TrafficUsage tu = (TrafficUsage)v;
                ToolStripMenuItem mi = new ToolStripMenuItem(tu.ToDisplayString());
                mi.Checked = tu == conf.TrafficUsage;
                mi.Click += delegate { conf.TrafficUsage = tu; };
                trafficUsageToolStripMenuItem.DropDownItems.Add(mi);                
            }

            hideWideStreamToolStripMenuItem.Checked = Config.Query<WorkspaceOptions>().MinimizedState == MinimizedState.ShowIconHide;
            suspendQueueToolStripMenuItem.Checked = conf.SuspendQueue;
        }

        private void showWideStreamToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (!Visible)
            {
                Show();
                WindowState = FormWindowState.Normal;
            }
            else
                Hide();
        }

        private void hideWideStreamToolStripMenuItem_Click(object sender, EventArgs e)
        {
			if (Config.Query<WorkspaceOptions>().MinimizedState == MinimizedState.ShowIconHide)
				Config.Query<WorkspaceOptions>().MinimizedState = MinimizedState.ShowIconDontHide;
            else
				Config.Query<WorkspaceOptions>().MinimizedState = MinimizedState.ShowIconHide;
        }


        private void addDownloadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DownloadPage page = (DownloadPage)ObtainPageIfLoaded(Page.DownloadManager);
            page.AddDownloads();
        }

        private void resumeAllDownloadsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DownloadPage page = (DownloadPage)ObtainPageIfLoaded(Page.DownloadManager);
            page.ResumeAllDownloads();
        }

        private void pauseAllDownloadsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DownloadPage page = (DownloadPage)ObtainPageIfLoaded(Page.DownloadManager);
            page.PauseAllDownloads();
        }

        private void suspendQueueToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DownloadPage page = (DownloadPage)ObtainPageIfLoaded(Page.DownloadManager);
            page.SuspendQueue();
        }

        private void Main_SizeChanged(object sender, EventArgs e)
        {
			if (WindowState == FormWindowState.Minimized && Config.Query<WorkspaceOptions>().MinimizedState == MinimizedState.ShowIconHide)
                Hide();
        }

        private void exitToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            Show();
            Close();
        }

        private void notifyIcon_MouseMove(object sender, MouseEventArgs e)
        {
            
        }

        private void notifyIcon_MouseDown(object sender, MouseEventArgs e)
        {
            
        }

        private void Main_Shown(object sender, EventArgs e)
        {
			WorkspaceOptions conf = Config.Query<WorkspaceOptions>();

            if (WindowState == FormWindowState.Minimized && 
				(Config.Query<RecoveryOptions>().WindowStateSave == WindowStateSaveOption.Always ||
				Config.Query<RecoveryOptions>().WindowStateSave == WindowStateSaveOption.OnlyMinimized) &&
                conf.MinimizedState == MinimizedState.ShowIconHide)
                Hide();

			if (UrlStartup && Config.Query<WorkspaceOptions>().MinimizedState == MinimizedState.ShowIconHide)
                Hide();
        }

        private void shareOurOpinionAboutAProductToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenLink("http://widestream.sourceforge.net/reviews.php?ref=" + Application.ProductVersion);
        }

        private void documentationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenLink("http://widestream.sourceforge.net/docs.php?ref=" + Application.ProductVersion);
        }

        private void wideStreamSiteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenLink("http://widestream.sourceforge.net/?ref=" + Application.ProductVersion);
        }

        private void readProjectDevelopersBlogToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenLink("http://widestream.sourceforge.net/blog.php?ref=" + Application.ProductVersion);
        }

        private void nextVersionWorkInProgressToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenLink("http://widestream.sourceforge.net/worklog.php?ref=" + Application.ProductVersion);
        }

      
        private void notifyIcon_DoubleClick(object sender, EventArgs e)
        {
            DisplayForm();
        }


        public void DisplayForm()
        {
            if (!Visible)
            {
                Show();
                WindowState = FormWindowState.Normal;
                ChangeCurrentPage(CurrentPage.PageType);
                Opacity = 1.0D;
                Height++;
                Height--;
            }
        }

		private void reportABugToolStripMenuItem_Click(object sender, EventArgs e)
		{
            OpenLink("https://sourceforge.net/tracker2/?func=add&group_id=231599&atid=1083647");
		}

		private void requestAfeatureToolStripMenuItem_Click(object sender, EventArgs e)
		{
            OpenLink("https://sourceforge.net/tracker2/?func=add&group_id=231599&atid=1083650");
		}

		private void requestsupportToolStripMenuItem_Click(object sender, EventArgs e)
		{
            OpenLink("https://sourceforge.net/tracker2/?func=add&group_id=231599&atid=1083648");
		}

		private void hostManagerToolStripMenuItem1_Click(object sender, EventArgs e)
		{
			ChangeCurrentPage(Page.HostManager);
		}

		private void activeDownloadsWindowToolStripMenuItem_Click(object sender, EventArgs e)
		{
			ActiveDownloadsManager.ShowActiveDownloadsWindow();
		}


        private void OpenLink(string link)
        {
            try
            {
                Process.Start(link);
            }
            catch (Exception)
            {
                Process.Start(new ProcessStartInfo("iexplorer", link));
            }
        }

        private void addinsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AddinsDialog dlg = new AddinsDialog();
            dlg.ShowDialog(this);
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ILog log = null;

            if (CurrentPage != null && (log = CurrentPage.ActiveLog) != null && 
                saveLogDialog.ShowDialog(this) == DialogResult.OK)
            {
                FileInfo fi = new FileInfo(saveLogDialog.FileName);
                log.Save(DetermineSaveType(fi.Extension), fi);
            }
        }


        private LogSaveType DetermineSaveType(string fileExt)
        {
            if (String.Compare(fileExt, ".htm", true) == 0 ||
                String.Compare(fileExt, ".html", true) == 0)
                return LogSaveType.Html;
            else if (String.Compare(fileExt, ".tab", true) == 0 ||
                String.Compare(fileExt, ".txt", true) == 0)
                return LogSaveType.TabDelimited;
            else if (String.Compare(fileExt, ".csv", true) == 0 ||
                String.Compare(fileExt, ".xls", true) == 0)
                return LogSaveType.CommaDelimited;
            else
                return LogSaveType.Default;
        }

   
    }
}
