﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Windows.Forms;
using WideStream.Workspace.ComponentModel;
using WideStream.Workspace.Forms;

namespace WideStream.Workspace.State
{
    public sealed class ConditionProcessor
    {
        #region Construction
        private static int[] conditionValues;

        static ConditionProcessor()
        {
            conditionValues = (int[])Enum.GetValues(typeof(EnableConditions));
            Functors = new Dictionary<EnableConditions,ConditionCallBack>();
            FillFunctors();
        }

        public ConditionProcessor(IPage page)
        {
            Cache = new Dictionary<EnableConditions,State>();
            Page = page;
        }
        #endregion


        #region Methods
        private static void FillFunctors()
        {
            ProcessClass(typeof(VisibleElementsState), EnableConditions.ElementVisible);
            ProcessClass(typeof(LogState), EnableConditions.LogPanel);
            ProcessClass(typeof(DownloadManagerState), EnableConditions.DownloadManager);
            ProcessClass(typeof(FtpManagerState), EnableConditions.FtpManager);
            ProcessClass(typeof(HostManagerState), EnableConditions.HostManager);
        }


        private static void ProcessClass(Type type, EnableConditions category)
        {
            foreach (var mi in type.GetMethods(BindingFlags.Public | BindingFlags.Static))
            {
                var attr = Attribute.GetCustomAttribute(mi, typeof(EnableConditionAttribute))
                    as EnableConditionAttribute;

                if (attr != null)
                {
                    Functors.Add(category | attr.Conditions, (ConditionCallBack)Delegate.
                        CreateDelegate(typeof(ConditionCallBack), mi));
                }
            }
        }


        public void ProcessItems(ToolStripItemCollection list)
        {
            foreach (ToolStripItem i in list)
            {
                if (i is ToolStripMenuItem && ((ToolStripMenuItem)i).DropDownItems.Count > 0)
                    ProcessItems(((ToolStripMenuItem)i).DropDownItems);
                else if (i.Tag != null && !String.IsNullOrEmpty(i.Tag as String))
                    TestCondition(i);
            }
        }


        private void TestCondition(ToolStripItem item)
        {
            EnableConditions con = (EnableConditions)Enum.Parse(
                typeof(EnableConditions), (String)item.Tag);
            EnableConditions category = default(EnableConditions);

            if ((con & EnableConditions.ElementVisible) == EnableConditions.ElementVisible)
                category = EnableConditions.ElementVisible;
            else if ((con & EnableConditions.LogPanel) == EnableConditions.LogPanel)
                category = EnableConditions.LogPanel;
            else if ((con & EnableConditions.DownloadManager) == EnableConditions.DownloadManager)
                category = EnableConditions.DownloadManager;
            else if ((con & EnableConditions.FtpManager) == EnableConditions.FtpManager)
                category = EnableConditions.FtpManager;
            else if ((con & EnableConditions.HostManager) == EnableConditions.HostManager)
                category = EnableConditions.HostManager;

            bool? enableCondition = null;
            bool? checkCondition = null;

            foreach (int i in conditionValues)
            {
                if (i != 0 && (i & (Int32)con) == i)
                {
                    var res = ExecuteCondition(category | (EnableConditions)i);

                    if (res == State.Default)
                        continue;

                    bool? check = res == State.Checked ? (bool?)true : res == State.Unchecked ? (bool?)false : null;
                    bool? enable = res == State.Enabled ? (bool?)true : res == State.Disabled ? (bool?)false : null;

                    if (check != null)
                        checkCondition = true && check.Value;

                    if (enable != null)
                        enableCondition = true && enable.Value;
                }
            }

            if (checkCondition != null)
                ((ToolStripMenuItem)item).SetChecked(checkCondition.Value);

            if (enableCondition != null)
                item.SetEnable(enableCondition.Value);
        }


        private State ExecuteCondition(EnableConditions condition)
        {
            State result = default(State);

            if (!Cache.TryGetValue(condition, out result))
            {
                ConditionCallBack func = null;
                
                if (!Functors.TryGetValue(condition, out func))
                    return State.Default;

                result = func(Page);
                Cache.Add(condition, result);
            }

            return result;
        }
        #endregion


        #region Properties
        public static Dictionary<EnableConditions,ConditionCallBack> Functors { get; private set; }

        public Dictionary<EnableConditions,State> Cache { get; private set; }

        public IPage Page { get; private set; }
        #endregion 
    }
}
