﻿using System;
using System.Linq;
using System.Windows.Forms;
using WideStream.Configuration;
using WideStream.Core.ComponentModel;
using WideStream.DownloadManager;
using WideStream.DownloadManager.Configuration;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.DownloadManager.Presentation;
using WideStream.FtpManager;
using WideStream.FtpManager.Configuration;
using WideStream.FtpManager.Presentation;
using WideStream.Logging.Configuration;
using WideStream.Workspace.ComponentModel;
using WideStream.Workspace.Configuration;
using WideStream.Workspace.Forms;
using WideStream.HostManager.Presentation;
using WideStream.HostManager.Configuration;
using WideStream.Startup.Resources;
using System.Collections.Generic;

namespace WideStream.Workspace.State
{
	internal sealed class ItemStateManager
    {
        #region Construction
        private Main form;
        private DateTime lastRunTime;
        private Dictionary<EnableConditions,Boolean> conditions;

        public ItemStateManager(Main form)
        {
            this.form = form;
            conditions = new Dictionary<EnableConditions,Boolean>();
            Application.Idle += (o, e) => ProcessControls();
        }
        #endregion


        #region Methods
        private void ProcessControls()
        {
            InternalProcessControls();
        }


        private void InternalProcessControls()
        {
            WorkspaceOptions gen = Config.Query<WorkspaceOptions>();
            var p = new ConditionProcessor(form.CurrentPage);
            
            if (form.LogVisible != gen.LogVisible)
			    form.LogVisible = gen.LogVisible;

            if (form.StatusBar.Visible != gen.StatusBarVisible)
			    form.StatusBar.Visible = gen.StatusBarVisible; 

            if (form.ToolBar.Visible != gen.ToolbarVisible)
			    form.ToolBar.Visible = gen.ToolbarVisible;

            DownloadPage dm = form.ObtainPageIfLoaded(Page.DownloadManager) as DownloadPage;
            form.TrayIconToolTip = dm != null ? dm.CurrentStatus : StringTable.GetString("Tray.NoInfo");
			form.TrayIconVisible = Config.Query<WorkspaceOptions>().MinimizedState != MinimizedState.HideIcon;
            ToolStripRenderMode rmd = gen.Theme == VisualTheme.ApplicationControlled ? ToolStripRenderMode.ManagerRenderMode :
                ToolStripRenderMode.System;
            
			LogOptions conf = Config.Query<LogOptions>();
            ProcessGridColumns<LogColumns>(form.LogView, (Int32)conf.LogColumns, 
                conf.AllowResizeColumns, conf.AllowRegroupColumns);
          
            if (form.CurrentPage != null)
            {
                string status = form.CurrentPage.CurrentStatus;

                if (form.StatusText != status)
                    form.StatusText = status;

                p.ProcessItems(form.CurrentPage.MainMenu.Items);
                p.ProcessItems(form.CurrentPage.ContextMenu.Items);
                p.ProcessItems(form.ToolBar.Items);
            }

            if (form.CurrentPage != null && form.CurrentPage.PageType == Page.DownloadManager)
            {
				DownloadManagerDisplayOptions queueConf = Config.Query<DownloadManagerDisplayOptions>();
                ProcessGridColumns<QueueColumns>(form.CurrentPage.DisplayControl, (Int32)queueConf.QueueColumns,
                    queueConf.AllowResizeColumns, queueConf.AllowRegroupColumns);
            }
            else if (form.CurrentPage != null && form.CurrentPage.PageType == Page.FtpExplorer)
            {
                FtpManagerOptions expConf = Config.Query<FtpManagerOptions>();
                ProcessGridColumns<ExplorerColumns>(form.CurrentPage.DisplayControl, (Int32)expConf.ExplorerColumns,
                    expConf.AllowResizeColumns, expConf.AllowRegroupColumns);
            }
            else if (form.CurrentPage != null && form.CurrentPage.PageType == Page.HostManager)
			{
                HostManagerOptions hmConf = Config.Query<HostManagerOptions>();
                ProcessGridColumns<HostManagerColumns>(form.CurrentPage.DisplayControl, (Int32)hmConf.DisplayColumns,
                    hmConf.AllowResizeColumns, hmConf.AllowRegroupColumns);
			}
            
            p.ProcessItems(form.MainMenuStrip.Items);
            p.ProcessItems(form.LogMenu.Items);
            p.ProcessItems(form.TrayMenu.Items);

            DownloadPage page = (DownloadPage)form.ObtainPageIfLoaded(Page.DownloadManager);

            if (page != null)
            {
				AutoSaveInterval interval = Config.Query<QueueManagerOptions>().AutoSaveInterval;

                if (interval != AutoSaveInterval.Never)
                {
                    if ((Int32)interval <= (DateTime.Now - lastRunTime).TotalMilliseconds)
                    {
                        page.SaveQueue();
                        lastRunTime = DateTime.Now;
                    }
                }
            }

            if (lastRunTime == default(DateTime))
                lastRunTime = DateTime.Now;
        }


        private void ProcessGridColumns<T>(DataGridView grid, int displayed, bool allowResize,
            bool allowRegroup) where T : struct
        {
            if (grid.AllowUserToResizeColumns != allowResize)
                grid.AllowUserToResizeColumns = allowResize;

            if (grid.AllowUserToOrderColumns != allowRegroup)
                grid.AllowUserToOrderColumns = allowRegroup;

            int[] values = (int[])Enum.GetValues(typeof(T));

            foreach (DataGridViewColumn c in grid.Columns)
            {
                bool? visible = null;

                foreach (int v in values)
                {
                    if (String.Compare(c.Name, ((T)((Object)v)).ToString(), true) == 0)
                        visible = (displayed & v) == v;
                }

                if (visible != null && c.Visible != visible.Value)
                    c.Visible = visible.Value;
            }
        }



        //private void ProcessMenuItems2(ToolStripItemCollection coll)
        //{
        //    foreach (ToolStripItem i in coll)
        //    {
        //        if (i is ToolStripMenuItem && ((ToolStripMenuItem)i).DropDownItems.Count > 0)
        //            ProcessMenuItems2(((ToolStripMenuItem)i).DropDownItems);
        //        else if (i.Tag != null && i.Tag as String != String.Empty)
        //        {
        //            EnableConditions conditions = (EnableConditions)Enum.Parse(
        //                typeof(EnableConditions), i.Tag as String);
                   
        //            if ((conditions & EnableConditions.ElementVisible) == EnableConditions.ElementVisible)
        //            {
        //                EnableConditions cond = conditions ^ EnableConditions.ElementVisible;

        //                switch (cond)
        //                {
        //                    case EnableConditions.StatusBar :
        //                        ((ToolStripMenuItem)i).Checked = form.StatusBarVisible;
        //                        break;
        //                    case EnableConditions.Log:
        //                        ((ToolStripMenuItem)i).Checked = form.LogVisible;
        //                        i.Enabled = form.CurrentPage != null && form.CurrentPage.PageType != Page.HostManager;							
        //                        break;
        //                    case EnableConditions.ToolBar:
        //                        ((ToolStripMenuItem)i).Checked = form.ToolBar.Visible;
        //                        break;
        //                    case EnableConditions.DownloadPage:
        //                        ((ToolStripMenuItem)i).Checked = form.CurrentPage != null &&
        //                            form.CurrentPage.PageType == Page.DownloadManager;
        //                        break;
        //                    case EnableConditions.FtpManagerPage:
        //                        ((ToolStripMenuItem)i).Checked = form.CurrentPage != null &&
        //                            form.CurrentPage.PageType == Page.FtpExplorer;
        //                        break;
        //                    case EnableConditions.HostManagerPage:
        //                        ((ToolStripMenuItem)i).Checked = form.CurrentPage != null &&
        //                            form.CurrentPage.PageType == Page.HostManager;
        //                        break;
        //                    case EnableConditions.ActiveDownloads:
        //                        ((ToolStripMenuItem)i).Checked = ActiveDownloadsManager.WindowShown;
        //                        break;
        //                }
        //            }
        //            else if ((conditions & EnableConditions.LogPanel) == EnableConditions.LogPanel)
        //            {
        //                EnableConditions cond = conditions ^ EnableConditions.LogPanel;
        //                bool preCondition = CheckPrecondition(ref cond, () => form.SelectedLogEntries > 0,
        //                    () => form.SelectedLogEntries > 0);

        //                switch (cond)
        //                {
        //                    case EnableConditions.HasAny:
        //                        i.SetEnable(preCondition && form.LogEntries > 0);
        //                        break;
        //                    default:
        //                        i.SetEnable(preCondition);
        //                        break;
        //                }
        //            }
        //            else if ((conditions & EnableConditions.DownloadManager) == EnableConditions.DownloadManager)
        //            {
        //                if (form.CurrentPage == null || form.CurrentPage.PageType != Page.DownloadManager)
        //                    i.SetEnable(false);
        //                else
        //                {
        //                    DownloadPage page = (DownloadPage)form.CurrentPage;
        //                    EnableConditions cond = conditions ^ EnableConditions.DownloadManager;
        //                    bool preCondition = CheckPrecondition(ref cond, () => page.SelectedDownload != Guid.Empty,
        //                        () => page.SelectedDownloads.Count > 0);

        //                    switch (cond)
        //                    {
        //                        case EnableConditions.HasAnyStarted:
        //                            i.SetEnable(QueueManager.Instance.ActiveDownloads.Count() > 0);
        //                            break;
        //                        case EnableConditions.HasAnyPaused:
        //                            {
        //                                bool en = QueueManager.Instance.Queue.Values.Count(
        //                                        p => p.Status == DownloadStatus.Paused ||
        //                                        p.Status == DownloadStatus.Failed ||
        //                                        p.Status == DownloadStatus.Aborted) > 0;
        //                                i.SetEnable(en);
        //                            }
        //                            break;
        //                        case EnableConditions.IsQueueSuspended:
        //                            ((ToolStripMenuItem)i).Checked = Config.Query<QueueManagerOptions>().SuspendQueue;
        //                            break;
        //                        case EnableConditions.HasAny:
        //                            i.SetEnable(preCondition && QueueManager.Instance.Queue.Count > 0);
        //                            break;
        //                        case EnableConditions.CanPause:
        //                            {
        //                                bool enabled = false;
        //                                DownloadStatus curState;

        //                                foreach (Guid k in page.SelectedDownloads)
        //                                    if (((curState = QueueManager.Instance.QueryStatus(k)) == DownloadStatus.Started &&
        //                                        QueueManager.Instance.SupportRanges(k)) || curState == DownloadStatus.Default)
        //                                        enabled = true;
        //                                    else
        //                                    {
        //                                        enabled = false;
        //                                        break;
        //                                    }

        //                                i.SetEnable(preCondition && enabled);
        //                            }
        //                            break;
        //                        case EnableConditions.CanStop:
        //                            {
        //                                bool enabled = false;

        //                                foreach (Guid k in page.SelectedDownloads)
        //                                    if (QueueManager.Instance.QueryStatus(k) == DownloadStatus.Started)
        //                                        enabled = true;
        //                                    else
        //                                    {
        //                                        enabled = false;
        //                                        break;
        //                                    }

        //                                i.SetEnable(preCondition && enabled);
        //                            }
        //                            break;
        //                        case EnableConditions.CanResume:
        //                            {
        //                                bool enabled = false;
        //                                DownloadStatus stat = default(DownloadStatus);

        //                                foreach (Guid k in page.SelectedDownloads)
        //                                    if ((stat = QueueManager.Instance.QueryStatus(k)) == DownloadStatus.Finished ||
        //                                        stat == DownloadStatus.Failed || stat == DownloadStatus.Aborted ||
        //                                        stat == DownloadStatus.Paused)
        //                                        enabled = true;
        //                                    else
        //                                    {
        //                                        enabled = false;
        //                                        break;
        //                                    }

        //                                i.SetEnable(preCondition && enabled);
        //                            }
        //                            break;
        //                        case EnableConditions.IsFinished:
        //                            i.SetEnable(preCondition && QueueManager.Instance.QueryStatus(
        //                                page.SelectedDownload) == DownloadStatus.Finished);
        //                            break;
        //                        default:
        //                            i.SetEnable(preCondition);
        //                            break;
        //                    }
        //                }
        //            }
        //            else if ((conditions & EnableConditions.FtpManager) == EnableConditions.FtpManager)
        //            {
        //                if (form.CurrentPage == null || form.CurrentPage.PageType != Page.FtpExplorer)
        //                    i.SetEnable(false);
        //                else
        //                {
        //                    FtpManagerPage page = (FtpManagerPage)form.CurrentPage;
        //                    EnableConditions cond = conditions ^ EnableConditions.FtpManager;
        //                    bool preCondition = CheckPrecondition(ref cond, () => page.SelectedFile != null,
        //                        () => page.SelectedFiles.Count > 0);

        //                    switch (cond)
        //                    {
        //                        case EnableConditions.HasAny:
        //                            i.SetEnable(preCondition && FtpExplorer.Instance.Properties != null);
        //                            break;
        //                        case EnableConditions.HasUri:
        //                            i.SetEnable(page.GetUri() != null);
        //                            break;
        //                        case EnableConditions.HasParent:
        //                            i.SetEnable(FtpExplorer.Instance.HasParent);
        //                            break;
        //                        case EnableConditions.CanStop:
        //                            i.SetEnable(page.CurrentStatus.Length > 0);
        //                            break;
        //                        default:
        //                            i.SetEnable(preCondition);
        //                            break;
        //                    }
        //                }
        //            }
        //            else if ((conditions & EnableConditions.HostManager) == EnableConditions.HostManager)
        //            {
        //                if (form.CurrentPage == null || form.CurrentPage.PageType != Page.HostManager)
        //                    i.SetEnable(false);
        //                else
        //                {
        //                    HostManagerPage page = (HostManagerPage)form.CurrentPage;
        //                    EnableConditions cond = conditions ^ EnableConditions.HostManager;
        //                    bool preCondition = CheckPrecondition(ref cond, () => page.DisplayControl.SelectedRows.Count == 1,
        //                        () => page.DisplayControl.SelectedRows.Count > 0);

        //                    switch (cond)
        //                    {
        //                        case EnableConditions.HasAny:
        //                            i.SetEnable(preCondition && page.DisplayControl.Rows.Count > 0);
        //                            break;
        //                        default:
        //                            i.SetEnable(preCondition);
        //                            break;
        //                    }
        //                }
        //            }
        //        }
        //    }
        //}


        //private bool CheckPrecondition(ref EnableConditions cond, Func<Boolean> onlyOneFunctor,
        //    Func<Boolean> anyFunctor)
        //{
        //    bool preCondition = true;

        //    if ((cond & EnableConditions.HasCurrentOnlyOne) == EnableConditions.HasCurrentOnlyOne)
        //    {
        //        cond = cond ^ EnableConditions.HasCurrentOnlyOne;
        //        preCondition = onlyOneFunctor();
        //    }
        //    else if ((cond & EnableConditions.HasCurrentAny) == EnableConditions.HasCurrentAny)
        //    {
        //        cond = cond ^ EnableConditions.HasCurrentAny;
        //        preCondition = anyFunctor();
        //    }

        //    return preCondition;
        //}
        #endregion
    }
}
