﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WideStream.Workspace.ComponentModel;
using WideStream.DownloadManager;
using WideStream.Configuration;
using WideStream.DownloadManager.Configuration;
using WideStream.DownloadManager.Network.ObjectModel;
using WideStream.DownloadManager.Presentation;
using WideStream.FtpManager.Presentation;
using WideStream.FtpManager;
using WideStream.HostManager.Presentation;
using WideStream.Startup;

namespace WideStream.Workspace.State
{
    public delegate State ConditionCallBack(IPage page);

    [AttributeUsage(AttributeTargets.Method, AllowMultiple = false, Inherited = false)]
    public sealed class EnableConditionAttribute : Attribute
    {
        #region Construction
        public EnableConditionAttribute(EnableConditions conditions)
        {
            Conditions = conditions;
        }
        #endregion


        #region Properties
        public EnableConditions Conditions { get; private set; }
        #endregion
    }


    public enum State
    {
        Default,

        Enabled,

        Disabled,

        Checked,

        Unchecked
    }



    public static class DownloadManagerState 
    {
        [EnableCondition(EnableConditions.HasCurrentOnlyOne)]
        public static State HasCurrentOnlyOne(IPage page)
        {
            return ((DownloadPage)page).SelectedDownload != Guid.Empty ?
                State.Enabled : State.Disabled;
        }


        [EnableCondition(EnableConditions.HasCurrentAny)]
        public static State HasCurrentAny(IPage page)
        {
            return ((DownloadPage)page).SelectedDownloads.Count > 0 ?
                State.Enabled : State.Disabled;
        } 


        [EnableCondition(EnableConditions.HasAnyStarted)]
        public static State HasAnyStarted(IPage page)
        {
            return QueueManager.Instance.ActiveDownloads.Count() > 0 ?
                State.Enabled : State.Disabled;
        }


        [EnableCondition(EnableConditions.HasAnyPaused)]
        public static State HasAnyPaused(IPage page)
        {
            return QueueManager.Instance.ActiveDownloads.Count() > 0 ?
                State.Enabled : State.Disabled;
        }


        [EnableCondition(EnableConditions.IsQueueSuspended)]
        public static State IsQueueSuspended(IPage page)
        {
            return Config.Query<QueueManagerOptions>().SuspendQueue ?
                State.Checked : State.Unchecked;
        }


        [EnableCondition(EnableConditions.CanPause)]
        public static State CanPause(IPage page)
        {
            bool enabled = false;
            DownloadStatus curState;

            foreach (Guid k in ((DownloadPage)page).SelectedDownloads)
                if (((curState = QueueManager.Instance.QueryStatus(k)) == DownloadStatus.Started &&
                    QueueManager.Instance.SupportRanges(k)) || curState == DownloadStatus.Default)
                    enabled = true;
                else
                {
                    enabled = false;
                    break;
                }

            return enabled ? State.Enabled : State.Disabled;
        }


        [EnableCondition(EnableConditions.CanStop)]
        public static State CanStop(IPage page)
        {
            bool enabled = false;

            foreach (Guid k in ((DownloadPage)page).SelectedDownloads)
                if (QueueManager.Instance.QueryStatus(k) == DownloadStatus.Started)
                    enabled = true;
                else
                {
                    enabled = false;
                    break;
                }

            return enabled ? State.Enabled : State.Disabled;
        }


        [EnableCondition(EnableConditions.CanResume)]
        public static State CanResume(IPage page)
        {
            bool enabled = false;
            DownloadStatus stat = default(DownloadStatus);

            foreach (Guid k in ((DownloadPage)page).SelectedDownloads)
                if ((stat = QueueManager.Instance.QueryStatus(k)) == DownloadStatus.Finished ||
                    stat == DownloadStatus.Failed || stat == DownloadStatus.Aborted ||
                    stat == DownloadStatus.Paused)
                    enabled = true;
                else
                {
                    enabled = false;
                    break;
                }

            return enabled ? State.Enabled : State.Disabled;
        }


        [EnableCondition(EnableConditions.IsFinished)]
        public static State IsFinished(IPage page)
        {
            var key = ((DownloadPage)page).SelectedDownload;

            return key != Guid.Empty && QueueManager.Instance.QueryStatus(key) == DownloadStatus.Finished ? 
                State.Enabled : State.Disabled;
        }
    }


    public static class FtpManagerState
    {
        [EnableCondition(EnableConditions.HasCurrentOnlyOne)]
        public static State HasCurrentOnlyOne(IPage page)
        {
            return ((FtpManagerPage)page).SelectedFile != null ?
                State.Enabled : State.Disabled;
        }


        [EnableCondition(EnableConditions.HasCurrentAny)]
        public static State HasCurrentAny(IPage page)
        {
            return ((FtpManagerPage)page).SelectedFiles.Count > 0 ?
                State.Enabled : State.Disabled;
        }


        [EnableCondition(EnableConditions.HasAny)]
        public static State HasAny(IPage page)
        {
            return FtpExplorer.Instance.Properties != null ?
                State.Enabled : State.Disabled;
        }


        [EnableCondition(EnableConditions.HasUri)]
        public static State HasUri(IPage page)
        {
            return ((FtpManagerPage)page).GetUri() != null ?
                State.Enabled : State.Disabled;
        }


        [EnableCondition(EnableConditions.HasParent)]
        public static State HasParent(IPage page)
        {
            return FtpExplorer.Instance.HasParent ?
                State.Enabled : State.Disabled;
        }


        [EnableCondition(EnableConditions.CanStop)]
        public static State CanStop(IPage page)
        {
            return ((FtpManagerPage)page).CurrentStatus.Length > 0 ?
                State.Enabled : State.Disabled;
        }
    }


    public static class HostManagerState
    {
        [EnableCondition(EnableConditions.HasCurrentOnlyOne)]
        public static State HasCurrentOnlyOne(IPage page)
        {
            return page.DisplayControl.SelectedRows.Count == 1 ?
                State.Enabled : State.Disabled;
        }


        [EnableCondition(EnableConditions.HasCurrentAny)]
        public static State HasCurrentAny(IPage page)
        {
            return page.DisplayControl.SelectedRows.Count > 0 ?
                State.Enabled : State.Disabled;
        }


        [EnableCondition(EnableConditions.HasAny)]
        public static State HasAny(IPage page)
        {
            return page.DisplayControl.Rows.Count > 0 ?
                State.Enabled : State.Disabled;
        }
    }


    public static class VisibleElementsState
    {
        [EnableCondition(EnableConditions.StatusBar)]
        public static State StatusBar(IPage page)
        {
            return MainForm != null && MainForm.StatusBarVisible ?
                State.Checked : State.Unchecked;
        }


        [EnableCondition(EnableConditions.ToolBar)]
        public static State ToolBar(IPage page)
        {
            return MainForm != null && MainForm.ToolBar.Visible ?
                State.Checked : State.Unchecked;
        }


        [EnableCondition(EnableConditions.DownloadPage)]
        public static State DownloadPage(IPage page)
        {
            return CurrentPage != null && CurrentPage.PageType == Page.DownloadManager ?
                State.Checked : State.Unchecked;
        }


        [EnableCondition(EnableConditions.FtpManagerPage)]
        public static State FtpManagerPage(IPage page)
        {
            return CurrentPage != null && CurrentPage.PageType == Page.FtpExplorer ?
                State.Checked : State.Unchecked;
        }


        [EnableCondition(EnableConditions.HostManagerPage)]
        public static State HostManagerPage(IPage page)
        {
            return CurrentPage != null && CurrentPage.PageType == Page.HostManager ?
                State.Checked : State.Unchecked;
        }


        [EnableCondition(EnableConditions.ActiveDownloads)]
        public static State ActiveDownloads(IPage page)
        {
            return ActiveDownloadsManager.WindowShown ? 
                State.Checked : State.Unchecked;
        }


        [EnableCondition(EnableConditions.Log)]
        public static State Log(IPage page)
        {
            var res1 = MainForm != null && MainForm.LogVisible ? 
                State.Checked : State.Unchecked;
            var res2 = CurrentPage != null && CurrentPage.PageType != Page.HostManager ?
                State.Enabled : State.Default;
            return res1 | res2;
        }


        private static IPage CurrentPage
        {
            get { return MainForm != null ? MainForm.CurrentPage : null; }
        }


        private static Main MainForm
        {
            get
            {
                return Program.Environment != null ? Program.Environment.MainForm : null;
            }
        }
    }


    public static class LogState
    {
        [EnableCondition(EnableConditions.HasCurrentOnlyOne)]
        public static State HasCurrentOnlyOne(IPage page)
        {
            return MainForm != null && MainForm.SelectedLogEntries > 0 ?
                State.Enabled : State.Disabled;
        }


        [EnableCondition(EnableConditions.HasCurrentAny)]
        public static State HasCurrentAny(IPage page)
        {
            return HasCurrentOnlyOne(page);
        }


        [EnableCondition(EnableConditions.HasAny)]
        public static State HasAny(IPage page)
        {
            return MainForm != null && MainForm.LogEntries > 0 ?
                State.Enabled : State.Disabled;
        }

        private static IPage CurrentPage
        {
            get { return MainForm != null ? MainForm.CurrentPage : null; }
        }


        private static Main MainForm
        {
            get
            {
                return Program.Environment != null ? Program.Environment.MainForm : null;
            }
        }
    }
}
